@@ -43,23 +43,11 @@ our %build_and_test_sets = (
 	# NOTE: Keep the stable or developer release branches synchronized with
 	# https://condor-wiki.cs.wisc.edu/index.cgi/wiki?p=DeveloperReleasePlan
 	'official_ports' => [
-		'x86_64_deb_6.0-updated', # this will switch to non-updated when NMI has that platform
-		'x86_64_deb_5.0',
-		'x86_64_rhap_5',
-		'x86_64_rhas_3',
-		'x86_deb_5.0',
-		'x86_rhap_5',
-		'x86_rhas_3',
-		'x86_winnt_5.1',
-		'x86_64_rhap_6.1-updated',
-		'x86_64_macos_10.6-updated',
 	],
 	
 	# NMI will need builds on a set of platforms that we do not provide in our
 	# core builds.	These are those platforms.
 	'nmi_one_offs' => [
-		#'x86_64_freebsd_8.2-updated',
-		#'x86_64_sol_5.11',
 	],
 	
 	# We will build on a set of machines that we want to be sure continue building
@@ -68,9 +56,6 @@ our %build_and_test_sets = (
 	# release - a build problem on this platform could indicate problems on a future
 	# release of RHEL.
 	'extra_builds' => [
-		#'x86_64_fedora_14-updated',
-		#'x86_64_opensuse_11.4-updated',
-		#'x86_64_macos_10.6-updated',
 	],
 	
 	'stduniv' => [
@@ -85,25 +70,7 @@ our %build_and_test_sets = (
 # For every build, test, and cross test, of condor everywhere,
 # these are the default prereqs _usually_ involved.
 ###############################################################################
-my @default_prereqs = (
-	'tar-1.14',
-	'patch-2.5.4',
-	'cmake-2.8.3',
-	'flex-2.5.4a',
-	'make-3.80',
-	'byacc-1.9',
-	'bison-1.25',
-	'wget-1.9.1',
-	'm4-1.4.1',
-	);
-
-# Hackery to test running in new batlab
-my $isNewBatlab = 0;
-if ((`hostname -f` eq "submit-1.batlab.org\n") ||
-	(`hostname -f` eq "submit-2.batlab.org\n")) {
-@default_prereqs = ();
-$isNewBatlab = 1;
-} 
+my @default_prereqs = ();
 
 ###############################################################################
 # Minimal build configuration
@@ -319,38 +286,18 @@ our %submit_info = (
 			'configure_args' => { @default_build_configure_args,
 								  '-DCLIPPED:BOOL' => 'OFF',
 			},
-			'prereqs'	=> ($isNewBatlab ? [] : [ 'libtool-1.5.26', 'cmake-2.8.3' ]),
-			'xtests'	=> ($isNewBatlab ? [] : [ 'x86_64_ubuntu_10.04', ]),
+			'prereqs'	=> [],
+			'xtests'	=> [],
 		},
 
 		'test' => {
 			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> ($isNewBatlab ? [] : [ 'java-1.4.2_05' ]),
+			'prereqs'	=> [],
 			'testclass' => [ @default_testclass ],
 		},
 	},
 
 
-	##########################################################################
-	# Platform Debian 6.0 on x86_64 (updated)
-	##########################################################################
-	# This is the name of the platform in old batlab
-	'x86_64_deb_6.0-updated' => {
-		'build' => {
-			'configure_args' => { @default_build_configure_args,
-				'-DCLIPPED:BOOL=OFF' => undef,
-			 },
-			'prereqs'	=> [ 'libtool-1.5.26', 'cmake-2.8.3' ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ 'java-1.4.2_05' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
 	##########################################################################
 	# Platform DEB 6 on x86_64 
 	##########################################################################
@@ -387,7 +334,7 @@ our %submit_info = (
 
 		'test' => {
 			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
+			'prereqs'	=> [ @default_prereqs ],
 			'testclass'	=> [ @default_testclass ],
 		},
 	},
@@ -396,6 +343,8 @@ our %submit_info = (
 	# Platform RHEL 6 on x86. Unmanaged.
 	##########################################################################
 	'x86_rhap_6.0-updated'	=> 'x86_rhap_6.0',
+	'x86_rhap_6.2'	=> 'x86_rhap_6.0',
+	'x86_rhap_6.3'	=> 'x86_rhap_6.0',
 
 	##########################################################################
 	# Platform RHEL 6 on x86_64
@@ -411,7 +360,7 @@ our %submit_info = (
 
 		'test' => {
 			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
+			'prereqs'	=> [ @default_prereqs ],
 			'testclass'	=> [ @default_testclass ],
 		},
 	},
@@ -440,6 +389,7 @@ our %submit_info = (
 
 	# And now, 6.1 has become 6.2 through the magic of auto-update
     'x86_64_rhap_6.2' => 'x86_64_rhap_6.1',
+    'x86_64_rhap_6.3' => 'x86_64_rhap_6.1',
 
 	##########################################################################
 	# Platform RHEL 5 on x86_64
@@ -457,13 +407,13 @@ our %submit_info = (
 
 		'test' => {
 			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
+			'prereqs'	=> [ @default_prereqs ],
 			'testclass' => [ @default_testclass ],
 		},
 	},
 
 	# This is the new batlab one
-	'x86_64_rhap_5.7'		=> {
+	'x86_64_rhap_5.8'		=> {
 		'build' => {
 			'configure_args' => { @default_build_configure_args,
 								  '-DCLIPPED:BOOL' => 'OFF',
@@ -480,27 +430,6 @@ our %submit_info = (
 	},
 
 
-	##########################################################################
-	# Platform RHEL 3 on x86_64
-	##########################################################################
-	'x86_64_rhas_3'		=> {
-		'build' => {
-			'configure_args' => { @default_build_configure_args,
-								  '-DCLIPPED:BOOL' => 'OFF',
-								  '-DWITH_LIBCGROUP:BOOL' => 'OFF',
-			},
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> [ ],
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05', 'perl-5.8.5',
-							 'VMware-server-1.0.7' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
 	##########################################################################
 	# Platform Debian 5 on x86
 	##########################################################################
@@ -509,95 +438,17 @@ our %submit_info = (
 			'configure_args' => { @default_build_configure_args,
 								  '-DCLIPPED:BOOL' => 'OFF',
 			},
-									  'prereqs' => [ 'libtool-1.5.26', 'cmake-2.8.3' ],
-									  'xtests'	=> [ ],
+			'prereqs' => [ ],
+			'xtests'	=> [ ],
 		},
 
 		'test' => {
 			'configure_args' => { @default_test_configure_args },
-			'prereqs'		=> [ 'java-1.4.2_05' ],
+			'prereqs'		=> [ ],
 			'testclass'	=> [ @default_testclass ],
 		},
 	},
 
-	##########################################################################
-	# Platform Mac OSX 10.4 on x86
-	##########################################################################
-	'x86_macos_10.4'	=> {
-		'build' => {
-			'configure_args' => { @default_build_configure_args },
-			'prereqs'	=> [ @default_prereqs,
-							 'coreutils-5.2.1',
-							 'libtool-1.5.26',],
-			'xtests'	=> [
-				'x86_64_macos_10.6',
-				'x86_64_macos_10.6-updated',
-				],
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ 
-				@default_prereqs, 
-				'java-1.4.2_12', 
-				'coreutils-5.2.1'
-				],
-				'testclass'		=> [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform Mac OSX 10.5 on x86_64
-	# condor actually builds naturally for this one, we just don't release it
-	##########################################################################
-	'x86_64_macos_10.5' => {
-		'build' => {
-			'configure_args' => { @default_build_configure_args },
-			'prereqs'	=> [
-				@default_prereqs,
-				'libtool-1.5.26',
-				],
-				'xtests'		=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform Mac OSX 10.5 on x86_64 with updates
-	# condor actually builds naturally for this one, we just don't release it
-	##########################################################################
-	'x86_64_macos_10.5-updated' => 'x86_64_macos_10.5',
-
-
-	##########################################################################
-	# Platform Mac OSX 10.6 on x86_64
-	##########################################################################
-	'x86_64_macos_10.6' => {
-		'build' => {
-			'configure_args' => {  @default_build_configure_args },
-			'prereqs'	=> [
-				@default_prereqs,
-				'libtool-1.5.26',
-				],
-				'xtests'		=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform Mac OSX 10.6 with updates on x86_64
-	##########################################################################
-	'x86_64_macos_10.6-updated' => 'x86_64_macos_10.6',
 
 	# This is new batlab macos 10.7 machine
 	'x86_64_macos_10.7' => {
@@ -616,7 +467,7 @@ our %submit_info = (
 	##########################################################################
 	# Platform RHEL 5 on x86
 	##########################################################################
-	'x86_rhap_5'		=> {
+	'x86_rhap_5.8'		=> {
 		'build' => {
 			'configure_args' => { @default_build_configure_args,
 								  '-DCLIPPED:BOOL' => 'OFF',
@@ -627,116 +478,7 @@ our %submit_info = (
 
 		'test' => {
 			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform RHEL 3 on x86
-	##########################################################################
-	'x86_rhas_3'		=> {
-		'build' => {
-			'configure_args' => { @default_build_configure_args,
-								  '-DCLIPPED:BOOL' => 'OFF',
-								  '-DWITH_LIBCGROUP:BOOL' => 'OFF',
-			},
-			'prereqs'	=> [ 
-				@default_prereqs,
-				'perl-5.8.5', 'gzip-1.3.3', 'autoconf-2.59'
-			],
-			'xtests'	=> [ 
-			 	'x86_64_rhas_3',
-			],
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.5.0_08', 'perl-5.8.5',
-							 'VMware-server-1.0.7' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-
-	# These describe what a human, sadly, had to figure out about certain
-	# ports that we do in a "one off" fashion. These ports are generally
-	# not released to the public, but are often needed by NMI to run their
-	# cluster. If by chance work happens on these ports to make them officially
-	# released, then they will move from this section to the above section, 
-	# and most likely with the above arguments to configure. Most of these
-	# builds of Condor are as clipped as possible to ensure compilation.
-
-	##########################################################################
-	# Platform Fedora 13 on x86_64
-	##########################################################################
-	'x86_64_fedora_13'	=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args },
 			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.5.0_08' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform Fedora 13 with updates on x86_64
-	##########################################################################
-	'x86_64_fedora_13-updated'	=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.5.0_08' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform Fedora 14 on x86_64
-	##########################################################################
-	'x86_64_fedora_14'	=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args,
-			},
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	'x86_64_fedora_15'				=> 'x86_64_fedora_14',
-	'x86_64_fedora_16'				=> 'x86_64_fedora_14',
-	'x86_64_fedora_17'				=> 'x86_64_fedora_14',
-	'x86_64_fedora_18'				=> 'x86_64_fedora_14',
-	##########################################################################
-	# Platform Fedora 14 with updates on x86_64
-	##########################################################################
-	'x86_64_fedora_14-updated'	=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args,
-			},
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.5.0_08' ],
 			'testclass' => [ @default_testclass ],
 		},
 	},
@@ -799,81 +541,6 @@ our %submit_info = (
 		},
 	},
 
-	##########################################################################
-	# Platform RHEL 5 on x86  (umanaged!)
-	# This might work.
-	##########################################################################
-	'unmanaged-x86_rhap_5'		=> {
-		'build' => {
-			'configure_args' => { @default_build_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-		
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.5.0_08' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform RHEL 5.2 on X86_64
-	# This might work.
-	# I suspect this could be a real port if we bothered.
-	##########################################################################
-	'x86_64_rhap_5.2'	=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform RHEL 5.3 on X86_64
-	# This might work.
-	# I suspect this could be a real port if we bothered.
-	##########################################################################
-	'x86_64_rhap_5.3'	=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform RHEL 5.3 with updates on X86_64
-	# This might work.
-	# I suspect this could be a real port if we bothered.
-	##########################################################################
-	'x86_64_rhap_5.3-updated'	=> {
-		'build' => {
-			'configure_args' => { @default_build_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
 	##########################################################################
 	# Platform RHEL 5.4 on X86_64
 	# This might work.
@@ -929,46 +596,6 @@ our %submit_info = (
 
 
 
-	##########################################################################
-	# Platform RHEL 4 on X86_64
-	# This might work.
-	##########################################################################
-	'x86_64_rhas_4'		=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args,
-								  '-DWITH_LIBCGROUP:BOOL' => 'OFF',
-			 },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.5.0_08', 'perl-5.8.9' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
-	##########################################################################
-	# Platform SLES 9 on x86_64
-	##########################################################################
-	'x86_64_sles_9'				=> {
-		'build' => {
-			'configure_args' =>{ @minimal_build_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'wget-1.9.1' ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => {
-				@default_test_configure_args,
-				
-			},
-			'prereqs'	=> [ @default_prereqs, 'wget-1.9.1', 'java-1.4.2_05' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
-
 	##########################################################################
 	# Platform Ubuntu 10.04 on x86_64
 	# This might work.
@@ -1006,22 +633,6 @@ our %submit_info = (
 		},
 	},
 
-	##########################################################################
-	# Platform RHEL 4 on x86
-	##########################################################################
-	'x86_rhas_4'		=> {
-		'build' => {
-			'configure_args' => { @minimal_build_configure_args },
-			'prereqs'	=> [ @default_prereqs ],
-			'xtests'	=> undef,
-		},
-
-		'test' => {
-			'configure_args' => { @default_test_configure_args },
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05', 'perl-5.8.5' ],
-			'testclass' => [ @default_testclass ],
-		},
-	},
 
 	##########################################################################
 	# Platform openSUSE 11.3 on x86_64 (& updated)
@@ -1060,7 +671,7 @@ our %submit_info = (
 								  '-DWITH_LIBVIRT:BOOL' => 'ON',
 								  '-DWITH_LIBXML2:BOOL' => 'ON',
 			},
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
+			'prereqs'	=> [ @default_prereqs ],
 			'xtests'	=> undef,
 		},
 
@@ -1069,7 +680,7 @@ our %submit_info = (
 				@default_test_configure_args
 				
 			},
-			'prereqs'	=> [ @default_prereqs, 'java-1.4.2_05' ],
+			'prereqs'	=> [ @default_prereqs ],
 			'testclass'	=> [ @default_testclass ],
 		},
 	},
