@@ -145,6 +145,10 @@ CCBListener::SendMsgToCCB(ClassAd &msg,bool blocking)
 		}
 		else if( !m_waiting_for_connect ) {
 			m_sock = ccb.makeConnectedSocket(Stream::reli_sock, CCB_TIMEOUT, 0, NULL, true /*nonblocking*/ );
+			if( !m_sock ) {
+				Disconnected();
+				return false;
+			}
 			m_waiting_for_connect = true;
 			incRefCount(); // do not let ourselves be deleted until called back
 			ccb.startCommand_nonblocking( cmd, m_sock, CCB_TIMEOUT, NULL, CCBListener::CCBConnectCallback, this, NULL, false, USE_TMP_SEC_SESSION );
@@ -343,6 +347,7 @@ CCBListener::ReadMsgFromCCB()
 	if( !m_sock ) {
 		return false;
 	}
+	m_sock->timeout(CCB_TIMEOUT);
 	ClassAd msg;
 	if( !msg.initFromStream( *m_sock ) || !m_sock->end_of_message() ) {
 		dprintf(D_ALWAYS,
