@@ -82,6 +82,20 @@ CCBServer::CCBServer():
 CCBServer::~CCBServer()
 {
 	CloseReconnectFile();
+	if( m_registered_handlers ) {
+		daemonCore->Cancel_Command(CCB_REGISTER);
+		daemonCore->Cancel_Command(CCB_REQUEST);
+		m_registered_handlers = false;
+	}
+	if( m_polling_timer != -1 ) {
+		daemonCore->Cancel_Timer( m_polling_timer );
+		m_polling_timer = -1;
+	}
+	CCBTarget *target=NULL;
+	m_targets.startIterations();
+	while( m_targets.iterate(target) ) {
+		RemoveTarget(target);
+	}
 }
 
 void
@@ -92,22 +106,22 @@ CCBServer::RegisterHandlers()
 	}
 	m_registered_handlers = true;
 
-	int rc = daemonCore->Register_Command(
+	int rc = daemonCore->Register_CommandWithPayload(
 		CCB_REGISTER,
 		"CCB_REGISTER",
 		(CommandHandlercpp)&CCBServer::HandleRegistration,
 		"CCBServer::HandleRegistration",
 		this,
-		DAEMON );
+		DAEMON);
 	ASSERT( rc >= 0 );
 
-	rc = daemonCore->Register_Command(
+	rc = daemonCore->Register_CommandWithPayload(
 		CCB_REQUEST,
 		"CCB_REQUEST",
 		(CommandHandlercpp)&CCBServer::HandleRequest,
 		"CCBServer::HandleRequest",
 		this,
-		READ );
+		READ);
 	ASSERT( rc >= 0 );
 
 }
@@ -226,6 +240,11 @@ CCBServer::HandleRegistration(int cmd,Stream *stream)
 	ReliSock *sock = (ReliSock *)stream;
 	ASSERT( cmd == CCB_REGISTER );
 
+		// Avoid lengthy blocking on communication with our peer.
+		// This command-handler should not get called until data
+		// is ready to read.
+	sock->timeout(1);
+
 	ClassAd msg;
 	sock->decode();
 	if( !msg.initFromStream( *sock ) || !sock->end_of_message() ) {
@@ -309,6 +328,11 @@ CCBServer::HandleRequest(int cmd,Stream *stream)
 	ReliSock *sock = (ReliSock *)stream;
 	ASSERT( cmd == CCB_REQUEST );
 
+		// Avoid lengthy blocking on communication with our peer.
+		// This command-handler should not get called until data
+		// is ready to read.
+	sock->timeout(1);
+
 	ClassAd msg;
 	sock->decode();
 	if( !msg.initFromStream( *sock ) || !sock->end_of_message() ) {
