@@ -26,7 +26,7 @@
 #include "list.h"
 #include "simplelist.h"
 #include "extArray.h"
-#include "condor_classad_util.h"
+#include "condor_classad.h"
 
 #include <iostream>
 #include <sstream>
@@ -248,7 +248,6 @@ bool ClassAdAnalyzer::
 AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
 {
 	ResourceGroup     rg;
-    classad::ClassAd  *converted_classad;
     classad::ClassAd  *explicit_classad;
     bool              success;
 
@@ -259,13 +258,7 @@ AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
 		return true;
 	}
 
-    converted_classad = toNewClassAd( request );
-	if( converted_classad == NULL ) {
-		buffer += "Unable to process job ClassAd";
-		buffer += "\n";
-		return true;
-	}
-    explicit_classad  = AddExplicitTargets( converted_classad );
+	explicit_classad  = AddExplicitTargets( request );
 
     ensure_result_initialized(explicit_classad);
     
@@ -273,9 +266,7 @@ AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
     offers.Rewind();
     ClassAd *ad;
     while((ad = offers.Next())) {
-      classad::ClassAd *new_ad = toNewClassAd(ad);
-      result_add_machine(*new_ad);
-      delete new_ad;
+      result_add_machine(*ad);
 
       if (do_basic_analysis) {
 	BasicAnalyze(request, ad);
@@ -284,7 +275,6 @@ AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
 
 	success = AnalyzeJobReqToBuffer( explicit_classad, rg, buffer );
 
-    delete converted_classad;
     delete explicit_classad;
     return success;
 }
@@ -313,7 +303,6 @@ AnalyzeJobAttrsToBuffer( ClassAd *request, ClassAdList &offers,
 						 string &buffer )
 {
 	ResourceGroup     rg;
-    classad::ClassAd  *converted_classad;
     classad::ClassAd  *explicit_classad;
     bool              success;
 
@@ -324,17 +313,10 @@ AnalyzeJobAttrsToBuffer( ClassAd *request, ClassAdList &offers,
 		return true;
 	}
 
-    converted_classad = toNewClassAd( request );
-	if( converted_classad == NULL ) {
-		buffer += "Unable to process job ClassAd";
-		buffer += "\n";
-		return true;
-	}
-    explicit_classad  = AddExplicitTargets( converted_classad );
+	explicit_classad  = AddExplicitTargets( request );
     ensure_result_initialized(explicit_classad);
 	success = AnalyzeJobAttrsToBuffer( explicit_classad, rg, buffer );
 
-    delete converted_classad;
     delete explicit_classad;
     return success;
 }
@@ -537,6 +519,7 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 			value_s = "";
 			condition->ToString( cond_s );
 			strncpy( cond, cond_s.c_str( ), 1023 );
+			cond[1023] = 0;
 
 			sprintf( info, "%i", condition->explain.numberOfMatches );
 
@@ -558,7 +541,7 @@ AnalyzeJobReqToBuffer( classad::ClassAd *request, ResourceGroup &offers, string
 			}
 			}
 
-   			if( strlen( cond ) < 46 ) {
+			if( strlen( cond ) < 46 ) {
 				sprintf( formatted, "%-4i%-34s%-20s%s\n", i, cond, info,
 						 suggest );
 			} else {
@@ -935,17 +918,10 @@ MakeResourceGroup( ClassAdList &caList, ResourceGroup &rg )
 	caList.Rewind( );
 	ad = caList.Next( );
 	while( ad ) {
-        classad::ClassAd *converted_classad;
         classad::ClassAd *explicit_classad;
 
-        converted_classad = toNewClassAd(ad);
-		if( converted_classad == NULL ) {
-				// could not convert to new ClassAd (parse error)
-			return false;
-		}
-        explicit_classad  = AddExplicitTargets(converted_classad);
+		explicit_classad  = AddExplicitTargets(ad);
 		newList.Append(explicit_classad);
-        delete converted_classad;
 		ad = caList.Next( );
 	}
 	if( !rg.Init( newList ) ) {
@@ -1198,7 +1174,8 @@ SuggestConditionModify( Profile *p, ResourceGroup &rg )
 	int vrNum = 0;
 	ExtArray<classad::Operation::OpKind> ops ( numConds );
 	ExtArray<Condition*> conds ( numConds );
-    ExtArray<bool> tooComplex ( numConds ); 
+//	ExtArray<bool> tooComplex ( numConds ); 
+	std::vector<bool> tooComplex( numConds, false);
 //	classad::Operation::OpKind op1, op2;
 	classad::Value val;
 	p->Rewind( );
@@ -1468,6 +1445,7 @@ FindConflicts( Profile *p, ResourceGroup &rg )
 	bvList.Rewind( );
 	while( bvList.Next( currBV ) ) {
 		if( currBV == NULL ) {
+			delete currIS;
 			return false;
 		}
 		currIS = new IndexSet( );
@@ -1481,6 +1459,9 @@ FindConflicts( Profile *p, ResourceGroup &rg )
 		currIS->GetCardinality( card );
 		if( card > 1 ) {
 			p->explain.conflicts->Append( currIS );
+		} else {
+			delete currIS;
+			currIS = NULL;
 		}
 	}
 
@@ -1962,7 +1943,6 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 	currentABV = NULL;
 	hrs = NULL;
 	currHR = NULL;
-	AnnotatedBoolVector *bestABV = NULL;
 	HyperRect *bestHR = NULL;
 	IndexSet hasContext;
 	IndexSet hasMachine;
@@ -1980,7 +1960,6 @@ AnalyzeAttributes( classad::ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caEx
 			hasMachine.GetCardinality( currNumContexts );
 			if( currNumContexts > maxNumContexts ) {
 				maxNumContexts = currNumContexts;
-				bestABV = currentABV;
 				bestHR = currHR;
 			}
 		}
@@ -2115,7 +2094,7 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 		return false;
 	}
 
-	classad::Operation::OpKind op;
+	classad::Operation::OpKind op = (classad::Operation::OpKind)0;
 	classad::Value val;
 	bool undef = false;
 	bool twoVals = false;
@@ -2385,6 +2364,7 @@ AddConstraint( ValueRange *&vr, Condition *condition )
 					else {
 						vr->Init( i, undef );
 					}
+					delete i;
 					return true;
 				}
 			case classad::Operation::EQUAL_OP:
