@@ -67,7 +67,9 @@ ExprToMultiProfile( classad::ExprTree *expr, MultiProfile *&mp )
 
 		kind = currentTree->GetKind( );
 			
-		if( kind == classad::ExprTree::ATTRREF_NODE ) {
+		if( kind == classad::ExprTree::ATTRREF_NODE ||
+			kind == classad::ExprTree::FN_CALL_NODE )
+		{
 			atLeftMostProfile = true;
 			continue;
 		}
@@ -75,6 +77,7 @@ ExprToMultiProfile( classad::ExprTree *expr, MultiProfile *&mp )
 		if( kind != classad::ExprTree::OP_NODE ) {
 			cerr << "error: bad form" << endl;
 				// error: bad form
+			delete currentProfile;
 			return false;
 		}
 
@@ -104,6 +107,7 @@ ExprToMultiProfile( classad::ExprTree *expr, MultiProfile *&mp )
 		if( !ExprToProfile( right, currentProfile ) ) {
 			cerr << "error: problem with ExprToProfile" << endl;
 				// error: problem with ExprToProfile
+			delete currentProfile;
 			return false;
 		}
 		else {
@@ -172,7 +176,9 @@ ExprToProfile( classad::ExprTree *expr, Profile *&p )
 
 		kind = currentTree->GetKind( );
 
-		if( kind == classad::ExprTree::ATTRREF_NODE ) {
+		if( kind == classad::ExprTree::ATTRREF_NODE ||
+			kind == classad::ExprTree::FN_CALL_NODE )
+		{
 			atLeftMostCondition = true;
 			continue;
 		}
@@ -180,6 +186,7 @@ ExprToProfile( classad::ExprTree *expr, Profile *&p )
 		if( kind != classad::ExprTree::OP_NODE ) {
 			cerr << "error: bad form" << endl;
 				// error: bad form
+			delete currentCondition;
 			return false;
 		}
 
@@ -269,6 +276,14 @@ ExprToCondition( classad::ExprTree *expr, Condition *&c )
 		return true;
 	}
 
+	if( kind == classad::ExprTree::FN_CALL_NODE ) {
+		if( !c->InitComplex( expr->Copy( ) ) ) {
+  				cerr << "error: problem with Condition::InitComplex" << endl;
+  				return false;
+		}
+		return true;
+	}
+
 	if( kind != classad::ExprTree::OP_NODE ) {
 		cerr << "error: no operator/attribute found" << endl;
 			// error: no operator/attribute  found;
