@@ -25,7 +25,7 @@
 #include "classad/sink.h"
 
 using namespace std;
-BEGIN_NAMESPACE(classad)
+namespace classad {
 
 //const int  KeySet::size  = 32;
 const int  KeySet::SUINT = sizeof(unsigned int)*8;//in bits
@@ -1184,4 +1184,4 @@ Augment( Rectangles &rec1, Rectangles &rec2, const ClassAd *ad,
 	return( true );
 }
 
-END_NAMESPACE
+} // namespace classad
