@@ -36,13 +36,13 @@ static unsigned int hashFunction (const StatsHashKey &key)
 	const char *p;
 
     for (p = key.type.Value(); p && *p;
-	     result = (result<<5) + result + (unsigned int)(*(p++)));
+	     result = (result<<5) + result + (unsigned int)(*(p++))) { }
 
     for (p = key.name.Value(); p && *p;
-	     result = (result<<5) + result + (unsigned int)(*(p++)));
+	     result = (result<<5) + result + (unsigned int)(*(p++))) { }
 
     for (p = key.ip_addr.Value(); p && *p;
-	     result = (result<<5) + result + (unsigned int)(*(p++)));
+	     result = (result<<5) + result + (unsigned int)(*(p++))) { }
 
     return result;
 }
@@ -662,7 +662,9 @@ CollectorDaemonStatsList::hashKey (StatsHashKey &key,
 	// get the IP and port of the daemon
 	if ( ad->LookupString (ATTR_MY_ADDRESS, buf, sizeof(buf) ) ) {
 		MyString	myString( buf );
-		parseIpPort( myString, key.ip_addr );
+		char* host = getHostFromAddr(myString.Value());
+		key.ip_addr = host;
+		free(host);
 	} else {
 		return false;
 	}
