@@ -41,7 +41,7 @@ int ViewServer::TimeStamp;
 int ViewServer::HistoryTimer;
 MyString ViewServer::DataFormat[DataSetCount];
 AccHash* ViewServer::GroupHash;
-int ViewServer::KeepHistory;
+bool ViewServer::KeepHistory;
 HashTable< MyString, int >* ViewServer::FileHash;
 ExtArray< ExtIntArray* >* ViewServer::TimesArray;
 ExtArray< ExtOffArray* >* ViewServer::OffsetsArray;
@@ -64,16 +64,12 @@ void ViewServer::Init()
 
 	// Check operation mode
 
-	KeepHistory=FALSE;
-	char* tmp=param("KEEP_POOL_HISTORY");
-	if( tmp ) {
-		if( *tmp == 'T' || *tmp == 't' ) KeepHistory=TRUE;
-		free( tmp );
-	}
+	KeepHistory = param_boolean_crufty("KEEP_POOL_HISTORY", false);
 
 	// We can't do this check at compile time, but we'll except if
 	// the startd states has changed and we haven't been updated
 	// to match
+	MSC_SUPPRESS_WARNING(6326) // warning comparing a constant to a constant.
 	if ( (int)VIEW_STATE_MAX != (int)_machine_max_state ) {
 		EXCEPT( "_max_state=%d (from condor_state.h) doesn't match "
 				" VIEW_STATE_MAX=%d (from view_server.h)",
@@ -134,7 +130,7 @@ void ViewServer::Init()
 	DataFormat[SubmittorData]="%d\t%s\t:\t%.0f\t%.0f\n";
 	DataFormat[SubmittorGroupsData]="%d\t%s\t:\t%.0f\t%.0f\n";
 	DataFormat[StartdData]="%d\t%s\t:\t%.0f\t%7.3f\t%.0f\n";
-	DataFormat[GroupsData]="%d\t%s\t:\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\n";
+	DataFormat[GroupsData]="%d\t%s\t:\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\t%.1f\n";
 	DataFormat[CkptData]="%d\t%s\t:\t%.3f\t%.3f\t%.3f\t%.3f\n";
 	
 	return;
@@ -223,7 +219,7 @@ void ViewServer::Shutdown()
 // HandleQuery to take care of replying to the query
 //-------------------------------------------------------------------
 
-int ViewServer::ReceiveHistoryQuery(Service* s, int command, Stream* sock)
+int ViewServer::ReceiveHistoryQuery(Service* /*s*/, int command, Stream* sock)
 {
 	dprintf(D_ALWAYS,"Got history query %d\n",command);
 
@@ -383,7 +379,7 @@ int ViewServer::SendListReply(Stream* sock,const MyString& FileName, int FromDat
 	}
 
 	// dprintf(D_ALWAYS,"Filename=%s\n",(const char*)FileName);
-	FILE* fp=safe_fopen_wrapper(FileName.Value(),"r");
+	FILE* fp=safe_fopen_wrapper_follow(FileName.Value(),"r");
 	if (!fp) return -1;
 
 		//find the offset to search at
@@ -472,7 +468,7 @@ int ViewServer::SendDataReply(Stream* sock,const MyString& FileName, int FromDat
 	}
 
 	OldTime = 0;
-	FILE* fp=safe_fopen_wrapper(FileName.Value(),"r");
+	FILE* fp=safe_fopen_wrapper_follow(FileName.Value(),"r");
 	if (!fp) return -1;
 
 		//find the offset to search at
@@ -566,7 +562,7 @@ ViewServer::addNewOffset(FILE* &fp, int &offset_ctr, int read_time, ExtIntArray*
 //-------------------------------------------------------------------
 
 fpos_t*
-ViewServer::findOffset(FILE* &fp, int FromDate, int ToDate, ExtIntArray* times_array, ExtOffArray* offsets) {
+ViewServer::findOffset(FILE* & /*fp*/, int FromDate, int ToDate, ExtIntArray* times_array, ExtOffArray* offsets) {
 	fpos_t* search_offset_ptr = NULL;
 	if( times_array->length() == 0 ) {
 
@@ -636,10 +632,14 @@ int ViewServer::FindFileStartTime(const char *Name)
 {
 	char Line[200];
 	int T=-1;
-	FILE* fp=safe_fopen_wrapper(Name,"r");
+	FILE* fp=safe_fopen_wrapper_follow(Name,"r");
 	if (fp) {
-		fgets(Line,sizeof(Line),fp);
-		T=ReadTime(Line);
+		if (fgets(Line,sizeof(Line),fp)) {
+			T=ReadTime(Line);
+		} else {
+			T=-1; // fgets failed, return -1 instead of parsing uninit memory
+			dprintf(D_ALWAYS, "Failed to parse first line of %s\n", Name);
+		}
 		fclose(fp);
 	}
 	dprintf(D_ALWAYS,"FileName=%s , StartTime=%d\n",Name,T);
@@ -733,7 +733,7 @@ void ViewServer::WriteHistory()
 			if (DataSet[i][j].NumSamples<DataSet[i][j].MaxSamples) continue;
 			DataSet[i][j].NumSamples=0;
 			dprintf(D_FULLDEBUG,"Openning file %s\n",DataSet[i][j].NewFileName.Value());
-			DataFile=safe_fopen_wrapper(DataSet[i][j].NewFileName.Value(),"a");
+			DataFile=safe_fopen_wrapper_follow(DataSet[i][j].NewFileName.Value(),"a");
 			if (!DataFile) {
 				dprintf(D_ALWAYS,"Could not open data file %s for appending!!! errno=%d\n",DataSet[i][j].NewFileName.Value(),errno);
 				EXCEPT("Could not open data file appending!!!");
@@ -743,7 +743,7 @@ void ViewServer::WriteHistory()
 
 			DataSet[i][j].AccData->startIterations();
 			while(DataSet[i][j].AccData->iterate(Key,GenRec)) {
-				sprintf(OutLine,DataFormat[i].Value(),TimeStamp,Key.Value(),GenRec->Data[0],GenRec->Data[1],GenRec->Data[2],GenRec->Data[3],GenRec->Data[4],GenRec->Data[5],GenRec->Data[6], GenRec->Data[7]);
+				sprintf(OutLine,DataFormat[i].Value(),TimeStamp,Key.Value(),GenRec->Data[0],GenRec->Data[1],GenRec->Data[2],GenRec->Data[3],GenRec->Data[4],GenRec->Data[5],GenRec->Data[6], GenRec->Data[7], GenRec->Data[8]);
 				delete GenRec;
 				fputs(OutLine, DataFile);
 			}
@@ -928,6 +928,9 @@ int ViewServer::StartdScanFunc(ClassAd* cad)
 	case backfill_state:
 		st=VIEW_STATE_BACKFILL;
 		break;
+	case drained_state:
+		st=VIEW_STATE_DRAINED;
+		break;
 	default:
 		dprintf( D_ALWAYS,
 				 "WARNING: Unknown machine state %d from '%s' (ignoring)",
