@@ -69,6 +69,7 @@ public:
 			Set ATTR_HOLD_REASON to the given reason.
 			@param constraint What jobs to act on
 			@param reason Why the action is being done
+			@param reason_code The hold subcode
 			@param result_type What kind of results you want
 			@param notify_scheduler Should the schedd notify the
  			controlling scheduler for this job?
@@ -77,6 +78,7 @@ public:
 			this ClassAd when they are done with the results.
 		*/
 	ClassAd* holdJobs( const char* constraint, const char* reason,
+					   const char* reason_code,
 					   CondorError * errstack,
 					   action_result_type_t result_type = AR_TOTALS,
 					   bool notify_scheduler = true );
@@ -144,6 +146,7 @@ public:
 			this ClassAd when they are done with the results.
 		*/
 	ClassAd* holdJobs( StringList* ids, const char* reason,
+					   const char* reason_code,
 					   CondorError * errstack,
 					   action_result_type_t result_type = AR_LONG,
 					   bool notify_scheduler = true );
@@ -236,7 +239,72 @@ public:
 						 action_result_type_t result_type = AR_TOTALS,
 						 bool notify_scheduler = true );
 
-		/** Clear dirty attributes for a list of job ids
+	
+	/** Suspend all jobs specified in the given StringList.  The list
+			should contain a comma-seperated list of cluster.proc job
+			ids.
+			@param ids What jobs to act on
+			@param vacate_type Graceful or fast vacate?
+			@param result_type What kind of results you want
+			@param notify_scheduler Should the schedd notify the
+ 			controlling scheduler for this job?
+			@return ClassAd containing results of this action, or NULL
+			if we couldn't get any results.  The caller must delete
+			this ClassAd when they are done with the results.
+		*/
+	ClassAd* suspendJobs( StringList* ids, const char* reason,
+						 CondorError * errstack,
+						 action_result_type_t result_type = AR_LONG,
+						 bool notify_scheduler = true );
+	
+	/** Suspend all jobs that match the given constraint.
+			@param constraint What jobs to act on
+			@param reason Why the action is being done
+			@param result_type What kind of results you want
+			@param notify_scheduler Should the schedd notify the
+ 			controlling scheduler for this job?
+			@return ClassAd containing results of this action, or NULL
+			if we couldn't get any results.  The caller must delete
+			this ClassAd when they are done with the results.
+		*/
+	ClassAd* suspendJobs( const char* constraint, const char* reason,
+						  CondorError * errstack,
+						  action_result_type_t result_type = AR_TOTALS,
+						  bool notify_scheduler = true );
+	
+	/** Continue all jobs specified in the given StringList.  The list
+			should contain a comma-seperated list of cluster.proc job
+			ids.
+			@param ids What jobs to act on
+			@param vacate_type Graceful or fast vacate?
+			@param result_type What kind of results you want
+			@param notify_scheduler Should the schedd notify the
+ 			controlling scheduler for this job?
+			@return ClassAd containing results of this action, or NULL
+			if we couldn't get any results.  The caller must delete
+			this ClassAd when they are done with the results.
+		*/
+	ClassAd* continueJobs( StringList* ids, const char* reason,
+						 CondorError * errstack,
+						 action_result_type_t result_type = AR_LONG,
+						 bool notify_scheduler = true );
+	
+		/** Continue all jobs that match the given constraint.
+			@param constraint What jobs to act on
+			@param reason Why the action is being done
+			@param result_type What kind of results you want
+			@param notify_scheduler Should the schedd notify the
+ 			controlling scheduler for this job?
+			@return ClassAd containing results of this action, or NULL
+			if we couldn't get any results.  The caller must delete
+			this ClassAd when they are done with the results.
+		*/
+	ClassAd* continueJobs( const char* constraint, const char* reason,
+						  CondorError * errstack,
+						  action_result_type_t result_type = AR_TOTALS,
+						  bool notify_scheduler = true );
+	
+	/** Clear dirty attributes for a list of job ids
 			@param ids What jobs to act on
 			@param result_type What kind of results you want
 		*/
@@ -330,6 +398,8 @@ private:
 			@param ids StringList of ids to operate on, or NULL
 			@param reason A string describing what we're doing
 			@param reason_attr_name Attribute name for the reason
+			@param reason_code A string such as an error code
+			@param reason_code_attr_name Attribute name for the reason_code
 			@param result_type What kind of results you want
 			@param notify_scheduler Should the schedd notify the
  			controlling scheduler for this job?
@@ -340,6 +410,7 @@ private:
 	ClassAd* actOnJobs( JobAction action, 
 						const char* constraint, StringList* ids, 
 						const char* reason, const char* reason_attr,
+						const char* reason_code, const char* reason_code_attr,
 						action_result_type_t result_type,
 						bool notify_scheduler, CondorError * errstack );
 
