@@ -29,6 +29,7 @@
 #include "condor_open.h"
 
 #include "mimetypes.h"
+#include "condor_sockfunc.h"
 
 #ifdef WIN32
 #define realpath(P,B) _fullpath((B),(P),_MAX_PATH)
@@ -51,7 +52,7 @@ int get_handler(struct soap *soap);
 struct soap *
 dc_soap_accept(Sock *socket, const struct soap *soap)
 {
-	struct soap *cursoap = soap_copy(soap);
+	struct soap *cursoap = soap_copy(const_cast<struct soap*>(soap));
 	ASSERT(cursoap);
 
 		// Mimic a gsoap soap_accept as follows:
@@ -60,7 +61,7 @@ dc_soap_accept(Sock *socket, const struct soap *soap)
 		//   3. increase size of send and receive buffers
 		//   4. set SO_KEEPALIVE [done automatically by CEDAR accept()]
 	cursoap->socket = socket->get_file_desc();
-	cursoap->peer = *socket->peer_addr();
+	cursoap->peer = socket->peer_addr().to_sin();
 	cursoap->recvfd = soap->socket;
 	cursoap->sendfd = soap->socket;
 	if ( cursoap->recv_timeout > 0 ) {
@@ -224,21 +225,18 @@ dc_soap_init(struct soap *&soap)
 						(details && *details) ? *details : "No details.");
 			}
 
-			struct sockaddr_in sockaddr;
-			SOCKET_LENGTH_TYPE namelen = sizeof(struct sockaddr_in);
-			if (getsockname(sock_fd,
-							(struct sockaddr *) &sockaddr,
-							&namelen)) {
+			condor_sockaddr sockaddr;
+			if (condor_getsockname(sock_fd, sockaddr)) {
 				dprintf(D_ALWAYS, "Failed to get name of SOAP SSL socket\n");
-			} else if (sizeof(struct sockaddr_in) == namelen) {
-				dprintf(D_ALWAYS,
-						"Setup SOAP SSL on port %d\n",
-						ntohs(sockaddr.sin_port));
 			} else {
 				dprintf(D_ALWAYS,
-						"Failed to get name of SOAP SSL socket, "
-						"unknown sockaddr returned\n");
-			}
+						"Setup SOAP SSL on port %d\n",
+						sockaddr.get_port());
+			} //else {
+				//	dprintf(D_ALWAYS,
+				//		"Failed to get name of SOAP SSL socket, "
+				//		"unknown sockaddr returned\n");
+				//}
 
 			ReliSock *sock = new ReliSock;
 			if (!sock) {
@@ -327,6 +325,8 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 	sockaddr.sin_addr.s_addr = htonl(ssl_soap->ip);
 	sockaddr.sin_port = htons(ssl_soap->port);
 
+	condor_sockaddr addr(&sockaddr);
+
 	current_soap = soap_copy(ssl_soap);
 	ASSERT(current_soap);
 
@@ -342,7 +342,7 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 		const char **details = soap_faultdetail(current_soap);
 		dprintf(D_ALWAYS,
 				"SOAP SSL connection attempt from %s failed: %s\n",
-				sin_to_string(&sockaddr),
+				addr.to_sinful().Value(),
 				(details && *details) ? *details : "No details.");
 
 		soap_done(current_soap);
@@ -353,14 +353,14 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 
 	dprintf(D_ALWAYS,
 			"SOAP SSL connection attempt from %s succeeded\n",
-			sin_to_string(&sockaddr));
+			addr.to_sinful().Value());
 
-	if (X509_V_OK != SSL_get_verify_result((const SSL*)current_soap->ssl)) {
+	if (X509_V_OK != SSL_get_verify_result((SSL*)current_soap->ssl)) {
 		dprintf(D_ALWAYS,
 				"SOAP SSL connection attempt from %s failed "
 				"because the client's certificate was not "
 				"verified.\n",
-				sin_to_string(&sockaddr));
+				addr.to_sinful().Value());
 
 		soap_done(current_soap);
 		free(current_soap);
@@ -370,12 +370,12 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 
 	X509 *peer_cert;
 	X509_NAME *peer_subject;
-	peer_cert = SSL_get_peer_certificate((const SSL*)current_soap->ssl);
+	peer_cert = SSL_get_peer_certificate((SSL*)current_soap->ssl);
 	if (NULL == peer_cert) {
 		dprintf(D_ALWAYS,
 				"SOAP SSL connection attempt from %s failed "
 				"because the client did not send a certificate.\n",
-				sin_to_string(&sockaddr));
+				addr.to_sinful().Value());
 
 		soap_done(current_soap);
 		free(current_soap);
@@ -388,7 +388,7 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 				"SOAP SSL connection attempt from %s failed "
 				"because the client's certificate's subject was not "
 				"found.\n",
-				sin_to_string(&sockaddr));
+				addr.to_sinful().Value());
 
 		soap_done(current_soap);
 		free(current_soap);
@@ -407,7 +407,7 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 				"SOAP SSL connection attempt from %s failed "
 				"because the client's certificate's subject was not "
 				"oneline-able.\n",
-				sin_to_string(&sockaddr));
+				addr.to_sinful().Value());
 
 		soap_done(current_soap);
 		free(current_soap);
@@ -420,7 +420,7 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 	}
 	dprintf(D_FULLDEBUG,
 			"SOAP SSL connection from %s, X509 subject: %s\n",
-			sin_to_string(&sockaddr),
+			addr.to_sinful().Value(),
 			subject);
 
 	MyString canonical_user;
@@ -445,7 +445,7 @@ handle_soap_ssl_socket(Service *, Stream *stream)
 			"SOAP SSL connection subject mapped to '%s'\n",
 			canonical_user.Value());
 
-	if (!daemonCore->Verify("SOAP SSL",SOAP_PERM, &sockaddr, canonical_user.Value())) {
+	if (!daemonCore->Verify("SOAP SSL",SOAP_PERM, addr, canonical_user.Value())) {
 
 		soap_done(current_soap);
 		free(current_soap);
@@ -518,6 +518,10 @@ int serve_file(struct soap *soap, const char *name, const char *type) {
     web_root_realpath = strdup(buf);
   }
   free(web_root_dir);
+
+  if (!web_root_realpath) {
+    return 404;
+  } 
   
   char * full_name = dircat(web_root_realpath,name);
   char * full_name_realpath = NULL;
@@ -555,7 +559,7 @@ int serve_file(struct soap *soap, const char *name, const char *type) {
     return 403;
   }
 
-  fstr = safe_fopen_wrapper(full_name_realpath, "rb");
+  fstr = safe_fopen_wrapper_follow(full_name_realpath, "rb");
 
   free(full_name_realpath);
   free(web_root_realpath);
