@@ -41,6 +41,9 @@ const int MAX_FIRES_PER_TIMEOUT = 3;
 extern void **curr_dataptr;
 extern void **curr_regdataptr;
 
+// disable warning about memory leaks due to exception. all memory freed on exit anyway
+MSC_DISABLE_WARNING(6211)
+
 
 TimerManager::TimerManager()
 {
@@ -108,7 +111,11 @@ int TimerManager::NewTimer(Service* s, unsigned deltawhen,
 		return -1;
 	}
 
-	new_timer->handler = handler;
+    if (daemonCore) {
+       daemonCore->dc_stats.New("Timer", event_descrip, AS_COUNT | IS_RCT | IF_NONZERO | IF_VERBOSEPUB);
+    }
+
+    new_timer->handler = handler;
 	new_timer->handlercpp = handlercpp;
 	new_timer->release = release;
 	new_timer->releasecpp = releasecpp;
@@ -322,12 +329,14 @@ void TimerManager::CancelAllTimers()
 // called while a handler is active (i.e. handler calls Timeout;
 // Timeout is not re-entrant).
 int
-TimerManager::Timeout()
+TimerManager::Timeout(int * pNumFired /*= NULL*/, double * pruntime /*=NULL*/)
 {
 	int				result, timer_check_cntr;
 	time_t			now, time_sample;
 	int				num_fires = 0;	// num of handlers called in this timeout
 
+    if (pNumFired) *pNumFired = 0;
+
 	if ( in_timeout != NULL ) {
 		dprintf(D_DAEMONCORE,"DaemonCore Timeout() called and in_timeout is non-NULL\n");
 		if ( timer_list == NULL ) {
@@ -438,7 +447,11 @@ TimerManager::Timeout()
 			}
 		}
 
-		// Make sure we didn't leak our priv state
+		if (pruntime) {           
+			*pruntime = daemonCore->dc_stats.AddRuntime(in_timeout->event_descrip, *pruntime);
+		}
+
+        // Make sure we didn't leak our priv state
 		daemonCore->CheckPrivState();
 
 		// Clear curr_dataptr
@@ -494,6 +507,7 @@ TimerManager::Timeout()
 	}
 
 	dprintf( D_DAEMONCORE, "DaemonCore Timeout() Complete, returning %d \n",result);
+    if (pNumFired) *pNumFired = num_fires;
 	in_timeout = NULL;
 	return(result);
 }
@@ -562,7 +576,6 @@ void TimerManager::DumpTimerList(int flag, const char* indent)
 void TimerManager::Start()
 {
 	struct timeval		timer;
-	int					rv;
 
 	for(;;)
 	{
@@ -579,12 +592,12 @@ void TimerManager::Start()
 			// no timer events registered...  only a signal
 			// can save us now!!
 			dprintf(D_DAEMONCORE,"TimerManager::Start() about to block with no events!\n");
-			rv = select(0,0,0,0,NULL);
+			select(0,0,0,0,NULL);
 		} else {
 			dprintf(D_DAEMONCORE,
 				"TimerManager::Start() about to block, timeout=%ld\n",
 				(long)timer.tv_sec);
-			rv = select(0,0,0,0, &timer);
+			select(0,0,0,0, &timer);
 		}		
 	}
 }
