@@ -45,7 +45,7 @@ const char* dagman_exe = "condor_dagman";
 const char* valgrind_exe = "valgrind";
 #endif
 
-int printUsage(); // NOTE: printUsage calls exit(1), so it doesn't return
+int printUsage(int iExitCode=1); // NOTE: printUsage calls exit(1), so it doesn't return
 void parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			SubmitDagShallowOptions &shallowOpts, int argc,
 			const char * const argv[]);
@@ -70,12 +70,14 @@ int submitDag( SubmitDagShallowOptions &shallowOpts );
 //---------------------------------------------------------------------------
 int main(int argc, char *argv[])
 {
+	param_functions *p_funcs = NULL;
 	printf("\n");
 
 		// Set up the dprintf stuff to write to stderr, so that Condor
 		// libraries which use it will write to the right place...
 	Termlog = true;
-	dprintf_config("condor_submit_dag"); 
+	p_funcs = get_param_functions();
+	dprintf_config("condor_submit_dag", p_funcs); 
 	DebugFlags = D_ALWAYS | D_NOHEADER;
 	config();
 
@@ -413,7 +415,7 @@ submitDag( SubmitDagShallowOptions &shallowOpts )
 //---------------------------------------------------------------------------
 bool fileExists(const MyString &strFile)
 {
-	int fd = safe_open_wrapper(strFile.Value(), O_RDONLY);
+	int fd = safe_open_wrapper_follow(strFile.Value(), O_RDONLY);
 	if (fd == -1)
 		return false;
 	close(fd);
@@ -440,18 +442,17 @@ void ensureOutputFilesExist(const SubmitDagDeepOptions &deepOpts,
 		}
 	}
 
+		// Get rid of the halt file (if one exists).
+	unlink( HaltFileName( shallowOpts.primaryDagFile ).Value() );
+
 	if (deepOpts.bForce)
 	{
 		unlink(shallowOpts.strSubFile.Value());
 		unlink(shallowOpts.strSchedLog.Value());
 		unlink(shallowOpts.strLibOut.Value());
 		unlink(shallowOpts.strLibErr.Value());
-		if (deepOpts.oldRescue) {
-			unlink(shallowOpts.strRescueFile.Value());
-		} else {
-			RenameRescueDagsAfter(shallowOpts.primaryDagFile.Value(),
-						shallowOpts.dagFiles.number() > 1, 0, maxRescueDagNum);
-		}
+		RenameRescueDagsAfter(shallowOpts.primaryDagFile.Value(),
+					shallowOpts.dagFiles.number() > 1, 0, maxRescueDagNum);
 	}
 
 		// Check whether we're automatically running a rescue DAG -- if
@@ -628,7 +629,7 @@ EnvFilter::ImportFilter( const MyString &var, const MyString &val ) const
 void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 			/* const */ SubmitDagShallowOptions &shallowOpts)
 {
-	FILE *pSubFile = safe_fopen_wrapper(shallowOpts.strSubFile.Value(), "w");
+	FILE *pSubFile = safe_fopen_wrapper_follow(shallowOpts.strSubFile.Value(), "w");
 	if (!pSubFile)
 	{
 		fprintf( stderr, "ERROR: unable to create submit file %s\n",
@@ -727,10 +728,6 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg(dagFile);
 	}
 
-	if(deepOpts.oldRescue) {
-		args.AppendArg("-Rescue");
-		args.AppendArg(shallowOpts.strRescueFile.Value());
-	}
     if(shallowOpts.iMaxIdle != 0) 
 	{
 		args.AppendArg("-MaxIdle");
@@ -757,6 +754,10 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		printf( "Warning: -NoEventChecks is ignored; please use "
 					"the DAGMAN_ALLOW_EVENTS config parameter instead\n");
 	}
+	if(!shallowOpts.bPostRun)
+	{
+		args.AppendArg("-DontAlwaysRunPost");
+	}
 	if(deepOpts.bAllowLogError)
 	{
 		args.AppendArg("-AllowLogError");
@@ -806,6 +807,11 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 		args.AppendArg("-Import_env");
 	}
 
+	if( deepOpts.priority != 0 ) {
+		args.AppendArg("-Priority");
+		args.AppendArg(deepOpts.priority);
+	}
+
 	MyString arg_str,args_error;
 	if(!args.GetArgsStringV1WackedOrV2Quoted(&arg_str,&args_error)) {
 		fprintf(stderr,"Failed to insert arguments: %s",args_error.Value());
@@ -857,8 +863,8 @@ void writeSubmitFile(/* const */ SubmitDagDeepOptions &deepOpts,
 
 		// Append user-specified stuff to submit file...
 		// ...first, the insert file, if any...
-	if (shallowOpts.appendFile.Value() != "") {
-		FILE *aFile = safe_fopen_wrapper(shallowOpts.appendFile.Value(), "r");
+	if (shallowOpts.appendFile != "") {
+		FILE *aFile = safe_fopen_wrapper_follow(shallowOpts.appendFile.Value(), "r");
 		if (!aFile)
 		{
 			fprintf( stderr, "ERROR: unable to read submit append file (%s)\n",
@@ -927,10 +933,9 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 				printf( "%s\n%s\n", CondorVersion(), CondorPlatform() );
 				exit( 0 );
 			}
-			else if (strArg.find("-help") != -1) // -help
+			else if (strArg.find("-help") != -1 || strArg.find("-h") != -1) // -help
 			{
-				printUsage();
-				exit( 0 );
+				printUsage(0);
 			}
 				// submit and stick to a specific schedd
 			else if (strArg.find("-schedd-daemon-ad-file") != -1)
@@ -1044,14 +1049,6 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 				}
 				shallowOpts.appendFile = argv[iArg];
 			}
-			else if (strArg.find("-oldr") != -1) // -oldrescue
-			{
-				if (iArg + 1 >= argc) {
-					fprintf(stderr, "-oldrescue argument needs a value\n");
-					printUsage();
-				}
-				deepOpts.oldRescue = (atoi(argv[++iArg]) != 0);
-			}
 			else if (strArg.find("-autor") != -1) // -autorescue
 			{
 				if (iArg + 1 >= argc) {
@@ -1102,11 +1099,23 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 			{
 				deepOpts.bVerbose = true;
 			}
+			else if ( (strArg.find("-dontalwaysrun") != -1) ) // DontAlwaysRunPost
+			{
+				shallowOpts.bPostRun = false;
+			}
 			else if ( parsePreservedArgs( strArg, iArg, argc, argv,
 						shallowOpts) )
 			{
 				// No-op here
 			}
+			else if( (strArg.find("-prio") != -1) ) // -priority
+			{
+				if(iArg + 1 >= argc) {
+					fprintf(stderr, "-priority argument needs a value\n");
+					printUsage();
+				}
+				deepOpts.priority = atoi(argv[++iArg]);
+			}
 			else
 			{
 				fprintf( stderr, "ERROR: unknown option %s\n", strArg.Value() );
@@ -1121,13 +1130,6 @@ parseCommandLine(SubmitDagDeepOptions &deepOpts,
 		printUsage();
 	}
 
-	if (deepOpts.oldRescue && deepOpts.autoRescue)
-	{
-		fprintf( stderr, "Error: DAGMAN_OLD_RESCUE and DAGMAN_AUTO_RESCUE "
-					"are both true.\n" );
-	    exit( 1 );
-	}
-
 	if (deepOpts.doRescueFrom < 0)
 	{
 		fprintf( stderr, "-dorescuefrom value must be non-negative; aborting.\n");
@@ -1196,7 +1198,7 @@ parsePreservedArgs(const MyString &strArg, int &argNum, int argc,
 }
 
 //---------------------------------------------------------------------------
-int printUsage() 
+int printUsage(int iExitCode) 
 {
     printf("Usage: condor_submit_dag [options] dag_file [dag_file_2 ... dag_file_n]\n");
     printf("  where dag_file1, etc., is the name of a DAG input file\n");
@@ -1228,8 +1230,6 @@ int printUsage()
     printf("    -config <filename>  (Specify a DAGMan configuration file)\n");
 	printf("    -append <command>   (Append specified command to .condor.sub file)\n");
 	printf("    -insert_sub_file <filename>   (Insert specified file into .condor.sub file)\n");
-	printf("    -OldRescue 0|1      (whether to write rescue DAG the old way;\n");
-	printf("         0 = false, 1 = true)\n");
 	printf("    -AutoRescue 0|1     (whether to automatically run newest rescue DAG;\n");
 	printf("         0 = false, 1 = true)\n");
 	printf("    -DoRescueFrom <number>  (run rescue DAG of given number)\n");
@@ -1241,5 +1241,6 @@ int printUsage()
 	printf("    -import_env         (explicitly import env into submit file)\n");
 	printf("    -DumpRescue         (DAGMan dumps rescue DAG and exits)\n");
 	printf("    -valgrind           (create submit file to run valgrind on DAGMan)\n");
-	exit(1);
+	printf("    -priority <priority> (jobs will run with this priority by default)\n");
+	exit(iExitCode);
 }
