@@ -196,7 +196,7 @@ submit_try( ArgList &args, CondorID &condorID, Job::job_type_t type,
 	debug_printf( DEBUG_NORMAL, "Submit generated %d job procs; "
 				"disallowed by DAGMAN_PROHIBIT_MULTI_JOBS setting\n",
 				jobProcCount );
-	main_shutdown_rescue( EXIT_ERROR );
+	main_shutdown_rescue( EXIT_ERROR, Dag::DAG_STATUS_ERROR );
   }
   
   return true;
@@ -230,7 +230,8 @@ bool
 condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 			   const char* DAGNodeName, MyString DAGParentNodeNames,
 			   List<MyString>* names, List<MyString>* vals,
-			   const char* directory, const char *logFile )
+			   const char* directory, const char *logFile,
+			   bool prohibitMultiJobs, bool hold_claim )
 {
 	TmpDir		tmpDir;
 	MyString	errMsg;
@@ -241,6 +242,32 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 		return false;
 	}
 
+	if ( prohibitMultiJobs ) {
+		MyString	errorMsg;
+		int queueCount = MultiLogFiles::getQueueCountFromSubmitFile(
+					MyString( cmdFile ), MyString( directory ),
+					errorMsg );
+		if ( queueCount == -1 ) {
+			debug_printf( DEBUG_QUIET, "ERROR in "
+						"MultiLogFiles::getQueueCountFromSubmitFile(): %s\n",
+						errorMsg.Value() );
+				// We don't just return false here because that would
+				// cause us to re-try the submit, which would obviously
+				// just fail again.
+			main_shutdown_rescue( EXIT_ERROR, Dag::DAG_STATUS_ERROR );
+			return false; // We won't actually get to here.
+		} else if ( queueCount != 1 ) {
+			debug_printf( DEBUG_QUIET, "ERROR: node %s job queues %d "
+						"job procs, but DAGMAN_PROHIBIT_MULTI_JOBS is "
+						"set\n", DAGNodeName, queueCount );
+				// We don't just return false here because that would
+				// cause us to re-try the submit, which would obviously
+				// just fail again.
+			main_shutdown_rescue( EXIT_ERROR, Dag::DAG_STATUS_ERROR );
+			return false; // We won't actually get to here.
+		}
+	}
+
 	ArgList args;
 
 	// construct arguments to condor_submit to add attributes to the
@@ -304,6 +331,20 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 		args.AppendArg( var.Value() );
 	}
 
+		// Set the special DAG_STATUS variable (mainly for use by
+		// "final" nodes).
+	args.AppendArg( "-a" );
+	MyString var = "DAG_STATUS = ";
+	var += dm.dag->_dagStatus;
+	args.AppendArg( var.Value() );
+
+		// Set the special FAILED_COUNT variable (mainly for use by
+		// "final" nodes).
+	args.AppendArg( "-a" );
+	var = "FAILED_COUNT = ";
+	var += dm.dag->NumNodesFailed();
+	args.AppendArg( var.Value() );
+
 		// how big is the command line so far
 	MyString display;
 	args.GetArgsStringForDisplay( &display );
@@ -321,13 +362,21 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 
 		// if we don't have room for DAGParentNodeNames, leave it unset
 	if( cmdLineSize + reserveNeeded + DAGParentNodeNamesLen > maxCmdLine ) {
-		debug_printf( DEBUG_NORMAL, "WARNING: node %s has too many parents "
+		debug_printf( DEBUG_NORMAL, "Warning: node %s has too many parents "
 					  "to list in its classad; leaving its DAGParentNodeNames "
 					  "attribute undefined\n", DAGNodeName );
+		check_warning_strictness( DAG_STRICT_3 );
 	} else {
 		args.AppendArgsFromArgList( parentNameArgs );
 	}
 
+	if( hold_claim ){
+		args.AppendArg( "-a" );
+		MyString holdit = MyString("+") + MyString(ATTR_JOB_KEEP_CLAIM_IDLE) + " = "
+			+ dm._claim_hold_time;
+		args.AppendArg( holdit.Value() );	
+	}
+
 	args.AppendArg( cmdFile );
 
 	bool success = do_submit( args, condorID, Job::TYPE_CONDOR,
@@ -427,7 +476,7 @@ fake_condor_submit( CondorID& condorID, const char* DAGNodeName,
 
 		// We need some value for submitHost for the event to be read
 		// correctly.
-	sprintf( subEvent.submitHost, "<dummy-submit-for-noop-job>");
+	subEvent.setSubmitHost( "<dummy-submit-for-noop-job>" );
 
 	MyString subEventNotes("DAG Node: " );
 	subEventNotes += DAGNodeName;
