@@ -131,7 +131,9 @@ class Job {
         If you update this enum, you *must* also update status_t_names
 		and the IsActive() method, etc.
     */
+	// WARNING!  status_t and status_t_names must be kept in sync!!
     enum status_t {
+		/** Job is not ready (for final) */ STATUS_NOT_READY,
         /** Job is ready for submission */ STATUS_READY,
         /** Job waiting for PRE script */  STATUS_PRERUN,
         /** Job has been submitted */      STATUS_SUBMITTED,
@@ -143,6 +145,7 @@ class Job {
     /** The string names for the status_t enumeration.  Use this the same
         way you would use the queue_t_names array.
     */
+	// WARNING!  status_t and status_t_names must be kept in sync!!
     static const char * status_t_names[];
 
 	// explanation text for errors
@@ -156,12 +159,9 @@ class Job {
 		@param directory Directory to run the node in, "" if current
 		       directory.  String is deep copied.
         @param cmdFile Path to condor cmd file.  String is deep copied.
-		@param prohibitMultiJobs whether submit files queueing multiple
-			   job procs are prohibited.
     */
     Job( const job_type_t jobType, const char* jobName,
-				const char* directory, const char* cmdFile,
-				bool prohibitMultiJobs );
+				const char* directory, const char* cmdFile ); 
   
     ~Job();
 
@@ -178,14 +178,17 @@ class Job {
 	job_type_t JobType() const;
 
 	bool AddPreScript( const char *cmd, MyString &whynot );
+	bool AddPreSkip( int exitCode, MyString &whynot );
 	bool AddPostScript( const char *cmd, MyString &whynot );
 	bool AddScript( bool post, const char *cmd, MyString &whynot );
 
+	void SetFinal(bool value) { _final = value; }
+	bool GetFinal() const { return _final; }
 	void SetNoop( bool value ) { _noop = value; }
-	bool GetNoop( void ) { return _noop; }
+	bool GetNoop( void ) const { return _noop; }
 
-    Script * _scriptPre;
-    Script * _scriptPost;
+	Script * _scriptPre;
+	Script * _scriptPost;
 
     ///
     inline set<JobID_t> & GetQueueRef (const queue_t queue) {
@@ -411,9 +414,12 @@ class Job {
 		@return the last event time.
 	*/
 	time_t GetLastEventTime() { return _lastEventTime; }
+	void FixPriority(Dag& dag);
 
+	bool HasPreSkip() const { return _preskip != PRE_SKIP_INVALID; }
+	int GetPreSkip() const;
+	
     /** */ CondorID _CondorID;
-    /** */ status_t _Status;
 
     // maximum number of times to retry this node
     int retry_max;
@@ -487,12 +493,6 @@ class Job {
 	int _jobProcsOnHold;
 
 private:
-
-		// Note: Init moved to private section because calling int more than
-		// once will cause a memory leak.  wenger 2005-06-24.
-	void Init( const char* jobName, const char *directory,
-				const char* cmdFile, bool prohibitMultiJobs );
-  
 		// Mark this node as failed because of an error in monitoring
 		// the log file.
   	void LogMonitorFailed();
@@ -516,6 +516,8 @@ private:
   
     // name given to the job by the user
     char* _jobName;
+
+    /** */ status_t _Status;
   
     /*  Job queues
 	    NOTE: indexed by queue_t
@@ -587,6 +589,21 @@ private:
 
 		// The time of the most recent event related to this job.
 	time_t _lastEventTime;
+
+		// Skip the rest of the node (and consider it successful) if the
+		// PRE script exits with this value.  (-1 means undefined.)
+	int _preskip;
+	int _pre_status;
+
+	enum {
+		PRE_SKIP_INVALID = -1,
+		PRE_SKIP_MIN = 0,
+		PRE_SKIP_MAX = 0xff,
+		NO_PRE_VALUE = -1
+	};
+
+	// whether this is a final job
+	bool _final;
 };
 
 /** A wrapper function for Job::Print which allows a NULL job pointer.
