@@ -41,6 +41,7 @@
 #include "extArray.h"
 #include "condor_string.h"  /* for strnewp() */
 #include "dagman_recursive_submit.h"
+#include "condor_getcwd.h"
 
 static const char   COMMENT    = '#';
 static const char * DELIMITERS = " \t";
@@ -85,6 +86,9 @@ static bool parse_reject(Dag  *dag, const char *filename,
 		int  lineNumber);
 static bool parse_jobstate_log(Dag  *dag, const char *filename,
 		int  lineNumber);
+static bool parse_pre_skip(Dag *dag, const char* filename,
+		int lineNumber);
+static bool parse_done(Dag  *dag, const char *filename, int  lineNumber);
 static MyString munge_job_name(const char *jobName);
 
 static MyString current_splice_scope(void);
@@ -159,11 +163,13 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 		tmpFilename = condor_basename( filename );
 	}
 
-	FILE *fp = safe_fopen_wrapper(tmpFilename, "r");
+	FILE *fp = safe_fopen_wrapper_follow(tmpFilename, "r");
 	if(fp == NULL) {
-		if(DEBUG_LEVEL(DEBUG_QUIET)) {
-			debug_printf( DEBUG_QUIET, "Could not open file %s for input\n", filename);
-		}
+		MyString cwd;
+		condor_getcwd( cwd );
+		debug_printf( DEBUG_QUIET, "Could not open file %s for input "
+					"(cwd %s) (errno %d, %s)\n", tmpFilename,
+					cwd.Value(), errno, strerror(errno));
 		return false;
    	}
 
@@ -188,6 +194,8 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 		if (line[0] == 0)       continue;  // Ignore blank lines
 		if (line[0] == COMMENT) continue;  // Ignore comments
 
+		debug_printf( DEBUG_DEBUG_3, "Parsing line <%s>\n", line );
+
 			// Note: strtok() could be replaced by MyString::Tokenize(),
 			// which is much safer, but I don't want to deal with that
 			// right now.  wenger 2005-02-02.
@@ -218,6 +226,7 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 				"Warning: the DAP token is deprecated and may be unsupported "
 				"in a future release.  Use the DATA token\n",
 				filename, lineNumber );
+			check_warning_strictness( DAG_STRICT_2 );
 		}
 
 		else if	(strcasecmp(token, "DATA") == 0) {
@@ -227,12 +236,21 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 					   "submitfile");
 		}
 
+		// Handle a SUBDAG spec
 		else if	(strcasecmp(token, "SUBDAG") == 0) {
 			parsed_line_successfully = parse_subdag( dag, 
 						Job::TYPE_CONDOR,
 						token, filename, lineNumber, tmpDirectory.Value() );
 		}
 
+		// Handle a FINAL spec
+		else if(strcasecmp(token, "FINAL") == 0) {
+			parsed_line_successfully = parse_node( dag, 
+					   Job::TYPE_CONDOR, token,
+					   filename, lineNumber, tmpDirectory.Value(), "",
+					   "submitfile" );
+		}
+
 		// Handle a SCRIPT spec
 		// Example Syntax is:  SCRIPT (PRE|POST) JobName ScriptName Args ...
 		else if ( strcasecmp(token, "SCRIPT") == 0 ) {
@@ -321,6 +339,18 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 		// Handle a JOBSTATE_LOG spec
 		else if(strcasecmp(token, "JOBSTATE_LOG") == 0) {
 			parsed_line_successfully = parse_jobstate_log(dag,
+				filename, lineNumber);
+		}
+		
+		// Handle a PRE_SKIP
+		else if(strcasecmp(token, "PRE_SKIP") == 0) {
+			parsed_line_successfully = parse_pre_skip(dag,
+				filename, lineNumber);
+		}
+
+		// Handle a DONE spec
+		else if(strcasecmp(token, "DONE") == 0) {
+			parsed_line_successfully = parse_done(dag,
 						filename, lineNumber);
 		}
 
@@ -329,7 +359,7 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 			debug_printf( DEBUG_QUIET, "%s (line %d): "
 				"Expected JOB, DATA, SUBDAG, SCRIPT, PARENT, RETRY, "
 				"ABORT-DAG-ON, DOT, VARS, PRIORITY, CATEGORY, MAXJOBS, "
-				"CONFIG, SPLICE or NODE_STATUS_FILE token\n",
+				"CONFIG, SPLICE, FINAL, NODE_STATUS_FILE, or PRE_SKIP token\n",
 				filename, lineNumber );
 			parsed_line_successfully = false;
 		}
@@ -346,6 +376,7 @@ bool parse (Dag *dag, const char *filename, bool useDagDir) {
 	// always remember which were the inital and final nodes for this dag.
 	// If this dag is used as a splice, then this information is very
 	// important to preserve when building dependancy links.
+	dag->LiftSplices(SELF);
 	dag->RecordInitialAndFinalNodes();
 
 	if ( useDagDir ) {
@@ -398,11 +429,24 @@ parse_node( Dag *dag, Job::job_type_t nodeType,
 
 		// first token is the node name
 	const char *nodeName = strtok( NULL, DELIMITERS );
+	if ( !nodeName ) {
+		debug_printf( DEBUG_QUIET, "ERROR: %s (line %d): no node name "
+					"specified\n", dagFile, lineNum );
+		debug_printf( DEBUG_QUIET, "%s\n", expectedSyntax.Value() );
+		return false;
+	}
+
 	MyString tmpNodeName = munge_job_name(nodeName);
 	nodeName = tmpNodeName.Value();
 
 		// next token is the submit file name
 	const char *submitFile = strtok( NULL, DELIMITERS );
+	if ( !submitFile ) {
+		debug_printf( DEBUG_QUIET, "ERROR: %s (line %d): no submit file "
+					"specified\n", dagFile, lineNum );
+		debug_printf( DEBUG_QUIET, "%s\n", expectedSyntax.Value() );
+		return false;
+	}
 
 		// next token (if any) is "DIR" "NOOP", or "DONE" (in that order)
 	const char* nextTok = strtok( NULL, DELIMITERS );
@@ -498,11 +542,13 @@ parse_node( Dag *dag, Job::job_type_t nodeType,
 		debug_printf( DEBUG_NORMAL, "Warning: the use of the JOB "
 					"keyword for nested DAGs is deprecated; please "
 					"use SUBDAG EXTERNAL instead" );
+		check_warning_strictness( DAG_STRICT_3 );
 	}
 
 	// looks ok, so add it
+	bool isFinal = strcasecmp( nodeTypeKeyword, "FINAL" ) == MATCH;
 	if( !AddNode( dag, nodeType, nodeName, directory,
-				submitFile, NULL, NULL, noop, done, whynot ) )
+				submitFile, NULL, NULL, noop, done, isFinal, whynot ) )
 	{
 		debug_printf( DEBUG_QUIET, "ERROR: %s (line %d): %s\n",
 					  dagFile, lineNum, whynot.Value() );
@@ -778,8 +824,7 @@ parse_parent(
 	}
 	
 	if (children.Number() < 1) {
-		debug_printf( DEBUG_QUIET, 
-					  "%s (line %d): Missing Child Job names\n",
+		debug_printf( DEBUG_QUIET, "%s (line %d): Missing Child Job names\n",
 					  filename, lineNumber );
 		exampleSyntax (example);
 		return false;
@@ -796,9 +841,9 @@ parse_parent(
 		children.Rewind();
 		while ((child = children.Next()) != NULL) {
 			if (!dag->AddDependency (parent, child)) {
-				debug_printf( DEBUG_QUIET,
-							  "ERROR: %s (line %d) failed to add dependency between "
-							  "parent node \"%s\" and child node \"%s\"\n",
+				debug_printf( DEBUG_QUIET, "ERROR: %s (line %d) failed"
+						" to add dependency between parent"
+						" node \"%s\" and child node \"%s\"\n",
 							  filename, lineNumber,
 							  parent->GetJobName(), child->GetJobName() );
 				return false;
@@ -845,6 +890,13 @@ parse_retry(
 					  filename, lineNumber, jobNameOrig );
 		return false;
 	}
+
+	if ( job->GetFinal() ) {
+		debug_printf( DEBUG_QUIET, 
+					  "ERROR: %s (line %d): Final job %s cannot have retries\n",
+					  filename, lineNumber, jobNameOrig );
+		return false;
+	}
 	
 	char *s = strtok( NULL, DELIMITERS );
 	if( s == NULL ) {
@@ -944,6 +996,13 @@ parse_abort(
 					  filename, lineNumber, jobNameOrig );
 		return false;
 	}
+
+	if ( job->GetFinal() ) {
+		debug_printf( DEBUG_QUIET, 
+					  "ERROR: %s (line %d): Final job %s cannot have ABORT-DAG-ON specification\n",
+					  filename, lineNumber, jobNameOrig );
+		return false;
+	}
 	
 		// Node abort value.
 	char *abortValStr = strtok( NULL, DELIMITERS );
@@ -1199,6 +1258,25 @@ static bool parse_vars(Dag *dag, const char *filename, int lineNumber) {
 						"names cannot begin with \"queue\"\n", varName.Value() );
 			return false;
 		}
+		// This will be inefficient for jobs with lots of variables
+		// As in O(N^2)
+		job->varNamesFromDag->Rewind();
+		job->varValsFromDag->Rewind();
+		while(MyString* s = job->varNamesFromDag->Next()){
+			job->varValsFromDag->Next(); // To keep up with varNamesFromDag
+			if(varName == *s){
+				debug_printf(DEBUG_NORMAL,"Warning: VAR \"%s\" "
+					"is already defined in job \"%s\" "
+					"(Discovered at file \"%s\", line %d)\n",
+					varName.Value(),job->GetJobName(),filename,
+					lineNumber);
+				check_warning_strictness( DAG_STRICT_2 );
+				debug_printf(DEBUG_NORMAL,"Warning: Setting VAR \"%s\" "
+					"= \"%s\"\n",varName.Value(),varValue.Value());
+				job->varNamesFromDag->DeleteCurrent();
+				job->varValsFromDag->DeleteCurrent();
+			}
+		}
 		debug_printf(DEBUG_DEBUG_1, "Argument added, Name=\"%s\"\tValue=\"%s\"\n", varName.Value(), varValue.Value());
 		bool appendResult;
 		appendResult = job->varNamesFromDag->Append(new MyString(varName));
@@ -1261,6 +1339,13 @@ parse_priority(
 		}
 	}
 
+	if ( job->GetFinal() ) {
+		debug_printf( DEBUG_QUIET, 
+					  "ERROR: %s (line %d): Final job %s cannot have priority\n",
+					  filename, lineNumber, jobNameOrig );
+		return false;
+	}
+
 	//
 	// Next token is the priority value.
 	//
@@ -1300,6 +1385,7 @@ parse_priority(
 		debug_printf( DEBUG_NORMAL, "Warning: new priority %d for node %s "
 					"overrides old value %d\n", priorityVal,
 					job->GetJobName(), job->_nodePriority );
+		check_warning_strictness( DAG_STRICT_2 );
 	}
 	job->_hasNodePriority = true;
 	job->_nodePriority = priorityVal;
@@ -1354,6 +1440,13 @@ parse_category(
 		}
 	}
 
+	if ( job->GetFinal() ) {
+		debug_printf( DEBUG_QUIET, 
+					  "ERROR: %s (line %d): Final job %s cannot have category\n",
+					  filename, lineNumber, jobNameOrig );
+		return false;
+	}
+
 	//
 	// Next token is the category name.
 	//
@@ -1415,7 +1508,7 @@ parse_splice(
 
 	// Check to make sure we don't already have a node with the name of the
 	// splice.
-	if (dag->NodeExists(spliceName.Value()) == true) {
+	if ( dag->NodeExists(spliceName.Value()) ) {
 		debug_printf( DEBUG_QUIET, 
 					  "%s (line %d): "
 					  " Splice name '%s' must not also be a node name.\n",
@@ -1536,6 +1629,13 @@ parse_splice(
 		return false;
 	}
 
+	// Splices cannot have final nodes.
+	if ( splice_dag->HasFinalNode() ) {
+		debug_printf( DEBUG_QUIET, "ERROR: splice %s has a final node; "
+					"splices cannot have final nodes\n", spliceName.Value() );
+		return false;
+	}
+
 	// munge the splice name
 	spliceName = munge_job_name(spliceName.Value());
 
@@ -1559,16 +1659,12 @@ parse_splice(
 			spliceName.Value());
 		return false;
 	}
-
-	// For now, we only keep track of the splice levels just below this dag.
-	dag->LiftChildSplices();
-
 	debug_printf(DEBUG_DEBUG_1, "Done parsing splice %s\n", spliceName.Value());
 
 	// pop the just pushed value off of the end of the ext array
 	free(_spliceScope[_spliceScope.getlast()]);
 	_spliceScope.truncate(_spliceScope.getlast() - 1);
-
+	debug_printf(DEBUG_DEBUG_1, "_spliceScope has length %d\n", _spliceScope.length());
 	return true;
 }
 
@@ -1765,6 +1861,152 @@ parse_jobstate_log(
 	return true;
 }
 
+//-----------------------------------------------------------------------------
+// 
+// Function: parse_pre_skip
+// Purpose:  Tell dagman to skip execution if the PRE script exits with a
+//           a certain code
+//-----------------------------------------------------------------------------
+bool 
+parse_pre_skip( Dag  *dag, 
+	const char *filename, 
+	int  lineNumber)
+{
+	const char * example = "PRE_SKIP JobName Exitcode";
+	Job * job = NULL;
+	MyString whynot;
+
+		//
+		// second token is the JobName
+		//
+	const char *jobName = strtok( NULL, DELIMITERS );
+	const char *jobNameOrig = jobName; // for error output
+	if ( jobName == NULL ) {
+		debug_printf( DEBUG_QUIET, "%s (line %d): Missing job name\n",
+				filename, lineNumber );
+		exampleSyntax( example );
+		return false;
+	} else if ( isReservedWord(jobName) ) {
+		debug_printf( DEBUG_QUIET,
+				"%s (line %d): JobName cannot be a reserved word\n",
+				filename, lineNumber );
+		exampleSyntax( example );
+		return false;
+	} else {
+		debug_printf( DEBUG_DEBUG_1, "jobName: %s\n", jobName );
+		MyString tmpJobName = munge_job_name( jobName );
+		jobName = tmpJobName.Value();
+
+		job = dag->FindNodeByName( jobName );
+		if (job == NULL) {
+			debug_printf( DEBUG_QUIET, 
+					"%s (line %d): Unknown Job %s\n",
+					filename, lineNumber, jobNameOrig );
+			return false;
+		}
+	}
+
+		//
+		// The rest of the line consists of the exitcode
+		//
+	const char *exitCodeStr = strtok( NULL, DELIMITERS );
+	if ( exitCodeStr == NULL ) {
+		debug_printf( DEBUG_QUIET, "%s (line %d): Missing exit code\n",
+				filename, lineNumber );
+		exampleSyntax( example );
+		return false;
+	}
+
+	char *tmp;
+	int exitCode = (int)strtol( exitCodeStr, &tmp, 10 );
+	if ( tmp == exitCodeStr ) {
+		debug_printf( DEBUG_QUIET,
+				"%s (line %d): Invalid exit code \"%s\"\n",
+				filename, lineNumber, exitCodeStr );
+		exampleSyntax( example );
+		return false;
+	}
+
+		//
+		// Anything else is garbage
+		//
+	const char *nextTok = strtok( NULL, DELIMITERS );
+	if ( nextTok ) {
+		debug_printf( DEBUG_QUIET, "ERROR: %s (line %d): invalid "
+				"parameter \"%s\"\n", filename, lineNumber, nextTok );
+		exampleSyntax( example );
+		return false;
+	}
+
+	if ( !job->AddPreSkip( exitCode, whynot ) ) {
+		debug_printf( DEBUG_SILENT, "ERROR: %s (line %d): failed to add "
+				"PRE_SKIP note to node %s: %s\n",
+				filename, lineNumber, jobNameOrig,
+				whynot.Value() );
+		return false;
+	}
+	return true;
+}
+
+//-----------------------------------------------------------------------------
+// 
+// Function: parse_done
+// Purpose:  Parse a line of the format "Done jobname"
+// 
+//-----------------------------------------------------------------------------
+static bool 
+parse_done(
+	Dag  *dag, 
+	const char *filename, 
+	int  lineNumber)
+{
+	const char *example = "Done JobName";
+	
+	const char *jobName = strtok( NULL, DELIMITERS );
+	const char *jobNameOrig = jobName; // for error output
+	if( jobName == NULL ) {
+		debug_printf( DEBUG_QUIET,
+					  "%s (line %d): Missing job name\n",
+					  filename, lineNumber );
+		exampleSyntax( example );
+		return false;
+	}
+
+	MyString tmpJobName = munge_job_name( jobName );
+	jobName = tmpJobName.Value();
+
+	//
+	// Check for illegal extra tokens.
+	//
+	char *extraTok = strtok( NULL, DELIMITERS );
+	if ( extraTok != NULL ) {
+		debug_printf( DEBUG_QUIET,
+					  "%s (line %d): Extra token (%s) on DONE line\n",
+					  filename, lineNumber, extraTok );
+		exampleSyntax( example );
+		return false;
+	}
+
+	Job *job = dag->FindNodeByName( jobName );
+	if( job == NULL ) {
+		debug_printf( DEBUG_QUIET, 
+					  "Warning: %s (line %d): Unknown Job %s\n",
+					  filename, lineNumber, jobNameOrig );
+		return !check_warning_strictness( DAG_STRICT_1, false );
+	}
+
+	if ( job->GetFinal() ) {
+		debug_printf( DEBUG_QUIET, 
+					  "Warning: %s (line %d): FINAL Job %s cannot be set to DONE\n",
+					  filename, lineNumber, jobNameOrig );
+		return !check_warning_strictness( DAG_STRICT_1, false );
+	}
+
+	job->SetStatus( Job::STATUS_DONE );
+	
+	return true;
+}
+
 static MyString munge_job_name(const char *jobName)
 {
 		//
@@ -1783,23 +2025,16 @@ static MyString munge_job_name(const char *jobName)
 
 static MyString current_splice_scope(void)
 {
-	int i;
-	MyString scope;
 	MyString tmp;
-
-	for (i = 0; i < _spliceScope.length(); i++)
-	{
-		tmp = _spliceScope[i];
+	MyString scope;
+	if(_spliceScope.length() > 0) {
 		// While a natural choice might have been : as a splice scoping
 		// separator, this character was chosen because it is a valid character
 		// on all the file systems we use (whereas : can't be a file system
 		// character on windows). The plus, and really anything other than :,
 		// isn't the best choice. Sorry.
-		scope += tmp + "+";
+		tmp = _spliceScope[_spliceScope.length() - 1];
+		scope = tmp + "+";
 	}
-
 	return scope;
 }
-
-
-
