@@ -61,6 +61,13 @@ AdminEvent::AdminEvent( void ) :
 	m_shutdownEnd = 0;
 	m_shutdownSize = 0;
 	m_newshutdownSize = 0;
+
+	m_lastShutdown = 0;
+	m_spoolStorage = 0;
+
+	m_newshutdownAdminRate = 0.0f;
+	m_timeNow = 0;
+	m_timeSinceNow = 0;
 }
 
 AdminEvent::~AdminEvent( void )
@@ -401,7 +408,7 @@ AdminEvent::spoolClassAd( ClassAd * ca_shutdownRate, char *direction )
 
 */
 
-int 
+void 
 AdminEvent::th_DoShutdown_States( void )
 {
 	int secondsforvacates = 0;
@@ -604,7 +611,6 @@ AdminEvent::th_DoShutdown_States( void )
 				"th_DoShutdown_States( Why Default Case )\n");
 			break;
 	}
-	return(0);
 }
 
 /*
@@ -613,7 +619,7 @@ AdminEvent::th_DoShutdown_States( void )
 
 */
 
-int 
+void
 AdminEvent::th_DoShutdown( void )
 {
 	DCMaster *		d;
@@ -632,10 +638,9 @@ AdminEvent::th_DoShutdown( void )
 	dprintf(D_ALWAYS,"call Shutdown now.....\n");
 	//d->sendMasterOff(wantTcp);
 	delete d;
-	return(0);
 }
 
-int 
+void 
 AdminEvent::th_maintainCheckpoints( void )
 {
 	ClassAd *ad;
@@ -791,7 +796,6 @@ AdminEvent::th_maintainCheckpoints( void )
 		m_mystate = EVENT_DONE;
 		changeState(EVENT_NOW, m_mystate);
 	}
-	return(0);
 }
 
 /*
@@ -1009,6 +1013,10 @@ AdminEvent::process_ShutdownTime( char *req_time )
 	// secs, day and hour from the next scan of the req_time`
 	tmnow = localtime(&m_timeNow);
 
+#ifdef WIN32
+
+#else
+
 	// find secs, hour and minutes
 	res = strptime(req_time,"%H:%M:%S",&tm);
 	if(res != NULL) {
@@ -1016,6 +1024,7 @@ AdminEvent::process_ShutdownTime( char *req_time )
 			"Processing Shutdown Time String<<LEFTOVERS--%s-->>\n",res);
 		//return(-1);
 	}
+#endif
 
 	// Get today into request structure
 	tm.tm_mday = tmnow->tm_mday;
@@ -1219,7 +1228,7 @@ AdminEvent::standardUProcess( int batchsz, bool vacate )
 		ad->LookupInteger( ATTR_JOB_START, jobstart ); 
 		ad->LookupInteger( ATTR_JOB_UNIVERSE, jobuniverse ); 
 		ad->LookupInteger( ATTR_IMAGE_SIZE, imagesize ); 
-		if (param_boolean("ALLOW_VM_CRUFT", true)) {
+		if (param_boolean("ALLOW_VM_CRUFT", false)) {
 			if (!ad->LookupInteger(ATTR_SLOT_ID, slotid)) {
 				ad->LookupInteger(ATTR_VIRTUAL_MACHINE_ID, slotid);
 			}
