@@ -32,6 +32,8 @@
 #include "condor_email.h"
 #include "classad_helpers.h"
 #include "classad_merge.h"
+#include "condor_holdcodes.h"
+#include "exit.h"
 
 #define HASH_TABLE_SIZE			500
 
@@ -410,7 +412,7 @@ void BaseJob::UpdateRuntimeStats()
 		accum_time += (float)( time(NULL) - shadowBirthdate );
 		jobAd->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, accum_time );
 		jobAd->Assign( ATTR_JOB_WALL_CLOCK_CKPT,(char *)NULL );
-		jobAd->Assign( ATTR_SHADOW_BIRTHDATE, 0 );
+		jobAd->AssignExpr( ATTR_SHADOW_BIRTHDATE, "UNDEFINED" );
 
 		requestScheddUpdate( this, false );
 
@@ -741,14 +743,19 @@ int BaseJob::EvalPeriodicJobExpr()
 
 	RestoreJobTime( old_run_time, old_run_time_dirty );
 
-	const char *reason = user_policy.FiringReason();
-	if ( reason == NULL ) {
+	MyString reason_buf;
+	int reason_code;
+	int reason_subcode;
+	user_policy.FiringReason(reason_buf,reason_code,reason_subcode);
+	char const *reason = reason_buf.Value();
+	if ( reason == NULL || !reason[0] ) {
 		reason = "Unknown user policy expression";
 	}
 
 	switch( action ) {
 	case UNDEFINED_EVAL:
-		JobHeld( reason );
+	case HOLD_IN_QUEUE:
+		JobHeld( reason, reason_code, reason_subcode );
 		SetEvaluateState();
 		break;
 	case STAYS_IN_QUEUE:
@@ -758,10 +765,6 @@ int BaseJob::EvalPeriodicJobExpr()
 		JobRemoved( reason );
 		SetEvaluateState();
 		break;
-	case HOLD_IN_QUEUE:
-		JobHeld( reason );
-		SetEvaluateState();
-		break;
 	case RELEASE_FROM_HOLD:
 			// When a job gets held and then released while the gridmanager
 			// is managing it, the gridmanager cleans up and deletes its
@@ -812,14 +815,19 @@ int BaseJob::EvalOnExitJobExpr()
 		jobAd->AssignExpr( ATTR_ON_EXIT_SIGNAL, "Undefined" );
 	}
 
-	const char *reason = user_policy.FiringReason();
-	if ( reason == NULL ) {
+	MyString reason_buf;
+	int reason_code;
+	int reason_subcode;
+	user_policy.FiringReason(reason_buf,reason_code,reason_subcode);
+	const char *reason = reason_buf.Value();
+	if ( reason == NULL || !reason[0] ) {
 		reason = "Unknown user policy expression";
 	}
 
 	switch( action ) {
 	case UNDEFINED_EVAL:
-		JobHeld( reason );
+	case HOLD_IN_QUEUE:
+		JobHeld( reason, reason_code, reason_subcode );
 		break;
 	case STAYS_IN_QUEUE:
 			// clean up job but don't set status to complete
@@ -827,9 +835,6 @@ int BaseJob::EvalOnExitJobExpr()
 	case REMOVE_FROM_QUEUE:
 		JobCompleted();
 		break;
-	case HOLD_IN_QUEUE:
-		JobHeld( reason );
-		break;
 	default:
 		EXCEPT( "Unknown action (%d) in BaseJob::EvalAtExitJobExpr", 
 				action );
@@ -1009,7 +1014,7 @@ WriteExecuteEventToUserLog( ClassAd *job_ad )
 						  sizeof(hostname) - 1 );
 
 	ExecuteEvent event;
-	strcpy( event.executeHost, hostname );
+	event.setExecuteHost( hostname );
 	int rc = ulog->writeEvent(&event,job_ad);
 	delete ulog;
 
@@ -1598,218 +1603,15 @@ WriteJobStatusKnownEventToUserLog( ClassAd *job_ad )
 	return true;
 }
 
-// TODO: This appears three times in the Condor source.  Unify?
-//   (It only is made visible in condor_shadow.jim's prototypes.h.)
-static char *
-d_format_time( double dsecs )
-{
-	int days, hours, minutes, secs;
-	static char answer[25];
-
-	const int SECONDS = 1;
-	const int MINUTES = (60 * SECONDS);
-	const int HOURS   = (60 * MINUTES);
-	const int DAYS    = (24 * HOURS);
-
-	secs = (int)dsecs;
-
-	days = secs / DAYS;
-	secs %= DAYS;
-
-	hours = secs / HOURS;
-	secs %= HOURS;
-
-	minutes = secs / MINUTES;
-	secs %= MINUTES;
-
-	(void)sprintf(answer, "%3d %02d:%02d:%02d", days, hours, minutes, secs);
-
-	return( answer );
-}
-
 void
-EmailTerminateEvent(ClassAd * job_ad, bool exit_status_known)
+EmailTerminateEvent(ClassAd * job_ad, bool   /*exit_status_known*/)
 {
-		// TODO We should use the Email class here, rather than duplicating
-		//   most of what it does.
 	if ( !job_ad ) {
 		dprintf(D_ALWAYS, 
 				"email_terminate_event called with invalid ClassAd\n");
 		return;
 	}
 
-	int cluster, proc;
-	job_ad->LookupInteger( ATTR_CLUSTER_ID, cluster );
-	job_ad->LookupInteger( ATTR_PROC_ID, proc );
-
-	int notification = NOTIFY_COMPLETE; // default
-	job_ad->LookupInteger(ATTR_JOB_NOTIFICATION,notification);
-
-	switch( notification ) {
-		case NOTIFY_NEVER:    return;
-		case NOTIFY_ALWAYS:   break;
-		case NOTIFY_COMPLETE: break;
-		case NOTIFY_ERROR:    return;
-		default:
-			dprintf(D_ALWAYS, 
-				"Condor Job %d.%d has unrecognized notification of %d\n",
-				cluster, proc, notification );
-				// When in doubt, better send it anyway...
-			break;
-	}
-
-	char subjectline[50];
-	sprintf( subjectline, "Condor Job %d.%d", cluster, proc );
-	FILE * mailer =  email_user_open( job_ad, subjectline );
-
-	if( ! mailer ) {
-		// Is message redundant?  Check email_user_open and euo's children.
-		dprintf(D_ALWAYS, 
-			"email_terminate_event failed to open a pipe to a mail program.\n");
-		return;
-	}
-
-		// gather all the info out of the job ad which we want to 
-		// put into the email message.
-	std::string JobName;
-	job_ad->LookupString( ATTR_JOB_CMD, JobName );
-
-	MyString Args;
-	ArgList::GetArgsStringForDisplay(job_ad,&Args);
-	
-	/*
-	// Not present.  Probably doesn't make sense for Globus
-	int had_core = FALSE;
-	job_ad->LookupBool( ATTR_JOB_CORE_DUMPED, had_core );
-	*/
-
-	int q_date = 0;
-	job_ad->LookupInteger(ATTR_Q_DATE,q_date);
-	
-	float remote_sys_cpu = 0.0;
-	job_ad->LookupFloat(ATTR_JOB_REMOTE_SYS_CPU, remote_sys_cpu);
-	
-	float remote_user_cpu = 0.0;
-	job_ad->LookupFloat(ATTR_JOB_REMOTE_USER_CPU, remote_user_cpu);
-	
-	int image_size = 0;
-	job_ad->LookupInteger(ATTR_IMAGE_SIZE, image_size);
-	
-	/*
-	int shadow_bday = 0;
-	job_ad->LookupInteger( ATTR_SHADOW_BIRTHDATE, shadow_bday );
-	*/
-	
-	float previous_runs = 0;
-	job_ad->LookupFloat( ATTR_JOB_REMOTE_WALL_CLOCK, previous_runs );
-	
-	time_t arch_time=0;	/* time_t is 8 bytes some archs and 4 bytes on other
-						   archs, and this means that doing a (time_t*)
-						   cast on & of a 4 byte int makes my life hell.
-						   So we fix it by assigning the time we want to
-						   a real time_t variable, then using ctime()
-						   to convert it to a string */
-	
-	time_t now = time(NULL);
-
-	fprintf( mailer, "Your Condor job %d.%d \n", cluster, proc);
-	if ( JobName.length() ) {
-		fprintf(mailer,"\t%s %s\n",JobName.c_str(),Args.Value());
-	}
-	if(exit_status_known) {
-		fprintf(mailer, "has ");
-
-		int int_val;
-		if( job_ad->LookupBool(ATTR_ON_EXIT_BY_SIGNAL, int_val) ) {
-			if( int_val ) {
-				if( job_ad->LookupInteger(ATTR_ON_EXIT_SIGNAL, int_val) ) {
-					fprintf(mailer, "exited with the signal %d.\n", int_val);
-				} else {
-					fprintf(mailer, "exited with an unknown signal.\n");
-					dprintf( D_ALWAYS, "(%d.%d) Job ad lacks %s.  "
-						 "Signal code unknown.\n", cluster, proc, 
-						 ATTR_ON_EXIT_SIGNAL);
-				}
-			} else {
-				if( job_ad->LookupInteger(ATTR_ON_EXIT_CODE, int_val) ) {
-					fprintf(mailer, "exited normally with status %d.\n",
-						int_val);
-				} else {
-					fprintf(mailer, "exited normally with unknown status.\n");
-					dprintf( D_ALWAYS, "(%d.%d) Job ad lacks %s.  "
-						 "Return code unknown.\n", cluster, proc, 
-						 ATTR_ON_EXIT_CODE);
-				}
-			}
-		} else {
-			fprintf(mailer,"has exited.\n");
-			dprintf( D_ALWAYS, "(%d.%d) Job ad lacks %s.  ",
-				 cluster, proc, ATTR_ON_EXIT_BY_SIGNAL);
-		}
-	} else {
-		fprintf(mailer,"has exited.\n");
-	}
-
-	/*
-	if( had_core ) {
-		fprintf( mailer, "Core file is: %s\n", getCoreName() );
-	}
-	*/
-
-	arch_time = q_date;
-	fprintf(mailer, "\n\nSubmitted at:        %s", ctime(&arch_time));
-	
-	double real_time = now - q_date;
-	arch_time = now;
-	fprintf(mailer, "Completed at:        %s", ctime(&arch_time));
-	
-	fprintf(mailer, "Real Time:           %s\n", 
-			d_format_time(real_time));
-
-
-	fprintf( mailer, "\n" );
-	
-		// TODO We don't necessarily have this information even if we do
-		//   have the exit status
-	if( exit_status_known ) {
-		fprintf(mailer, "Virtual Image Size:  %d Kilobytes\n\n", image_size);
-	}
-
-	double rutime = remote_user_cpu;
-	double rstime = remote_sys_cpu;
-	double trtime = rutime + rstime;
-	/*
-	double wall_time = now - shadow_bday;
-	fprintf(mailer, "Statistics from last run:\n");
-	fprintf(mailer, "Allocation/Run time:     %s\n",d_format_time(wall_time) );
-	*/
-		// TODO We don't necessarily have this information even if we do
-		//   have the exit status
-	if( exit_status_known ) {
-		fprintf(mailer, "Remote User CPU Time:    %s\n", d_format_time(rutime) );
-		fprintf(mailer, "Remote System CPU Time:  %s\n", d_format_time(rstime) );
-		fprintf(mailer, "Total Remote CPU Time:   %s\n\n", d_format_time(trtime));
-	}
-
-	/*
-	double total_wall_time = previous_runs + wall_time;
-	fprintf(mailer, "Statistics totaled from all runs:\n");
-	fprintf(mailer, "Allocation/Run time:     %s\n",
-			d_format_time(total_wall_time) );
-
-	// TODO: Can we/should we get this for Globus jobs.
-		// TODO: deal w/ total bytes <- obsolete? in original code)
-	float network_bytes;
-	network_bytes = bytesSent();
-	fprintf(mailer, "\nNetwork:\n" );
-	fprintf(mailer, "%10s Run Bytes Received By Job\n", 
-			metric_units(network_bytes) );
-	network_bytes = bytesReceived();
-	fprintf(mailer, "%10s Run Bytes Sent By Job\n",
-			metric_units(network_bytes) );
-	*/
-
-	email_custom_attributes(mailer, job_ad);
-
-	email_close(mailer);
+	Email email;
+	email.sendExit(job_ad, JOB_EXITED);
 }
