@@ -24,14 +24,10 @@
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
-#include "spooled_job_files.h"
-#include "daemon.h"
-#include "dc_schedd.h"
-#include "setenv.h"
+#include "condor_config.h"
 
 #include "gridmanager.h"
 #include "infnbatchjob.h"
-#include "condor_config.h"
 
 
 // GridManager job states
@@ -114,15 +110,15 @@ bool INFNBatchJobAdMatch( const ClassAd *job_ad ) {
 		//   system names should be used (pbs, lsf). Glite are the only
 		//   people who care about the old value. This changed happend in
 		//   Condor 6.7.12.
-		// TODO: Why are we doing a substring match? These are the exact
-		//   string we expect to see.
 	if ( job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe ) &&
 		 universe == CONDOR_UNIVERSE_GRID &&
 		 job_ad->LookupString( ATTR_GRID_RESOURCE, resource ) &&
 		 ( strncasecmp( resource.c_str(), "blah", 4 ) == 0 ||
-		   strncasecmp( resource.c_str(), "pbs", 4 ) == 0 ||
-		   strncasecmp( resource.c_str(), "lsf", 4 ) == 0 ||
-		   strncasecmp( resource.c_str(), "nqs", 4 ) == 0 ||
+		   strncasecmp( resource.c_str(), "batch", 5 ) == 0 ||
+		   strncasecmp( resource.c_str(), "pbs", 3 ) == 0 ||
+		   strncasecmp( resource.c_str(), "lsf", 3 ) == 0 ||
+		   strncasecmp( resource.c_str(), "nqs", 3 ) == 0 ||
+		   strncasecmp( resource.c_str(), "sge", 3 ) == 0 ||
 		   strncasecmp( resource.c_str(), "naregi", 6 ) == 0 ) ) {
 
 		return true;
@@ -144,9 +140,10 @@ int INFNBatchJob::maxConnectFailures = 3;		// default value
 INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	: BaseJob( classad )
 {
-	char buff[4096];
+	std::string buff;
 	std::string error_string = "";
 	char *gahp_path;
+	ArgList gahp_args;
 
 	gahpAd = NULL;
 	gmState = GM_INIT;
@@ -158,6 +155,7 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 	remoteJobId = NULL;
 	lastPollTime = 0;
 	pollNow = false;
+	myResource = NULL;
 	gahp = NULL;
 	jobProxy = NULL;
 	remoteProxyExpireTime = 0;
@@ -168,38 +166,75 @@ INFNBatchJob::INFNBatchJob( ClassAd *classad )
 		jobAd->AssignExpr( ATTR_HOLD_REASON, "Undefined" );
 	}
 
-	buff[0] = '\0';
+	buff = "";
 	jobAd->LookupString( ATTR_GRID_RESOURCE, buff );
-	if ( buff[0] != '\0' ) {
-		batchType = strdup( buff );
+	if ( buff != "" ) {
+		const char *token;
+		Tokenize( buff );
+
+		token = GetNextToken( " ", false );
+		if ( !strcmp( "batch", token ) ) {
+			token = GetNextToken( " ", false );
+		}
+		batchType = strdup( token );
+
+		while ( (token = GetNextToken( " ", false )) ) {
+			gahp_args.AppendArg( token );
+		}
 	} else {
 		sprintf( error_string, "%s is not set in the job ad",
 							  ATTR_GRID_RESOURCE );
 		goto error_exit;
 	}
 
-	buff[0] = '\0';
+	buff = "";
 	jobAd->LookupString( ATTR_GRID_JOB_ID, buff );
-	if ( buff[0] != '\0' ) {
-		SetRemoteJobId( strchr( buff, ' ' ) + 1 );
+	if ( buff != "" ) {
+		SetRemoteJobId( strchr( buff.c_str(), ' ' ) + 1 );
 	} else {
 		remoteState = JOB_STATE_UNSUBMITTED;
 	}
 
 	strupr( batchType );
 
-	sprintf( buff, "%s_GAHP", batchType );
-	gahp_path = param(buff);
-	if ( gahp_path == NULL ) {
-		sprintf( error_string, "%s not defined", buff );
-		goto error_exit;
+	if ( gahp_args.Count() > 0 ) {
+		gahp_path = param( "REMOTE_GAHP" );
+		if ( gahp_path == NULL ) {
+			sprintf( error_string, "REMOTE_GAHP not defined" );
+			goto error_exit;
+		}
+	} else {
+		sprintf( buff, "%s_GAHP", batchType );
+		gahp_path = param(buff.c_str());
+		if ( gahp_path == NULL ) {
+			gahp_path = param( "BATCH_GAHP" );
+			if ( gahp_path == NULL ) {
+				sprintf( error_string, "Neither %s nor %s defined", buff.c_str(),
+						 "BATCH_GAHP" );
+				goto error_exit;
+			}
+		}
 	}
-	gahp = new GahpClient( batchType, gahp_path );
+
+	buff = batchType;
+	if ( gahp_args.Count() > 0 ) {
+		sprintf_cat( buff, "/%s", gahp_args.GetArg( 0 ) );
+	}
+	gahp = new GahpClient( buff.c_str(), gahp_path, &gahp_args );
 	free( gahp_path );
 
+	myResource = INFNBatchResource::FindOrCreateResource( batchType,
+														  gahp_args.GetArg(0) );
+	myResource->RegisterJob( this );
+	if ( remoteJobId ) {
+		myResource->AlreadySubmitted( this );
+	}
+
 		// Does this have to be lower-case for SetRemoteJobId()?
+
 	strlwr( batchType );
 
+	ASSERT( gahp != NULL );
 	gahp->setNotificationTimerId( evaluateStateTid );
 	gahp->setMode( GahpClient::normal );
 	gahp->setTimeout( gahpCallTimeout );
@@ -227,6 +262,9 @@ INFNBatchJob::~INFNBatchJob()
 	if ( jobProxy != NULL ) {
 		ReleaseProxy( jobProxy, (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	}
+	if ( myResource ) {
+		myResource->UnregisterJob( this );
+	}
 	if ( batchType != NULL ) {
 		free( batchType );
 	}
@@ -273,6 +311,8 @@ void INFNBatchJob::doEvaluateState()
 		old_gm_state = gmState;
 		old_remote_state = remoteState;
 
+		ASSERT ( gahp != NULL || gmState == GM_HOLD || gmState == GM_DELETE );
+
 		switch ( gmState ) {
 		case GM_INIT: {
 			// This is the state all jobs start in when the GlobusJob object
@@ -355,6 +395,13 @@ void INFNBatchJob::doEvaluateState()
 			// another one.
 			if ( now >= lastSubmitAttempt + submitInterval ) {
 				char *job_id_string = NULL;
+
+				// Once RequestSubmit() is called at least once, you must
+				// CancelRequest() once you're done with the request call
+				if ( myResource->RequestSubmit( this ) == false ) {
+					break;
+				}
+
 				if ( gahpAd == NULL ) {
 					gahpAd = buildSubmitAd();
 				}
@@ -362,6 +409,7 @@ void INFNBatchJob::doEvaluateState()
 					gmState = GM_HOLD;
 					break;
 				}
+
 				rc = gahp->blah_job_submit( gahpAd, &job_id_string );
 				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
 					 rc == GAHPCLIENT_COMMAND_PENDING ) {
@@ -383,6 +431,7 @@ void INFNBatchJob::doEvaluateState()
 							 procID.cluster, procID.proc,
 							 gahp->getErrorString() );
 					errorString = gahp->getErrorString();
+					myResource->CancelSubmit( this );
 					gmState = GM_UNSUBMITTED;
 					reevaluate_state = true;
 				}
@@ -520,6 +569,7 @@ void INFNBatchJob::doEvaluateState()
 				// cleared in GM_CLEAR_REQUEST (it might go to GM_HOLD first).
 				SetRemoteJobId( NULL );
 				requestScheddUpdate( this, false );
+				myResource->CancelSubmit( this );
 				gmState = GM_CLEAR_REQUEST;
 			}
 			} break;
@@ -545,6 +595,7 @@ void INFNBatchJob::doEvaluateState()
 					break;
 				}
 				SetRemoteJobId( NULL );
+				myResource->CancelSubmit( this );
 			}
 			if ( condorState == REMOVED ) {
 				gmState = GM_DELETE;
@@ -572,6 +623,7 @@ void INFNBatchJob::doEvaluateState()
 			}
 			errorString = "";
 			SetRemoteJobId( NULL );
+			myResource->CancelSubmit( this );
 			JobIdle();
 			if ( submitLogged ) {
 				JobEvicted();
@@ -772,7 +824,7 @@ void INFNBatchJob::ProcessRemoteAd( ClassAd *remote_ad )
 
 BaseResource *INFNBatchJob::GetResource()
 {
-	return (BaseResource *)NULL;
+	return (BaseResource *)myResource;
 }
 
 ClassAd *INFNBatchJob::buildSubmitAd()
@@ -796,7 +848,7 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 
 	int index;
 	const char *attrs_to_copy[] = {
-		ATTR_JOB_CMD,
+//		ATTR_JOB_CMD,
 		ATTR_JOB_ARGUMENTS1,
 		ATTR_JOB_ARGUMENTS2,
 		ATTR_JOB_ENVIRONMENT1,
@@ -831,6 +883,24 @@ ClassAd *INFNBatchJob::buildSubmitAd()
 		}
 	}
 
+	expr = "";
+	jobAd->LookupString( ATTR_JOB_REMOTE_IWD, expr );
+	if ( !expr.IsEmpty() ) {
+		submit_ad->Assign( ATTR_JOB_IWD, expr );
+	}
+
+	// The blahp expects the Cmd attribute to contain the full pathname
+	// of the job executable.
+	jobAd->LookupString( ATTR_JOB_CMD, expr );
+	if ( expr.FindChar( '/' ) < 0 ) {
+		std::string fullpath;
+		submit_ad->LookupString( ATTR_JOB_IWD, fullpath );
+		sprintf_cat( fullpath, "/%s", expr.Value() );
+		submit_ad->Assign( ATTR_JOB_CMD, fullpath );
+	} else {
+		submit_ad->Assign( ATTR_JOB_CMD, expr );
+	}
+
 		// CRUFT: In the current glite code, jobs have a grid-type of 'blah'
 		//   and the blahp looks at the attribute "gridtype" for 'pbs' or
 		//   'lsf'. Until the blahp changes to look at GridResource, set
