@@ -24,7 +24,7 @@
 #include "condor_string.h"	// for strnewp and friends
 #include "condor_daemon_core.h"
 #include "basename.h"
-#include "condor_ckpt_name.h"
+#include "spooled_job_files.h"
 #include "nullfile.h"
 #include "filename_tools.h"
 #include "condor_url.h"
@@ -77,24 +77,31 @@ static const char *GMStateNames[] = {
 	"GM_START"
 };
 
+#define REMOTE_STATE_ACCEPTING		"ACCEPTING"
 #define REMOTE_STATE_ACCEPTED		"ACCEPTED"
 #define REMOTE_STATE_PREPARING		"PREPARING"
+#define REMOTE_STATE_PREPARED		"PREPARED"
 #define REMOTE_STATE_SUBMITTING		"SUBMITTING"
-#define REMOTE_STATE_INLRMS			"INLRMS"
-#define REMOTE_STATE_CANCELLING		"CANCELLING"
+#define REMOTE_STATE_INLRMS_R		"INLRMS: R"
+#define REMOTE_STATE_INLRMS_Q		"INLRMS: Q"
+#define REMOTE_STATE_INLRMS_S		"INLRMS: S"
+#define REMOTE_STATE_INLRMS_E		"INLRMS: E"
+#define REMOTE_STATE_INLRMS_O		"INLRMS: O"
+#define REMOTE_STATE_KILLING		"KILLING"
+#define REMOTE_STATE_EXECUTED		"EXECUTED"
 #define REMOTE_STATE_FINISHING		"FINISHING"
 #define REMOTE_STATE_FINISHED		"FINISHED"
-#define REMOTE_STATE_PENDING		"PENDING:PREPARING"
+#define REMOTE_STATE_FAILED			"FAILED"
+#define REMOTE_STATE_KILLED			"KILLED"
+#define REMOTE_STATE_DELETED		"DELETED"
 
 #define REMOTE_STDOUT_NAME	"_condor_stdout"
 #define REMOTE_STDERR_NAME	"_condor_stderr"
 
 // Filenames are case insensitive on Win32, but case sensitive on Unix
 #ifdef WIN32
-#	define file_strcmp _stricmp
 #	define file_contains contains_anycase
 #else
-#	define file_strcmp strcmp
 #	define file_contains contains
 #endif
 
@@ -130,11 +137,11 @@ void NordugridJobReconfig()
 
 bool NordugridJobAdMatch( const ClassAd *job_ad ) {
 	int universe;
-	MyString resource;
+	std::string resource;
 	if ( job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe ) &&
 		 universe == CONDOR_UNIVERSE_GRID &&
 		 job_ad->LookupString( ATTR_GRID_RESOURCE, resource ) &&
-		 strncasecmp( resource.Value(), "nordugrid ", 10 ) == 0 ) {
+		 strncasecmp( resource.c_str(), "nordugrid ", 10 ) == 0 ) {
 
 		return true;
 	}
@@ -155,7 +162,7 @@ NordugridJob::NordugridJob( ClassAd *classad )
 	: BaseJob( classad )
 {
 	char buff[4096];
-	MyString error_string = "";
+	std::string error_string = "";
 	char *gahp_path = NULL;
 
 	remoteJobId = NULL;
@@ -182,10 +189,11 @@ NordugridJob::NordugridJob( ClassAd *classad )
 		jobAd->AssignExpr( ATTR_HOLD_REASON, "Undefined" );
 	}
 
-	jobProxy = AcquireProxy( jobAd, error_string, evaluateStateTid );
+	jobProxy = AcquireProxy( jobAd, error_string,
+							 (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	if ( jobProxy == NULL ) {
 		if ( error_string == "" ) {
-			error_string.sprintf( "%s is not set in the job ad",
+			sprintf( error_string, "%s is not set in the job ad",
 								  ATTR_X509_USER_PROXY );
 		}
 		goto error_exit;
@@ -197,7 +205,7 @@ NordugridJob::NordugridJob( ClassAd *classad )
 		goto error_exit;
 	}
 	snprintf( buff, sizeof(buff), "NORDUGRID/%s",
-			  jobProxy->subject->subject_name );
+			  jobProxy->subject->fqan );
 	gahp = new GahpClient( buff, gahp_path );
 	gahp->setNotificationTimerId( evaluateStateTid );
 	gahp->setMode( GahpClient::normal );
@@ -210,34 +218,33 @@ NordugridJob::NordugridJob( ClassAd *classad )
 	jobAd->LookupString( ATTR_GRID_RESOURCE, buff );
 	if ( buff[0] != '\0' ) {
 		const char *token;
-		MyString str = buff;
 
-		str.Tokenize();
+		Tokenize( buff );
 
-		token = str.GetNextToken( " ", false );
-		if ( !token || stricmp( token, "nordugrid" ) ) {
-			error_string.sprintf( "%s not of type nordugrid",
+		token = GetNextToken( " ", false );
+		if ( !token || strcasecmp( token, "nordugrid" ) ) {
+			sprintf( error_string, "%s not of type nordugrid",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
 
-		token = str.GetNextToken( " ", false );
+		token = GetNextToken( " ", false );
 		if ( token && *token ) {
 			resourceManagerString = strdup( token );
 		} else {
-			error_string.sprintf( "%s missing server name",
+			sprintf( error_string, "%s missing server name",
 								  ATTR_GRID_RESOURCE );
 			goto error_exit;
 		}
 
 	} else {
-		error_string.sprintf( "%s is not set in the job ad",
+		sprintf( error_string, "%s is not set in the job ad",
 							  ATTR_GRID_RESOURCE );
 		goto error_exit;
 	}
 
 	myResource = NordugridResource::FindOrCreateResource( resourceManagerString,
-														  jobProxy->subject->subject_name );
+														  jobProxy );
 	myResource->RegisterJob( this );
 
 	buff[0] = '\0';
@@ -255,8 +262,8 @@ NordugridJob::NordugridJob( ClassAd *classad )
 
  error_exit:
 	gmState = GM_HOLD;
-	if ( !error_string.IsEmpty() ) {
-		jobAd->Assign( ATTR_HOLD_REASON, error_string.Value() );
+	if ( !error_string.empty() ) {
+		jobAd->Assign( ATTR_HOLD_REASON, error_string.c_str() );
 	}
 	return;
 }
@@ -264,7 +271,7 @@ NordugridJob::NordugridJob( ClassAd *classad )
 NordugridJob::~NordugridJob()
 {
 	if ( jobProxy != NULL ) {
-		ReleaseProxy( jobProxy, evaluateStateTid );
+		ReleaseProxy( jobProxy, (TimerHandlercpp)&BaseJob::SetEvaluateState, this );
 	}
 	if ( myResource ) {
 		myResource->UnregisterJob( this );
@@ -292,13 +299,14 @@ void NordugridJob::Reconfig()
 	gahp->setTimeout( gahpCallTimeout );
 }
 
-int NordugridJob::doEvaluateState()
+void NordugridJob::doEvaluateState()
 {
 	int old_gm_state;
 	bool reevaluate_state = true;
 	time_t now = time(NULL);
 
-	bool done;
+	bool attr_exists;
+	bool attr_dirty;
 	int rc;
 
 	daemonCore->Reset_Timer( evaluateStateTid, TIMER_NEVER );
@@ -358,40 +366,30 @@ int NordugridJob::doEvaluateState()
 					executeLogged = true;
 				}
 
-				gmState = GM_RECOVER_QUERY;
+				if ( remoteJobState == "" ||
+					 remoteJobState == REMOTE_STATE_ACCEPTING ||
+					 remoteJobState == REMOTE_STATE_ACCEPTED ||
+					 remoteJobState == REMOTE_STATE_PREPARING ) {
+					gmState = GM_RECOVER_QUERY;
+				} else {
+					gmState = GM_SUBMITTED;
+				}
 			}
 			} break;
 		case GM_RECOVER_QUERY: {
 			if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_CANCEL;
 			} else {
-				char *new_status = NULL;
-				rc = gahp->nordugrid_status( resourceManagerString,
-											 remoteJobId, new_status );
-				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
-					 rc == GAHPCLIENT_COMMAND_PENDING ) {
-					break;
-				} else if ( rc != 0 ) {
-					// What to do about failure?
-					errorString = gahp->getErrorString();
-					dprintf( D_ALWAYS, "(%d.%d) job probe failed: %s\n",
-							 procID.cluster, procID.proc,
-							 errorString.Value() );
+				if ( m_lastRemoteStatusUpdate > enteredCurrentGmState ) {
+					if ( remoteJobState == REMOTE_STATE_ACCEPTING ||
+						 remoteJobState == REMOTE_STATE_ACCEPTED ||
+						 remoteJobState == REMOTE_STATE_PREPARING ) {
+						gmState = GM_STAGE_IN;
+					} else {
+						gmState = GM_SUBMITTED;
+					}
+				} else if ( m_currentStatusUnknown ) {
 					gmState = GM_CANCEL;
-					break;
-				} else {
-					remoteJobState = new_status;
-					SetRemoteJobStatus( new_status );
-				}
-				if ( new_status ) {
-					free( new_status );
-				}
-				lastProbeTime = now;
-				if ( remoteJobState == REMOTE_STATE_ACCEPTED ||
-					 remoteJobState == REMOTE_STATE_PREPARING ) {
-					gmState = GM_STAGE_IN;
-				} else {
-					gmState = GM_SUBMITTED;
 				}
 			}
 			} break;
@@ -438,7 +436,7 @@ int NordugridJob::doEvaluateState()
 				}
 				rc = gahp->nordugrid_submit( 
 										resourceManagerString,
-										RSL->Value(),
+										RSL->c_str(),
 										job_id );
 				if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
 					 rc == GAHPCLIENT_COMMAND_PENDING ) {
@@ -453,13 +451,12 @@ int NordugridJob::doEvaluateState()
 					SetRemoteJobId( job_id );
 					free( job_id );
 					WriteGridSubmitEventToUserLog( jobAd );
-					myResource->SubmitComplete( this );
 					gmState = GM_SUBMIT_SAVE;
 				} else {
 					errorString = gahp->getErrorString();
 					dprintf(D_ALWAYS,"(%d.%d) job submit failed: %s\n",
 							procID.cluster, procID.proc,
-							errorString.Value() );
+							errorString.c_str() );
 					myResource->CancelSubmit( this );
 					gmState = GM_UNSUBMITTED;
 				}
@@ -476,8 +473,9 @@ int NordugridJob::doEvaluateState()
 			if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_CANCEL;
 			} else {
-				done = requestScheddUpdate( this );
-				if ( !done ) {
+				jobAd->GetDirtyFlag( ATTR_GRID_JOB_ID, &attr_exists, &attr_dirty );
+				if ( attr_exists && attr_dirty ) {
+					requestScheddUpdate( this, true );
 					break;
 				}
 				gmState = GM_STAGE_IN;
@@ -502,14 +500,17 @@ int NordugridJob::doEvaluateState()
 			} else if ( rc != 0 ) {
 				errorString = gahp->getErrorString();
 				dprintf( D_ALWAYS, "(%d.%d) file stage in failed: %s\n",
-						 procID.cluster, procID.proc, errorString.Value() );
+						 procID.cluster, procID.proc, errorString.c_str() );
 				gmState = GM_CANCEL;
 			} else {
 				gmState = GM_SUBMITTED;
 			}
 			} break;
 		case GM_SUBMITTED: {
-			if ( remoteJobState == REMOTE_STATE_FINISHED ) {
+			if ( remoteJobState == REMOTE_STATE_FINISHED ||
+				 remoteJobState == REMOTE_STATE_FAILED ||
+				 remoteJobState == REMOTE_STATE_KILLED ||
+				 remoteJobState == REMOTE_STATE_DELETED ) {
 					gmState = GM_EXIT_INFO;
 			} else if ( condorState == REMOVED || condorState == HELD ) {
 				gmState = GM_CANCEL;
@@ -521,6 +522,7 @@ int NordugridJob::doEvaluateState()
 					lastProbeTime = 0;
 					probeNow = false;
 				}
+/*
 				if ( now >= lastProbeTime + probeInterval ) {
 					gmState = GM_PROBE_JOB;
 					break;
@@ -530,6 +532,7 @@ int NordugridJob::doEvaluateState()
 					delay = (lastProbeTime + probeInterval) - now;
 				}				
 				daemonCore->Reset_Timer( evaluateStateTid, delay );
+*/
 			}
 			} break;
 		case GM_PROBE_JOB: {
@@ -547,9 +550,13 @@ int NordugridJob::doEvaluateState()
 					errorString = gahp->getErrorString();
 					dprintf( D_ALWAYS, "(%d.%d) job probe failed: %s\n",
 							 procID.cluster, procID.proc,
-							 errorString.Value() );
+							 errorString.c_str() );
 				} else {
-					remoteJobState = new_status;
+					if ( new_status ) {
+						remoteJobState = new_status;
+					} else {
+						remoteJobState = "";
+					}
 					SetRemoteJobStatus( new_status );
 				}
 				if ( new_status ) {
@@ -560,35 +567,53 @@ int NordugridJob::doEvaluateState()
 			}
 			} break;
 		case GM_EXIT_INFO: {
-			bool normal_exit;
-			int exit_code;
-			float wallclock = 0;
-			float sys_cpu = 0;
-			float user_cpu = 0;
-			rc = gahp->nordugrid_exit_info( resourceManagerString, remoteJobId,
-											normal_exit, exit_code, wallclock,
-											sys_cpu, user_cpu );
+			std::string filter;
+			StringList reply;
+			sprintf( filter, "nordugrid-job-globalid=gsiftp://%s:2811/jobs/%s",
+							resourceManagerString, remoteJobId );
+
+			rc = gahp->nordugrid_ldap_query( resourceManagerString, "mds-vo-name=local,o=grid", filter.c_str(), "nordugrid-job-usedcputime,nordugrid-job-usedwalltime,nordugrid-job-exitcode", reply );
 			if ( rc == GAHPCLIENT_COMMAND_NOT_SUBMITTED ||
 				 rc == GAHPCLIENT_COMMAND_PENDING ) {
 				break;
 			} else if ( rc != 0 ) {
 				errorString = gahp->getErrorString();
 				dprintf( D_ALWAYS, "(%d.%d) exit info gathering failed: %s\n",
-						 procID.cluster, procID.proc, errorString.Value() );
+						 procID.cluster, procID.proc, errorString.c_str() );
 				gmState = GM_CANCEL;
 			} else {
-				normalExit = normal_exit;
-				exitCode = exit_code;
-				if ( normalExit ) {
-					jobAd->Assign( ATTR_ON_EXIT_BY_SIGNAL, false );
-					jobAd->Assign( ATTR_ON_EXIT_CODE, exitCode );
-				} else {
-					jobAd->Assign( ATTR_ON_EXIT_BY_SIGNAL, true );
-					jobAd->Assign( ATTR_ON_EXIT_SIGNAL, exitCode );
+				int exit_code = -1;
+				int wallclock = -1;
+				int cpu = -1;
+				const char *entry;
+				reply.rewind();
+				while ( (entry = reply.next()) ) {
+					if ( !strncmp( entry, "nordugrid-job-usedcputime: ", 27 ) ) {
+						entry = strchr( entry, ' ' ) + 1;
+						cpu = atoi( entry );
+					} else if ( !strncmp( entry, "nordugrid-job-usedwalltime: ", 28 ) ) {
+						entry = strchr( entry, ' ' ) + 1;
+						wallclock = atoi( entry );
+					} else if ( !strncmp( entry, "nordugrid-job-exitcode: ", 24 ) ) {
+						entry = strchr( entry, ' ' ) + 1;
+						exit_code = atoi( entry );
+					}
 				}
-				jobAd->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, wallclock );
-				jobAd->Assign( ATTR_JOB_REMOTE_SYS_CPU, sys_cpu );
-				jobAd->Assign( ATTR_JOB_REMOTE_USER_CPU, user_cpu );
+				if ( exit_code < 0 || wallclock < 0 || cpu < 0 ) {
+					dprintf( D_ALWAYS, "(%d.%d) exit info missing\n",
+							 procID.cluster, procID.proc );
+					gmState = GM_CANCEL;
+					break;
+				}
+				if ( exit_code > 128 ) {
+					jobAd->Assign( ATTR_ON_EXIT_BY_SIGNAL, true );
+					jobAd->Assign( ATTR_ON_EXIT_SIGNAL, exit_code - 128 );
+				} else {
+					jobAd->Assign( ATTR_ON_EXIT_BY_SIGNAL, false );
+					jobAd->Assign( ATTR_ON_EXIT_CODE, exit_code );
+				}
+				jobAd->Assign( ATTR_JOB_REMOTE_WALL_CLOCK, wallclock * 60 );
+				jobAd->Assign( ATTR_JOB_REMOTE_USER_CPU, cpu * 60 );
 				gmState = GM_STAGE_OUT;
 			}
 			} break;
@@ -618,7 +643,7 @@ int NordugridJob::doEvaluateState()
 			} else if ( rc != 0 ) {
 				errorString = gahp->getErrorString();
 				dprintf( D_ALWAYS, "(%d.%d) file stage out failed: %s\n",
-						 procID.cluster, procID.proc, errorString.Value() );
+						 procID.cluster, procID.proc, errorString.c_str() );
 				gmState = GM_CANCEL;
 			} else {
 				gmState = GM_DONE_SAVE;
@@ -628,8 +653,9 @@ int NordugridJob::doEvaluateState()
 			if ( condorState != HELD && condorState != REMOVED ) {
 				JobTerminated();
 				if ( condorState == COMPLETED ) {
-					done = requestScheddUpdate( this );
-					if ( !done ) {
+					jobAd->GetDirtyFlag( ATTR_JOB_STATUS, &attr_exists, &attr_dirty );
+					if ( attr_exists && attr_dirty ) {
+						requestScheddUpdate( this, true );
 						break;
 					}
 				}
@@ -644,7 +670,7 @@ int NordugridJob::doEvaluateState()
 			} else if ( rc != 0 ) {
 				errorString = gahp->getErrorString();
 				dprintf( D_ALWAYS, "(%d.%d) job cleanup failed: %s\n",
-						 procID.cluster, procID.proc, errorString.Value() );
+						 procID.cluster, procID.proc, errorString.c_str() );
 				gmState = GM_HOLD;
 				break;
 			}
@@ -671,7 +697,7 @@ int NordugridJob::doEvaluateState()
 				// What to do about a failed cancel?
 				errorString = gahp->getErrorString();
 				dprintf( D_ALWAYS, "(%d.%d) job cancel failed: %s\n",
-						 procID.cluster, procID.proc, errorString.Value() );
+						 procID.cluster, procID.proc, errorString.c_str() );
 				gmState = GM_FAILED;
 			}
 			} break;
@@ -771,8 +797,11 @@ int NordugridJob::doEvaluateState()
 			// through. However, since we registered update events the
 			// first time, requestScheddUpdate won't return done until
 			// they've been committed to the schedd.
-			done = requestScheddUpdate( this );
-			if ( !done ) {
+			const char *name;
+			ExprTree *expr;
+			jobAd->ResetExpr();
+			if ( jobAd->NextDirtyExpr(name, expr) ) {
+				requestScheddUpdate( this, true );
 				break;
 			}
 			if ( remoteJobState != "" ) {
@@ -802,7 +831,7 @@ int NordugridJob::doEvaluateState()
 				jobAd->LookupString( ATTR_HOLD_REASON, holdReason,
 									 sizeof(holdReason) - 1 );
 				if ( holdReason[0] == '\0' && errorString != "" ) {
-					strncpy( holdReason, errorString.Value(),
+					strncpy( holdReason, errorString.c_str(),
 							 sizeof(holdReason) - 1 );
 				}
 				if ( holdReason[0] == '\0' ) {
@@ -843,8 +872,6 @@ int NordugridJob::doEvaluateState()
 		}
 
 	} while ( reevaluate_state );
-
-	return TRUE;
 }
 
 BaseResource *NordugridJob::GetResource()
@@ -861,25 +888,24 @@ void NordugridJob::SetRemoteJobId( const char *job_id )
 		remoteJobId = NULL;
 	}
 
-	MyString full_job_id;
+	std::string full_job_id;
 	if ( job_id ) {
-		full_job_id.sprintf( "nordugrid %s %s", resourceManagerString,
+		sprintf( full_job_id, "nordugrid %s %s", resourceManagerString,
 							 job_id );
 	}
-	BaseJob::SetRemoteJobId( full_job_id.Value() );
+	BaseJob::SetRemoteJobId( full_job_id.c_str() );
 }
 
-MyString *NordugridJob::buildSubmitRSL()
+std::string *NordugridJob::buildSubmitRSL()
 {
 	int transfer_exec = TRUE;
-	MyString *rsl = new MyString;
-	MyString buff;
+	std::string *rsl = new std::string;
 	StringList *stage_list = NULL;
 	StringList *stage_local_list = NULL;
 	char *attr_value = NULL;
-	MyString rsl_suffix;
-	MyString iwd;
-	MyString executable;
+	std::string rsl_suffix;
+	std::string iwd;
+	std::string executable;
 
 	if ( jobAd->LookupString( ATTR_NORDUGRID_RSL, rsl_suffix ) &&
 						   rsl_suffix[0] == '&' ) {
@@ -895,7 +921,7 @@ MyString *NordugridJob::buildSubmitRSL()
 
 	//Start off the RSL
 	attr_value = param( "FULL_HOSTNAME" );
-	rsl->sprintf( "&(savestate=yes)(action=request)(hostname=%s)", attr_value );
+	sprintf( *rsl, "&(savestate=yes)(action=request)(hostname=%s)", attr_value );
 	free( attr_value );
 	attr_value = NULL;
 
@@ -907,7 +933,7 @@ MyString *NordugridJob::buildSubmitRSL()
 	// If we're transferring the executable, strip off the path for the
 	// remote machine, since it refers to the submit machine.
 	if ( transfer_exec ) {
-		*rsl += condor_basename( executable.Value() );
+		*rsl += condor_basename( executable.c_str() );
 	} else {
 		*rsl += executable;
 	}
@@ -919,7 +945,7 @@ MyString *NordugridJob::buildSubmitRSL()
 		if(!args.AppendArgsFromClassAd(jobAd,&arg_errors)) {
 			dprintf(D_ALWAYS,"(%d.%d) Failed to read job arguments: %s\n",
 					procID.cluster, procID.proc, arg_errors.Value());
-			errorString.sprintf("Failed to read job arguments: %s\n",
+			sprintf(errorString,"Failed to read job arguments: %s\n",
 					arg_errors.Value());
 			delete rsl;
 			return NULL;
@@ -931,7 +957,7 @@ MyString *NordugridJob::buildSubmitRSL()
 					dprintf(D_ALWAYS,
 							"(%d.%d) Failed to get job arguments: %s\n",
 							procID.cluster,procID.proc,arg_errors.Value());
-					errorString.sprintf("Failed to get job arguments: %s\n",
+					sprintf(errorString,"Failed to get job arguments: %s\n",
 							arg_errors.Value());
 					delete rsl;
 					return NULL;
@@ -955,7 +981,7 @@ MyString *NordugridJob::buildSubmitRSL()
 	// execute bit on the transferred executable.
 	if ( transfer_exec ) {
 		*rsl += ")(executables=";
-		*rsl += condor_basename( executable.Value() );
+		*rsl += condor_basename( executable.c_str() );
 	}
 
 	if ( jobAd->LookupString( ATTR_JOB_INPUT, &attr_value ) == 1) {
@@ -980,7 +1006,7 @@ MyString *NordugridJob::buildSubmitRSL()
 			*rsl += "(";
 			*rsl += condor_basename(file);
 			if ( IsUrl( file ) ) {
-				rsl->sprintf_cat( " \"%s\")", file );
+				sprintf_cat( *rsl, " \"%s\")", file );
 			} else {
 				*rsl += " \"\")";
 			}
@@ -1023,7 +1049,7 @@ MyString *NordugridJob::buildSubmitRSL()
 			*rsl += "(";
 			*rsl += condor_basename(file);
 			if ( IsUrl( local_file ) ) {
-				rsl->sprintf_cat( " \"%s\")", local_file );
+				sprintf_cat( *rsl, " \"%s\")", local_file );
 			} else {
 				*rsl += " \"\")";
 			}
@@ -1037,11 +1063,11 @@ MyString *NordugridJob::buildSubmitRSL()
 
 	*rsl += ')';
 
-	if ( !rsl_suffix.IsEmpty() ) {
+	if ( !rsl_suffix.empty() ) {
 		*rsl += rsl_suffix;
 	}
 
-dprintf(D_FULLDEBUG,"*** RSL='%s'\n",rsl->Value());
+dprintf(D_FULLDEBUG,"*** RSL='%s'\n",rsl->c_str());
 	return rsl;
 }
 
@@ -1050,24 +1076,24 @@ StringList *NordugridJob::buildStageInList()
 	StringList *tmp_list = NULL;
 	StringList *stage_list = NULL;
 	char *filename = NULL;
-	MyString buf;
-	MyString iwd;
+	std::string buf;
+	std::string iwd;
 	int transfer = TRUE;
 
 	if ( jobAd->LookupString( ATTR_JOB_IWD, iwd ) ) {
-		if ( iwd.Length() > 1 && iwd[iwd.Length() - 1] != '/' ) {
+		if ( iwd.length() > 1 && iwd[iwd.length() - 1] != '/' ) {
 			iwd += '/';
 		}
 	}
 
 	jobAd->LookupString( ATTR_TRANSFER_INPUT_FILES, buf );
-	tmp_list = new StringList( buf.Value(), "," );
+	tmp_list = new StringList( buf.c_str(), "," );
 
 	jobAd->LookupBool( ATTR_TRANSFER_EXECUTABLE, transfer );
 	if ( transfer ) {
 		jobAd->LookupString( ATTR_JOB_CMD, buf );
-		if ( !tmp_list->file_contains( buf.Value() ) ) {
-			tmp_list->append( buf.Value() );
+		if ( !tmp_list->file_contains( buf.c_str() ) ) {
+			tmp_list->append( buf.c_str() );
 		}
 	}
 
@@ -1075,9 +1101,9 @@ StringList *NordugridJob::buildStageInList()
 	jobAd->LookupBool( ATTR_TRANSFER_INPUT, transfer );
 	if ( transfer && jobAd->LookupString( ATTR_JOB_INPUT, buf ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
-		if ( ! nullFile(buf.Value()) ) {
-			if ( !tmp_list->file_contains( buf.Value() ) ) {
-				tmp_list->append( buf.Value() );
+		if ( ! nullFile(buf.c_str()) ) {
+			if ( !tmp_list->file_contains( buf.c_str() ) ) {
+				tmp_list->append( buf.c_str() );
 			}
 		}
 	}
@@ -1087,11 +1113,11 @@ StringList *NordugridJob::buildStageInList()
 	tmp_list->rewind();
 	while ( ( filename = tmp_list->next() ) ) {
 		if ( filename[0] == '/' || IsUrl( filename ) ) {
-			buf.sprintf( "%s", filename );
+			sprintf( buf, "%s", filename );
 		} else {
-			buf.sprintf( "%s%s", iwd.Value(), filename );
+			sprintf( buf, "%s%s", iwd.c_str(), filename );
 		}
-		stage_list->append( buf.Value() );
+		stage_list->append( buf.c_str() );
 	}
 
 	delete tmp_list;
@@ -1102,23 +1128,16 @@ StringList *NordugridJob::buildStageInList()
 StringList *NordugridJob::buildStageOutList()
 {
 	StringList *stage_list = NULL;
-	MyString buf;
-	MyString iwd = "/";
+	std::string buf;
 	bool transfer = TRUE;
 
-	if ( jobAd->LookupString( ATTR_JOB_IWD, iwd ) ) {
-		if ( iwd.Length() > 1 && iwd[iwd.Length() - 1] != '/' ) {
-			iwd += '/';
-		}
-	}
-
 	jobAd->LookupString( ATTR_TRANSFER_OUTPUT_FILES, buf );
-	stage_list = new StringList( buf.Value(), "," );
+	stage_list = new StringList( buf.c_str(), "," );
 
 	jobAd->LookupBool( ATTR_TRANSFER_OUTPUT, transfer );
 	if ( transfer && jobAd->LookupString( ATTR_JOB_OUTPUT, buf ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
-		if ( ! nullFile(buf.Value()) ) {
+		if ( ! nullFile(buf.c_str()) ) {
 			if ( !stage_list->file_contains( REMOTE_STDOUT_NAME ) ) {
 				stage_list->append( REMOTE_STDOUT_NAME );
 			}
@@ -1129,7 +1148,7 @@ StringList *NordugridJob::buildStageOutList()
 	jobAd->LookupBool( ATTR_TRANSFER_ERROR, transfer );
 	if ( transfer && jobAd->LookupString( ATTR_JOB_ERROR, buf ) == 1) {
 		// only add to list if not NULL_FILE (i.e. /dev/null)
-		if ( ! nullFile(buf.Value()) ) {
+		if ( ! nullFile(buf.c_str()) ) {
 			if ( !stage_list->file_contains( REMOTE_STDERR_NAME ) ) {
 				stage_list->append( REMOTE_STDERR_NAME );
 			}
@@ -1145,13 +1164,13 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 	char *remaps = NULL;
 	MyString local_name;
 	char *remote_name;
-	MyString stdout_name = "";
-	MyString stderr_name = "";
-	MyString buff;
-	MyString iwd = "/";
+	std::string stdout_name = "";
+	std::string stderr_name = "";
+	std::string buff;
+	std::string iwd = "/";
 
 	if ( jobAd->LookupString( ATTR_JOB_IWD, iwd ) ) {
-		if ( iwd.Length() > 1 && iwd[iwd.Length() - 1] != '/' ) {
+		if ( iwd.length() > 1 && iwd[iwd.length() - 1] != '/' ) {
 			iwd += '/';
 		}
 	}
@@ -1168,9 +1187,9 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 			// stdout and stderr don't get remapped, and their paths
 			// are evaluated locally
 		if ( strcmp( REMOTE_STDOUT_NAME, remote_name ) == 0 ) {
-			local_name = stdout_name.Value();
+			local_name = stdout_name.c_str();
 		} else if ( strcmp( REMOTE_STDERR_NAME, remote_name ) == 0 ) {
-			local_name = stderr_name.Value();
+			local_name = stderr_name.c_str();
 		} else if( remaps && filename_remap_find( remaps, remote_name,
 												  local_name ) ) {
 				// file is remapped
@@ -1182,9 +1201,9 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 			 || IsUrl( local_name.Value() ) ) {
 			buff = local_name;
 		} else {
-			buff.sprintf( "%s%s", iwd.Value(), local_name.Value() );
+			sprintf( buff, "%s%s", iwd.c_str(), local_name.Value() );
 		}
-		stage_local_list->append( buff.Value() );
+		stage_local_list->append( buff.c_str() );
 	}
 
 	if ( remaps ) {
@@ -1193,3 +1212,28 @@ StringList *NordugridJob::buildStageOutLocalList( StringList *stage_list )
 
 	return stage_local_list;
 }
+
+void NordugridJob::NotifyNewRemoteStatus( const char *status )
+{
+	if ( SetRemoteJobStatus( status ) ) {
+		remoteJobState = status;
+		SetEvaluateState();
+
+		if ( condorState == IDLE &&
+			 ( remoteJobState == REMOTE_STATE_INLRMS_R ||
+			   remoteJobState == REMOTE_STATE_INLRMS_E ||
+			   remoteJobState == REMOTE_STATE_EXECUTED ||
+			   remoteJobState == REMOTE_STATE_FINISHING ||
+			   remoteJobState == REMOTE_STATE_FINISHED ||
+			   remoteJobState == REMOTE_STATE_FAILED ) ) {
+			JobRunning();
+		} else if ( condorState == RUNNING &&
+					( remoteJobState == REMOTE_STATE_INLRMS_Q ||
+					  remoteJobState == REMOTE_STATE_INLRMS_S ) ) {
+			JobIdle();
+		}
+	}
+	if ( gmState == GM_RECOVER_QUERY ) {
+		SetEvaluateState();
+	}
+}
