@@ -71,16 +71,30 @@
 #define D_UNUSED4       (1<<30)
 #define D_NOHEADER      (1<<31)
 #define D_ALL           (~(0) & (~(D_NOHEADER)))
-
-#if defined(__cplusplus)
+#ifdef __cplusplus
+#include <string>
+#include <map>
+#include "param_functions.h"
 extern "C" {
 #endif
 
+#if _MSC_VER >= 1400 /* VC++ 2005 version */
+#define PREFAST_NORETURN __declspec(noreturn)
+# ifdef _PREFAST_
+// inform MSVC_ANALYZE of some true thing that it's having trouble figuring out for itself.
+#  define PREFAST_ASSUME(x) __analysis_assume(!!(x))
+# else
+#  define PREFAST_ASSUME(x)
+# endif
+#else
+#define PREFAST_NORETURN
+#define PREFAST_ASSUME(x)
+#endif
+
 extern int DebugFlags;	/* Bits to look for in dprintf */
 extern int Termlog;		/* Are we logging to a terminal? */
 extern int DebugShouldLockToAppend; /* Should we lock the file before each write? */
 
-
 /* DebugId is a function that may be registered to be called to insert text
  * into the header of a line that is about to be logged to the debug log file.
  * It should treat its arguments similarly to how sprintf_realloc() does.
@@ -89,13 +103,15 @@ extern int DebugShouldLockToAppend; /* Should we lock the file before each write
 extern int (*DebugId)(char **buf,int *bufpos,int *buflen);
 
 void dprintf ( int flags, const char *fmt, ... ) CHECK_PRINTF_FORMAT(2,3);
-
-void dprintf_config( const char *subsys );
+#ifdef __cplusplus
+void dprintf_config( const char *subsys, param_functions * p_funcs = NULL );
+#endif
 void _condor_dprintf_va ( int flags, const char* fmt, va_list args );
 int _condor_open_lock_file(const char *filename,int flags, mode_t perm);
-void _EXCEPT_ ( const char *fmt, ... ) CHECK_PRINTF_FORMAT(1,2);
+void PREFAST_NORETURN _EXCEPT_ ( const char *fmt, ... ) CHECK_PRINTF_FORMAT(1,2);
 void Suicide(void);
 void set_debug_flags( const char *strflags );
+void PREFAST_NORETURN _condor_dprintf_exit( int error_code, const char* msg );
 void _condor_fd_panic( int line, const char *file );
 void _condor_set_debug_flags( const char *strflags );
 
@@ -128,6 +144,10 @@ void dprintf_reset_lock_delay(void);
    start of program or last call to dprintf_reset_lock_delay */
 double dprintf_get_lock_delay(void);
 
+/* get a count of dprintf messages written (for statistics)
+*/
+int dprintf_getCount(void);
+
 /* wrapper for fclose() that soaks up EINTRs up to maxRetries number of times.
  */
 int fclose_wrapper( FILE *stream, int maxRetries );
@@ -156,12 +176,27 @@ extern int	_EXCEPT_Line;			/* Line number of the exception    */
 extern const char	*_EXCEPT_File;		/* File name of the exception      */
 extern int	_EXCEPT_Errno;			/* errno from most recent system call */
 extern int (*_EXCEPT_Cleanup)(int,int,const char*);	/* Function to call to clean up (or NULL) */
-extern void _EXCEPT_(const char*, ...) CHECK_PRINTF_FORMAT(1,2);
+extern PREFAST_NORETURN void _EXCEPT_(const char*, ...) CHECK_PRINTF_FORMAT(1,2);
 
 #if defined(__cplusplus)
 }
 #endif
 
+#if defined(__cplusplus)
+bool debug_open_fds(std::map<int,bool> &open_fds);
+
+class _condor_auto_save_runtime
+{
+public:
+    _condor_auto_save_runtime(double & store); // save result here
+    ~_condor_auto_save_runtime();
+    double   current_runtime();
+    double & runtime;
+    double   begin;
+};
+
+#endif // defined(__cplusplus)
+
 #ifndef CONDOR_ASSERT
 #define CONDOR_ASSERT(cond) \
 	if( !(cond) ) { EXCEPT("Assertion ERROR on (%s)",#cond); }
