@@ -73,5 +73,9 @@ const int UTIL_ERR_CLOSE_FILE = 9002;
 const int UTIL_ERR_GET_CWD = 9003;
 const int UTIL_ERR_LOG_FILE = 9004;
 
+const int DRAINING_ALREADY_IN_PROGRESS    = 10001;
+const int DRAINING_NO_MATCHING_REQUEST_ID = 10002;
+const int DRAINING_CHECK_EXPR_FAILED      = 10003;
+
 #endif
 
