@@ -26,6 +26,7 @@
 #include "stream.h"
 #include "CondorError.h"
 #include "condor_perms.h"
+#include "condor_sockaddr.h"
 
 // retry failed connects for CONNECT_TIMEOUT seconds
 #define CONNECT_TIMEOUT 10
@@ -185,6 +186,10 @@ public:
 	*/
 	bool readReady();
 
+	/**	@return true if a complete message is ready to be read
+	*/
+	virtual bool msgReady() = 0;
+
         //------------------------------------------
         // Encryption support below
         //------------------------------------------
@@ -250,24 +255,21 @@ public:
 	*/
 
     /// peer's port and IP address in a struct sockaddr_in.
-	struct sockaddr_in *peer_addr();
+	condor_sockaddr peer_addr();
 
-	/// peer's port number
+	/// peer's port number 
 	int peer_port();
 
 	/// peer's IP address, string verison (e.g. "128.105.101.17")
 	const char* peer_ip_str();
 
 	/// peer's IP address, integer version (e.g. 2154390801)
-	unsigned int peer_ip_int();
 
 	/// is peer a local interface, aka did this connection originate from a local process?
 	bool peer_is_local();
 
-    /// my port and IP address in a struct sockaddr_in
-    /// @args: the address is returned via 'sin'
-    /// @ret: 0 if succeed, -1 if failed
-    int my_addr(struct sockaddr_in *sin);
+    /// my port and IP address in a class condor_sockaddr
+	condor_sockaddr my_addr();
 
 	/// my IP address, string version (e.g. "128.105.101.17")
 	virtual const char* my_ip_str();
@@ -275,9 +277,6 @@ public:
 	/// local port number
 	int get_port();
 
-	/// local ip address integer
-	unsigned int get_ip_int();
-
     /// sinful address of mypoint() in the form of "<a.b.c.d:pppp>"
     char * get_sinful();
 
@@ -346,11 +345,18 @@ public:
 		/// authenticated
 	bool isAuthenticated() const;
 
+    ///
+	virtual int authenticate(const char * auth_methods, CondorError* errstack, int timeout);
+    ///
+	// method_used should be freed by the caller when finished with it
+	virtual int authenticate(KeyInfo *&ki, const char * auth_methods, CondorError* errstack, int timeout, char **method_used=NULL);
 
 	/// if we are connecting, merges together Stream::get_deadline
 	/// and connect_timeout_time()
 	virtual time_t get_deadline();
 
+	void invalidateSock();
+
 
 //	PRIVATE INTERFACE TO ALL SOCKS
 //
@@ -424,11 +430,6 @@ protected:
 	virtual int hdr_encrypt();
 	///
 	virtual bool is_hdr_encrypt();
-    ///
-	virtual int authenticate(const char * auth_methods, CondorError* errstack, int timeout);
-    ///
-	// method_used should be freed by the caller when finished with it
-	virtual int authenticate(KeyInfo *&ki, const char * auth_methods, CondorError* errstack, int timeout, char **method_used=NULL);
     ///
 	virtual bool is_encrypt();
 #ifdef WIN32
@@ -446,7 +447,6 @@ protected:
 	///
 	int move_descriptor_up();
 
-
     /// called whenever the bound or connected state changes
     void addr_changed();
 
@@ -469,7 +469,7 @@ protected:
 	SOCKET			_sock;
 	sock_state		_state;
 	int				_timeout;
-	struct sockaddr_in _who;	// endpoint of "connection"
+	condor_sockaddr			_who;	// endpoint of "connection"
 	char *			m_connect_addr;
 	char *          _fqu;
 	char *          _fqu_user_part;
@@ -485,6 +485,8 @@ protected:
 	CONDOR_MD_MODE      mdMode_;        // MAC mode
 	KeyInfo           * mdKey_;
 
+	static bool guess_address_string(char const* host, int port, condor_sockaddr& addr);
+
 private:
 	bool initialize_crypto(KeyInfo * key);
         //------------------------------------------
@@ -494,7 +496,7 @@ private:
         //------------------------------------------
 
 	int _condor_read(SOCKET fd, char *buf, int sz, int timeout);
-	int _condor_write(SOCKET fd, char *buf, int sz, int timeout);
+	int _condor_write(SOCKET fd, const char *buf, int sz, int timeout);
 	int bindWithin(const int low, const int high, bool outbound);
 	///
 	// Buffer to hold the string version of our peer's IP address. 
@@ -573,12 +575,13 @@ private:
 	void cancel_connect();
 
 	/**
-	   Private helper that sees if we're GCB enabled, if we're doing
-	   an outbound connection, and if so, uses GCB_local_bind() to
-	   avoid pounding the GCB broker for all outbound connections.
+	   Private helper that sees if we're CCB enabled, if we're doing
+	   an outbound connection, and if so, uses CCB_local_bind() to
+	   avoid pounding the CCB broker for all outbound connections.
 	*/
-	int _bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
-		SOCKET_LENGTH_TYPE len, bool outbound, bool loopback);
+	//int _bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
+	//	SOCKET_LENGTH_TYPE len, bool outbound, bool loopback);
+	int _bind_helper(int fd, const condor_sockaddr& addr, bool outbound, bool loopback);
 };
 
 #endif /* SOCK_H */
