@@ -24,6 +24,7 @@
 #include "condor_debug.h"
 #include "internet.h"
 #include "condor_md.h"       // Condor_MD_MAC
+#include "condor_sockfunc.h"
 
 #define USABLE_PACKET_SIZE SAFE_MSG_FRAGMENT_SIZE - SAFE_MSG_HEADER_SIZE
 const char THIS_IS_TOO_UGLY_FOR_THE_SAKE_OF_BACKWARD[] = "CRAP";
@@ -679,7 +680,7 @@ int _condorOutMsg::putn(const char *dta, const int size) {
  *	          -1, if fails
  */
 int _condorOutMsg::sendMsg(const int sock,
-                           const struct sockaddr* who,
+                           const condor_sockaddr& who,
                            _condorMsgID msgID,
                            unsigned char * mac)
 {
@@ -698,9 +699,11 @@ int _condorOutMsg::sendMsg(const int sock,
 		tempPkt->makeHeader(false, seqNo++, msgID, md);
 		msgLen    += tempPkt->length;
 
-		sent = sendto(sock, tempPkt->dataGram,
+		
+		sent = condor_sendto(sock, tempPkt->dataGram,
 		              tempPkt->length + SAFE_MSG_HEADER_SIZE,
-                      0, who, sizeof(struct sockaddr));
+                      0, who);
+
 		if(sent != tempPkt->length + SAFE_MSG_HEADER_SIZE) {
 			dprintf(D_ALWAYS, "sendMsg:sendto failed - errno: %d\n", errno);
 			headPacket = tempPkt;
@@ -716,7 +719,7 @@ int _condorOutMsg::sendMsg(const int sock,
 
 		dprintf( D_NETWORK, "SEND [%d] %s ", sent, sock_to_string(sock) );
 		dprintf( D_NETWORK|D_NOHEADER, "%s\n",
-				 sin_to_string((sockaddr_in *)who) );
+				 who.to_sinful().Value());
 		total += sent;
 		delete tempPkt;
         md = 0;
@@ -732,8 +735,8 @@ int _condorOutMsg::sendMsg(const int sock,
 			// versions of Condor.  The crypto header may still
 			// be there, since that is in the buffer starting at
 			// the position pointed to by "data".
-		sent = sendto(sock, lastPacket->data, lastPacket->length,
-		              0, who, sizeof(struct sockaddr));
+		sent = condor_sendto(sock, lastPacket->data, lastPacket->length,
+							 0, who);
 		if(sent != lastPacket->length) {
 			dprintf( D_ALWAYS, 
 				 "SafeMsg: sending small msg failed. errno: %d\n",
@@ -747,15 +750,15 @@ int _condorOutMsg::sendMsg(const int sock,
         //}
         //dprintf(D_NETWORK, "--->packet [%d bytes]: %s\n", sent, str);
 		dprintf( D_NETWORK, "SEND [%d] %s ", sent, sock_to_string(sock) );
-		dprintf( D_NETWORK|D_NOHEADER, "%s\n", sin_to_string((sockaddr_in *)who) );
+		dprintf( D_NETWORK|D_NOHEADER, "%s\n", who.to_sinful().Value());
 		total = sent;
     }
     else {
         lastPacket->makeHeader(true, seqNo, msgID, md);
         msgLen += lastPacket->length;
-        sent = sendto(sock, lastPacket->dataGram,
+        sent = condor_sendto(sock, lastPacket->dataGram,
                       lastPacket->length + SAFE_MSG_HEADER_SIZE,
-                      0, who, sizeof(struct sockaddr));
+                      0, who);
         if(sent != lastPacket->length + SAFE_MSG_HEADER_SIZE) {
             dprintf( D_ALWAYS, "SafeMsg: sending last packet failed. errno: %d\n", errno );
             headPacket->reset();
@@ -767,7 +770,7 @@ int _condorOutMsg::sendMsg(const int sock,
         //}
         //dprintf(D_NETWORK, "--->packet [%d bytes]: %s\n", sent, str);
         dprintf( D_NETWORK, "SEND [%d] %s ", sent, sock_to_string(sock) );
-        dprintf( D_NETWORK|D_NOHEADER, "%s\n", sin_to_string((sockaddr_in *)who) );
+        dprintf( D_NETWORK|D_NOHEADER, "%s\n", who.to_sinful().Value());
         total += sent;
     }
 
@@ -1166,7 +1169,7 @@ int _condorInMsg::getPtr(void *&buf, char delim)
 		n += msgbufsize;
 		tempPkt++;
 		tempData = 0;
-		if(tempPkt == SAFE_MSG_NO_OF_DIR_ENTRY) {
+		if(tempPkt >= SAFE_MSG_NO_OF_DIR_ENTRY) {
 			if(!tempDir->nextDir) {
 				return -1;
 			}
@@ -1175,15 +1178,13 @@ int _condorInMsg::getPtr(void *&buf, char delim)
 		} else if(!tempDir->dEntry[tempPkt].dGram) { // was the last packet
 			if( D_FULLDEBUG & DebugFlags )
 				dprintf(D_NETWORK,
-				        "\nSafeMsg::getPtr: get to end & '%c'\
-					  not found\n", delim);
+				        "SafeMsg::getPtr: get to end & '%c' not found\n", delim);
 			return -1;
 		}
 	}
 
 	if( D_FULLDEBUG & DebugFlags )
-		dprintf(D_NETWORK, "SafeMsg::_longMsg::getPtr:\
-		                    found delim = %c & length = %lu\n",
+		dprintf(D_NETWORK, "SafeMsg::_longMsg::getPtr: found delim = %c & length = %lu\n",
 			  delim, (unsigned long)n);
 	if( n > tempBufLen ) {
 		if(tempBuf) {
@@ -1281,7 +1282,7 @@ void _condorInMsg::dumpMsg()
     sprintf(str, "ID: %s, %d, %lu, %d\n",
             inet_ntoa(in), msgID.pid, msgID.time, msgID.msgNo);
     sprintf(&str[strlen(str)], "len:%lu, lastNo:%d, rcved:%d, lastTime:%lu\n",
-            msgLen, lastNo, received, lastTime);
+            msgLen, lastNo, received, (long)lastTime);
     dprintf(D_NETWORK, "========================\n%s\n===================\n", str);
 
     /*
