@@ -25,7 +25,7 @@
 #include "sock.h"
 #include "condor_network.h"
 #include "internet.h"
-#include "my_hostname.h"
+#include "ipv6_hostname.h"
 #include "condor_debug.h"
 #include "condor_socket_types.h"
 #include "get_port_range.h"
@@ -33,10 +33,8 @@
 #include "daemon_core_sock_adapter.h"
 #include "selector.h"
 #include "authentication.h"
-
-#if HAVE_EXT_GCB
-#include "GCB.h"
-#endif
+#include "condor_sockfunc.h"
+#include "condor_ipv6.h"
 
 #ifdef HAVE_EXT_OPENSSL
 #include "condor_crypt_blowfish.h"
@@ -70,7 +68,7 @@ Sock::Sock() : Stream() {
 	connect_state.host = NULL;
 	connect_state.port = 0;
 	connect_state.connect_failure_reason = NULL;
-	memset(&_who, 0, sizeof(struct sockaddr_in));
+	_who.clear();
 
     crypto_ = NULL;
     mdMode_ = MD_OFF;
@@ -102,7 +100,7 @@ Sock::Sock(const Sock & orig) : Stream() {
 	connect_state.host = NULL;
 	connect_state.port = 0;
 	connect_state.connect_failure_reason = NULL;
-	memset( &_who, 0, sizeof( struct sockaddr_in ) );
+	_who.clear();
 
     crypto_ = NULL;
     mdMode_ = MD_OFF;
@@ -377,17 +375,17 @@ int Sock::move_descriptor_up()
 
 int Sock::assign(SOCKET sockd)
 {
-	int my_type = SOCK_DGRAM;
+	int		my_type = SOCK_DGRAM;
 
 	if (_state != sock_virgin) return FALSE;
 
 	if (sockd != INVALID_SOCKET){
 		_sock = sockd;		/* Could we check for correct protocol ? */
+		/* should we check that sockd matches to IPv6 mode? */
 		_state = sock_assigned;
 
-		memset(&_who, 0, sizeof(struct sockaddr_in));
-		SOCKET_LENGTH_TYPE addrlen = sizeof(_who);
-		getpeername(_sock,(struct sockaddr *)&_who, (socklen_t*)&addrlen);
+		_who.clear();
+		condor_getpeername(_sock, _who);
 
 		if ( _timeout > 0 ) {
 			timeout_no_timeout_multiplier( _timeout );
@@ -395,6 +393,12 @@ int Sock::assign(SOCKET sockd)
 		return TRUE;
 	}
 
+	int af_type;
+	if (_condor_is_ipv6_mode())
+		af_type = AF_INET6;
+	else
+		af_type = AF_INET;
+
 	switch(type()){
 		case safe_sock:
 			my_type = SOCK_DGRAM;
@@ -409,7 +413,7 @@ int Sock::assign(SOCKET sockd)
 #ifndef WIN32 /* Unix */
 	errno = 0;
 #endif
-	if ((_sock = socket(AF_INET, my_type, 0)) == INVALID_SOCKET) {
+	if ((_sock = socket(af_type, my_type, 0)) == INVALID_SOCKET) {
 #ifndef WIN32 /* Unix... */
 		if ( errno == EMFILE ) {
 			_condor_fd_panic( __LINE__, __FILE__ ); /* Calls dprintf_exit! */
@@ -455,6 +459,7 @@ int
 Sock::bindWithin(const int low_port, const int high_port, bool outbound)
 {
 	bool bind_all = (bool)_condor_bind_all_interfaces();
+	bool ipv6_mode = _condor_is_ipv6_mode();
 
 	// Use hash function with pid to get the starting point
     struct timeval curTime;
@@ -472,17 +477,24 @@ Sock::bindWithin(const int low_port, const int high_port, bool outbound)
 
 	int this_trial = start_trial;
 	do {
-		sockaddr_in		sin;
+		condor_sockaddr			addr;
 		int bind_return_val;
 
-		memset(&sin, 0, sizeof(sockaddr_in));
-		sin.sin_family = AF_INET;
+		addr.clear();
 		if( bind_all ) {
-			sin.sin_addr.s_addr = INADDR_ANY;
+			if (ipv6_mode)
+				addr.set_ipv6();
+			else
+				addr.set_ipv4();
+			addr.set_addr_any();
 		} else {
-			sin.sin_addr.s_addr = htonl(my_ip_addr());
+			addr = get_local_ipaddr();
+			// what if the socket type does not match?
+			// e.g. addr is ipv6 but ipv6 mode is not turned on?
+			if (addr.is_ipv4() && ipv6_mode)
+				addr.convert_to_ipv6();
 		}
-		sin.sin_port = htons((u_short)this_trial++);
+		addr.set_port((unsigned short)this_trial++);
 
 #ifndef WIN32
 		priv_state old_priv;
@@ -493,9 +505,7 @@ Sock::bindWithin(const int low_port, const int high_port, bool outbound)
 		}
 #endif
 
-		bind_return_val = _bind_helper(_sock, 
-			(SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE)&sin, 
-			sizeof(sockaddr_in), outbound, false);
+		bind_return_val = _bind_helper(_sock, addr, outbound, false);
 
         addr_changed();
 
@@ -531,7 +541,7 @@ Sock::bindWithin(const int low_port, const int high_port, bool outbound)
 
 int Sock::bind(bool outbound, int port, bool loopback)
 {
-	sockaddr_in		sin;
+	condor_sockaddr addr;
 	int bind_return_value;
 
 	// Following lines are added because some functions in condor call
@@ -587,17 +597,21 @@ int Sock::bind(bool outbound, int port, bool loopback)
 		}
 	} else {
 			// Bind to a dynamic port.
-		memset(&sin, 0, sizeof(sockaddr_in));
-		sin.sin_family = AF_INET;
+
+		if (_condor_is_ipv6_mode())
+			addr.set_ipv6();
+		else
+			addr.set_ipv4();
 		if( loopback ) {
-			sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
-		}
-		else if( (bool)_condor_bind_all_interfaces() ) {
-			sin.sin_addr.s_addr = htonl(INADDR_ANY);
+			addr.set_loopback();
+		} else if( (bool)_condor_bind_all_interfaces() ) {
+			addr.set_addr_any();
 		} else {
-			sin.sin_addr.s_addr = htonl(my_ip_addr());
+			addr = get_local_ipaddr();
+			if (addr.is_ipv4() && _condor_is_ipv6_mode())
+				addr.convert_to_ipv6();
 		}
-		sin.sin_port = htons((u_short)port);
+		addr.set_port((unsigned short)port);
 
 #ifndef WIN32
 		priv_state old_priv;
@@ -608,7 +622,7 @@ int Sock::bind(bool outbound, int port, bool loopback)
 		}
 #endif
 
-		bind_return_value = _bind_helper(_sock, (sockaddr *)&sin, sizeof(sockaddr_in), outbound, loopback);
+		bind_return_value = _bind_helper(_sock, addr, outbound, loopback);
 
         addr_changed();
 
@@ -721,40 +735,56 @@ int Sock::setsockopt(int level, int optname, const char* optval, int optlen)
 	return TRUE; 
 }
 
+bool Sock::guess_address_string(char const* host, int port, condor_sockaddr& addr) {
+	dprintf(D_HOSTNAME, "Guess address string for host = %s, port = %d\n",
+			host, port);
+	/* might be in <x.x.x.x:x> notation				*/
+	if (host[0] == '<') {
+		addr.from_sinful(host);
+		dprintf(D_HOSTNAME, "it was sinful string. ip = %s, port = %d\n",
+				addr.to_ip_string().Value(), addr.get_port());
+	}
+	/* try to get a decimal notation 	 			*/
+	else if ( addr.from_ip_string(host) ) {
+			// nothing to do here
+		addr.set_port(port);
+	}
+	/* if dotted notation fails, try host database	*/
+	else{
+		std::vector<condor_sockaddr> addrs;
+		addrs = resolve_hostname(host);
+		if (addrs.empty())
+			return false;
+		addr = addrs.front();
+		addr.set_port(port);
+	}
+	return true;
+}
+
 int Sock::do_connect(
 	char const	*host,
 	int		port,
 	bool	non_blocking_flag
 	)
 {
-	hostent		*hostp;
-	unsigned long	inaddr;
-
 	if (!host || port < 0) return FALSE;
 
-	memset(&_who, 0, sizeof(sockaddr_in));
-	_who.sin_family = AF_INET;
-	_who.sin_port = htons((u_short)port);
+	_who.clear();
+	if (!guess_address_string(host, port, _who)) {
+		return FALSE;
+	}
 
-    int ret;
+	if (_condor_is_ipv6_mode() && _who.is_ipv4())
+		_who.convert_to_ipv6();
 
-	/* might be in <x.x.x.x:x> notation				*/
+		// current code handles sinful string and just hostname differently.
+		// however, why don't we just use sinful string at all?
 	if (host[0] == '<') {
-		string_to_sin(host, &_who);
 		set_connect_addr(host);
 	}
-	/* try to get a decimal notation 	 			*/
-	else if ( (ret = inet_pton(AF_INET, host, &inaddr)) > 0 ) {
-		memcpy((char *)&_who.sin_addr, &inaddr, sizeof(inaddr));
-		set_connect_addr(sin_to_string(&_who));
+	else { // otherwise, just use ip string.
+		set_connect_addr(_who.to_ip_string().Value());
 	}
-	/* if dotted notation fails, try host database	*/
-	else{
-		if ((hostp = condor_gethostbyname(host)) == (hostent *)0) return FALSE;
-		memcpy(&_who.sin_addr, hostp->h_addr, hostp->h_length);
-		set_connect_addr(sin_to_string(&_who));
-	}
-
     addr_changed();
 
 	// now that we have set _who (useful for getting informative
@@ -1124,7 +1154,7 @@ bool Sock::do_connect_tryit()
 	}
 
 
-	if (::connect(_sock, (sockaddr *)&_who, sizeof(sockaddr_in)) == 0) {
+	if (condor_connect(_sock, _who) == 0) {
 		if ( connect_state.non_blocking_flag ) {
 			// Pretend that we haven't connected yet so that there is
 			// only one code path for all non-blocking connects.
@@ -1296,7 +1326,7 @@ int Sock::close()
         free(connect_state.host);
     }
 	connect_state.host = NULL;
-	memset(&_who, 0, sizeof( struct sockaddr_in ) );
+	_who.clear();
     addr_changed();
 	
 	return TRUE;
@@ -1361,6 +1391,10 @@ Sock::readReady() {
 		return false;
 	}
 
+	if( msgReady() ) {
+		return true;
+	}
+
 	selector.add_fd( _sock, Selector::IO_READ );
 	selector.set_timeout( 0 );
 	selector.execute();
@@ -1533,34 +1567,36 @@ char * Sock::serializeCryptoInfo(char * buf)
     // other junk from reli_sock as well. Hence the code below. Hao
     ASSERT(ptmp);
 
-    sscanf(ptmp, "%d*", &encoded_len);
-    if ( encoded_len > 0 ) {
+    int citems = sscanf(ptmp, "%d*", &encoded_len);
+    if ( citems == 1 && encoded_len > 0 ) {
         len = encoded_len/2;
         kserial = (unsigned char *) malloc(len);
+        ASSERT ( kserial )
 
         // skip the *
         ptmp = strchr(ptmp, '*');
-		ASSERT( ptmp );
+        ASSERT( ptmp );
         ptmp++;
 
         // Reading protocol
-        sscanf(ptmp, "%d*", &protocol);
+        citems = sscanf(ptmp, "%d*", &protocol);
         ptmp = strchr(ptmp, '*');
-		ASSERT( ptmp );
+        ASSERT( ptmp && citems == 1 );
         ptmp++;
 
         // read the encryption mode
         int encryption_mode = 0;
-        sscanf(ptmp, "%d*", &encryption_mode);
+        citems = sscanf(ptmp, "%d*", &encryption_mode);
         ptmp = strchr(ptmp, '*');
-        ASSERT( ptmp );
+        ASSERT( ptmp && citems == 1 );
         ptmp++;
 
         // Now, convert from Hex back to binary
         unsigned char * ptr = kserial;
         unsigned int hex;
         for(int i = 0; i < len; i++) {
-            sscanf(ptmp, "%2X", &hex);
+            citems = sscanf(ptmp, "%2X", &hex);
+			if (citems != 1) break;
             *ptr = (unsigned char)hex;
 			ptmp += 2;  // since we just consumed 2 bytes of hex
 			ptr++;      // since we just stored a single byte of binary
@@ -1593,21 +1629,23 @@ char * Sock::serializeMdInfo(char * buf)
     // other junk from reli_sock as well. Hence the code below. Hao
     ASSERT(ptmp);
 
-    sscanf(ptmp, "%d*", &encoded_len);
-    if ( encoded_len > 0 ) {
+    int citems = sscanf(ptmp, "%d*", &encoded_len);
+    if ( 1 == citems && encoded_len > 0 ) {
         len = encoded_len/2;
         kmd = (unsigned char *) malloc(len);
+        ASSERT( kmd );
 
         // skip the *
         ptmp = strchr(ptmp, '*');
-		ASSERT( ptmp );
+        ASSERT( ptmp );
         ptmp++;
 
         // Now, convert from Hex back to binary
         unsigned char * ptr = kmd;
         unsigned int hex;
         for(int i = 0; i < len; i++) {
-            sscanf(ptmp, "%2X", &hex);
+            citems = sscanf(ptmp, "%2X", &hex);
+            if (citems != 1) break;
             *ptr = (unsigned char)hex;
 			ptmp += 2;  // since we just consumed 2 bytes of hex
 			ptr++;      // since we just stored a single byte of binary
@@ -1708,6 +1746,7 @@ char * Sock::serialize(char *buf)
 	ASSERT(verstring);
 	memset(verstring,0,verstring_len+1);
 	strncpy(verstring,buf,verstring_len);
+	verstring[verstring_len] = 0;
 	if( verstring_len ) {
 			// daemoncore does not like spaces in our serialized string
 		char *s;
@@ -1772,36 +1811,47 @@ Sock::addr_changed()
     _sinful_peer_buf[0] = '\0';
 }
 
-struct sockaddr_in *
+condor_sockaddr
 Sock::peer_addr()
 {
-	return &_who;
+	return _who;
 }
 
 
 int
 Sock::peer_port()
 {
-	return (int) ntohs( _who.sin_port );
+		//return (int) ntohs( _who.sin_port );
+	return (int)(_who.get_port());
 }
 
 
+// [OBSOLETE]
+/*
 unsigned int
 Sock::peer_ip_int()
 {
 	return (unsigned int) ntohl( _who.sin_addr.s_addr );
 }
+*/
 
 
 const char *
 Sock::peer_ip_str()
 {
+	if (!_peer_ip_buf[0]) {
+		MyString peer_ip = _who.to_ip_string();
+		strcpy(_peer_ip_buf, peer_ip.Value());
+	}
+	return _peer_ip_buf;
+		/*
     if( _peer_ip_buf[0] ) {
         return _peer_ip_buf;
     }
     strncpy( _peer_ip_buf, inet_ntoa(_who.sin_addr), IP_STRING_BUF_SIZE );
     _peer_ip_buf[IP_STRING_BUF_SIZE-1] = '\0';
 	return _peer_ip_buf;
+		*/
 }
 
 // is peer a local interface, aka did this connection originate from a local process?
@@ -1810,6 +1860,34 @@ Sock::peer_ip_str()
 bool 
 Sock::peer_is_local()
 {
+		// peer_is_local is called rarely and by few call sites.
+		// making hashtable for both ipv4 and ipv6 addresses does seem to
+		// be worth implementation.
+
+	if (!peer_addr().is_valid())
+		return false;
+
+	bool result;
+	condor_sockaddr addr = peer_addr();
+		// ... but use any old ephemeral port.
+	addr.set_port(0);
+	int sock = ::socket(addr.get_aftype(), SOCK_DGRAM, IPPROTO_UDP);
+		// invoke OS bind, not cedar bind - cedar bind does not allow us
+		// to specify the local address.
+	if (condor_bind(sock, addr) < 0) {
+		// failed to bind.  assume we failed  because the peer address is
+		// not local.
+		result = false;
+	} else {
+		// bind worked, assume address has a local interface.
+		result = true;
+	}
+	// must not forget to close the socket we just created!
+	::closesocket(sock);
+	return result;
+	
+		/*
+
 	// Keep a static cache of results, since determining if the peer is local
 	// is somewhat expensive. Flush the cache every 20 min to deal w/
 	// interfaces on the machine being activated/deactivated during runtime.
@@ -1860,7 +1938,8 @@ Sock::peer_is_local()
         return false;
     }
 		// Bind to the *same address* as our peer ....
-	struct sockaddr_in mySockAddr = *( peer_addr() );
+		//struct sockaddr_in mySockAddr = *( peer_addr() );
+	sockaddr_in mySockAddr = peer_addr().to_sin();
 		// ... but use any old ephemeral port.
     mySockAddr.sin_port = htons( 0 );
 		// invoke OS bind, not cedar bind - cedar bind does not allow us
@@ -1883,66 +1962,48 @@ Sock::peer_is_local()
 
 	// return the result to the caller
 	return result;
+		*/
 }
 
-
-// my port and IP address in a struct sockaddr_in
-// @args: the address is returned via 'sin'
-// @ret: 0 if succeed, -1 if failed
-int
-Sock::my_addr( struct sockaddr_in *sin )
+condor_sockaddr
+Sock::my_addr() 
 {
-    struct sockaddr_in *tmp = getSockAddr(_sock);
-    if (tmp == NULL) return -1;
-
-    memcpy(sin, tmp, sizeof(struct sockaddr_in));
-    return 0;
+	condor_sockaddr addr;
+	condor_getsockname_ex(_sock, addr);
+	return addr;
 }
 
 const char *
 Sock::my_ip_str()
 {
-    if( _my_ip_buf[0] ) {
-        return _my_ip_buf;
-    }
-	struct sockaddr_in sin;
-	if(my_addr(&sin) == -1) {
-		return NULL;
+	if (!_my_ip_buf[0]) {
+		MyString ip_str = my_addr().to_ip_string();
+		strcpy(_my_ip_buf, ip_str.Value());
 	}
-    strncpy( _my_ip_buf, inet_ntoa(sin.sin_addr), IP_STRING_BUF_SIZE );
-    _my_ip_buf[IP_STRING_BUF_SIZE-1] = '\0';
 	return _my_ip_buf;
 }
 
 char *
 Sock::get_sinful()
 {       
-    if( _sinful_self_buf[0] ) {
-        return _sinful_self_buf;
+    if( !_sinful_self_buf[0] ) {
+		condor_sockaddr addr;
+		int ret = condor_getsockname_ex(_sock, addr);
+		if (ret == 0) {
+			MyString sinful_self = addr.to_sinful();
+			strcpy(_sinful_self_buf, sinful_self.Value());
     }
-    struct sockaddr_in *tmp = getSockAddr(_sock);
-    if (tmp == NULL) return NULL;
-    char const *s = sin_to_string(tmp);
-	if(!s) {
-		return NULL;
 	}
-	strncpy(_sinful_self_buf,s,SINFUL_STRING_BUF_SIZE);
-    _sinful_self_buf[SINFUL_STRING_BUF_SIZE-1] = '\0';
 	return _sinful_self_buf;
 }
 
 char *
 Sock::get_sinful_peer()
 {       
-    if( _sinful_peer_buf[0] ) {
-        return _sinful_peer_buf;
-    }
-	char const *s = sin_to_string(&_who);
-	if(!s) {
-		return NULL;
+	if ( !_sinful_peer_buf[0] ) {
+		MyString sinful_peer = _who.to_sinful();
+		strcpy(_sinful_peer_buf, sinful_peer.Value());
 	}
-	ASSERT(strlen(s) < sizeof(_sinful_peer_buf));
-	strcpy(_sinful_peer_buf,s);
 	return _sinful_peer_buf;
 }
 
@@ -1959,27 +2020,10 @@ Sock::default_peer_description()
 int
 Sock::get_port()
 {
-	sockaddr_in	addr;
-
-	SOCKET_LENGTH_TYPE addr_len;
-	
-	addr_len = sizeof(sockaddr_in);
-
-	if (getsockname(_sock, (sockaddr *)&addr, &addr_len) < 0) return -1;
-	return (int) ntohs(addr.sin_port);
-}
-
-
-unsigned int 
-Sock::get_ip_int()
-{
-	sockaddr_in	addr;
-	SOCKET_LENGTH_TYPE addr_len;
-
-	addr_len = sizeof(sockaddr_in);
-
-	if (getsockname(_sock, (sockaddr *)&addr, &addr_len) < 0) return 0;
-	return (unsigned int) ntohl(addr.sin_addr.s_addr);
+	condor_sockaddr addr;
+	if (condor_getsockname(_sock, addr) < 0)
+		return -1;
+	return addr.get_port();
 }
 
 #if !defined(WIN32)
@@ -2212,29 +2256,14 @@ bool Sock :: is_encrypt()
 
 
 int
-Sock::_bind_helper(int fd, SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE addr,
-	SOCKET_LENGTH_TYPE len, bool outbound, bool loopback)
+Sock::_bind_helper(int fd, const condor_sockaddr& addr, bool outbound, bool loopback)
 {
 	int rval;
 
-#if HAVE_EXT_GCB
-	if (outbound || loopback) {
-		rval = GCB_local_bind(fd, 
-			/* XXX This evil, evil typecast is here because
-				this codepath only exists on linux. The
-				real way to fix this is to parameterize
-				the functions signatures all the way down to the
-				the Generic_bind() call in the GCB
-				external. */
-			const_cast<struct sockaddr*>(addr),
-			len);
-	}
-	else {
-		rval = ::bind(fd, (SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE)addr, len);
-	}
-#else
-	rval = ::bind(fd, (SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE)addr, len);
-#endif /* HAVE_EXT_GCB */
+	if (outbound) {} // To remove unused variable warning
+	if (loopback) {} // To remove unused variable warning
+		//rval = ::bind(fd, (SOCKET_ADDR_CONST_BIND SOCKET_ADDR_TYPE)addr, len);
+	rval = condor_bind(fd, addr);
 	return rval;
 }
 
@@ -2431,3 +2460,9 @@ Sock::canEncrypt()
 {
 	return crypto_ != NULL;
 }
+
+void
+Sock::invalidateSock()
+{
+	_sock = INVALID_SOCKET; 
+}
