@@ -25,8 +25,6 @@
 #include "HashTable.h"
 #include "RoutedJob.h"
 
-#define WANT_CLASSAD_NAMESPACE
-#undef open
 #include "classad/classad_distribution.h"
 
 #if HAVE_JOB_HOOKS
