@@ -28,8 +28,6 @@
 #include "VanillaToGrid.h"
 #include "filename_tools.h"
 #include "string_list.h"
-#define WANT_CLASSAD_NAMESPACE
-#undef open
 #include "classad/classad_distribution.h"
 
 
@@ -81,8 +79,14 @@ bool VanillaToGrid::vanillaToGrid(classad::ClassAd * ad, int target_universe, co
 	ad->Delete(ATTR_SERVER_TIME);
 	ad->Delete(ATTR_AUTO_CLUSTER_ID);
 	ad->Delete(ATTR_AUTO_CLUSTER_ATTRS);
+	ad->Delete( ATTR_STAGE_IN_FINISH );
+	ad->Delete( ATTR_STAGE_IN_START );
 
-
+	if( is_sandboxed ) {
+		char attr_name[100];
+		snprintf(attr_name,100,"SUBMIT_%s",ATTR_JOB_IWD);
+		ad->Delete(attr_name);  // the presence of this would prevent schedd from rewriting spooled iwd
+	}
 
 	// Stuff to reset
 	ad->InsertAttr(ATTR_JOB_STATUS, 1); // Idle
