@@ -27,9 +27,6 @@
 #include "Scheduler.h"
 #include "JobRouter.h"
 
-// about self
-DECL_SUBSYSTEM("JOB_ROUTER", SUBSYSTEM_TYPE_SCHEDD );	// used by Daemon Core
-
 
 Scheduler *schedd;
 JobRouter *job_router;
@@ -37,7 +34,7 @@ NewClassAdJobLogConsumer *log_consumer;
 
 //-------------------------------------------------------------
 
-void main_init(int argc, char *argv[])
+void main_init(int   /*argc*/, char ** /*argv*/)
 {
 	dprintf(D_ALWAYS, "main_init() called\n");
 
@@ -91,15 +88,14 @@ void main_shutdown_graceful()
 
 //-------------------------------------------------------------
 
-void
-main_pre_dc_init( int argc, char* argv[] )
+int
+main( int argc, char **argv )
 {
-		// dprintf isn't safe yet...
+	set_mySubSystem("JOB_ROUTER", SUBSYSTEM_TYPE_SCHEDD );	// used by Daemon Core
+
+	dc_main_init = main_init;
+	dc_main_config = main_config;
+	dc_main_shutdown_fast = main_shutdown_fast;
+	dc_main_shutdown_graceful = main_shutdown_graceful;
+	return dc_main( argc, argv );
 }
-
-
-void
-main_pre_command_sock_init( )
-{
-}
-
