@@ -25,11 +25,9 @@
 #include "master.h"
 #include "condor_daemon_core.h"
 #include "exit.h"
-#include "my_hostname.h"
 #include "basename.h"
 #include "condor_email.h"
 #include "condor_environ.h"
-#include "condor_parameters.h"
 #include "daemon_list.h"
 #include "sig_name.h"
 #include "internet.h"
@@ -40,6 +38,9 @@
 #include "stat_info.h"
 #include "shared_port_endpoint.h"
 #include "condor_fix_access.h"
+#include "condor_sockaddr.h"
+#include "ipv6_hostname.h"
+#include "setenv.h"
 
 #if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
 #if defined(HAVE_DLOPEN) || defined(WIN32)
@@ -55,6 +56,7 @@ extern int		update_interval;
 extern int		check_new_exec_interval;
 extern int		preen_interval;
 extern int		new_bin_delay;
+extern StopStateT new_bin_restart_mode;
 extern char*	FS_Preen;
 extern			ClassAd* ad;
 extern int		NT_ServiceFlag; // TRUE if running on NT as an NT Service
@@ -102,7 +104,7 @@ extern char*		MasterName;
 ///////////////////////////////////////////////////////////////////////////
 // daemon Class
 ///////////////////////////////////////////////////////////////////////////
-daemon::daemon(char *name, bool is_daemon_core, bool is_h )
+daemon::daemon(const char *name, bool is_daemon_core, bool is_h )
 {
 	char	buf[1000];
 
@@ -219,42 +221,23 @@ int
 daemon::runs_on_this_host()
 {
 	char	*tmp;
-	char	hostname[512];
-	static char	**this_host_addr_list = 0;
-	static int		host_addr_count;
-	struct hostent	*hp;
-	int		i, j;
+	static bool this_host_addr_cached = false;
+	static std::vector<condor_sockaddr> this_host_addr;
 
 
 	if ( flag_in_config_file != NULL ) {
 		if (strncmp(flag_in_config_file, "BOOL_", 5) == MATCH) {
-			tmp	= param( flag_in_config_file);
-			if ( tmp && (*tmp == 't' || *tmp == 'T')) {
-				runs_here = TRUE;
-			} else {
-				runs_here = FALSE;
-			}
+			runs_here =
+				param_boolean_crufty(flag_in_config_file, false) ? TRUE : FALSE;
 		} else {
-			if (this_host_addr_list == 0) {
-				if (condor_gethostname(hostname, sizeof(hostname)) < 0) {
-					EXCEPT( "gethostname(0x%p,%d)", hostname, 
-						   sizeof(hostname));
-				}
-				if( (hp=condor_gethostbyname(hostname)) == 0 ) {
-					EXCEPT( "gethostbyname(%s)", hostname );
-				}
-				for (host_addr_count = 0; hp->h_addr_list[host_addr_count];
-					 host_addr_count++ ) {
-					/* Empty Loop */
-				}
-				this_host_addr_list = (char **) malloc(host_addr_count * 
-													   sizeof (char *));
-				for (i = 0; i < host_addr_count; i++) {
-					this_host_addr_list[i] = (char *) malloc(hp->h_length);
-					memcpy(this_host_addr_list[i], hp->h_addr_list[i],
-						   hp->h_length);
+			if (!this_host_addr_cached) {
+				MyString local_hostname = get_local_hostname();
+				this_host_addr = resolve_hostname(local_hostname);
+				if (!this_host_addr.empty()) {
+					this_host_addr_cached = true;
 				}
 			}
+			
 			/* Get the name of the host on which this daemon should run */
 			tmp = param( flag_in_config_file );
 			if (!tmp) {
@@ -263,15 +246,15 @@ daemon::runs_on_this_host()
 				return FALSE;
 			}
 			runs_here = FALSE;
-			hp = condor_gethostbyname( tmp );
-			if (hp == 0) {
+
+			std::vector<condor_sockaddr> addrs = resolve_hostname(tmp);
+			if (addrs.empty()) {
 				dprintf(D_ALWAYS, "Master couldn't lookup host %s\n", tmp);
 				return FALSE;
 			} 
-			for (i = 0; i < host_addr_count; i++) {
-				for (j = 0; hp->h_addr_list[j]; j++) {
-					if (memcmp(this_host_addr_list[i], hp->h_addr_list[j],
-							   hp->h_length) == MATCH) {
+			for (unsigned i = 0; i < this_host_addr.size(); ++i) {
+				for (unsigned j = 0; j < addrs.size(); ++j) {
+					if (this_host_addr[i].compare_address(addrs[j])) {
 						runs_here = TRUE;
 						break;
 					}
@@ -283,19 +266,7 @@ daemon::runs_on_this_host()
 	// X_RUNS_HERE controls whether or not to run kbdd if it's presented in
 	// the config file
 	{
-		tmp = param("X_RUNS_HERE");
-		if(tmp)
-		{
-			if(*tmp == 'T' || *tmp == 't')
-			{
-				runs_here = TRUE;
-			}
-			else
-			{
-				runs_here = FALSE;
-			}
-		free(tmp);
-		}
+		runs_here = param_boolean_crufty("X_RUNS_HERE", false) ? TRUE : FALSE;
 	}
 	return runs_here;
 }
@@ -604,7 +575,7 @@ int daemon::RealStart( )
 	shortname = condor_basename( process_name );
 
 	if( access(process_name,X_OK) != 0 ) {
-		dprintf(D_ALWAYS, "%s: Cannot execute\n", process_name );
+		dprintf(D_ALWAYS, "%s: Cannot execute (errno=%d, %s)\n", process_name, errno, strerror(errno) );
 		pid = 0; 
 		// Schedule to try and restart it a little later
 		Restart();
@@ -612,15 +583,15 @@ int daemon::RealStart( )
 	}
 
 	if( !m_after_startup_wait_for_file.IsEmpty() ) {
+		MSC_SUPPRESS_WARNING_FIXME(6031)
 		remove( m_after_startup_wait_for_file.Value() );
 	}
 
-		// Collector needs to listen on a well known port, as does the
-		// Negotiator.  
 		// We didn't want them to use root for any reason, but b/c of
 		// evil in the security code where we're looking up host certs
 		// in the keytab file, we still need root afterall. :(
 	bool wants_condor_priv = false;
+		// Collector needs to listen on a well known port.
 	if ( strcmp(name_in_config_file,"COLLECTOR") == 0 ) {
 
 			// Go through all of the
@@ -631,10 +602,11 @@ int daemon::RealStart( )
 		dprintf ( 
 			D_FULLDEBUG, 
 			"Looking for matching Collector on '%s' ...\n", 
-			my_full_hostname () );
+			get_local_fqdn().Value());
 		CollectorList* collectors = NULL;
 		if ((collectors = daemonCore->getCollectorList())) {
-			char * my_hostname = my_full_hostname();
+			MyString my_fqdn_str = get_local_fqdn();
+			const char * my_hostname = my_fqdn_str.Value();
 			Daemon * my_daemon;
 			collectors->rewind();
 			while (collectors->next (my_daemon)) {
@@ -1213,7 +1185,8 @@ daemon::Obituary( int status )
     char buf[1000];
 
 	MyString email_subject;
-	email_subject.sprintf("Problem %s: %s ", my_full_hostname(), condor_basename(process_name));
+	email_subject.sprintf("Problem %s: %s ", get_local_fqdn().Value(), 
+						  condor_basename(process_name));
 	if ( was_not_responding ) {
 		email_subject += "killed (unresponsive)";
 	} else {
@@ -1239,8 +1212,8 @@ daemon::Obituary( int status )
         return;
     }
 
-	fprintf( mailer, "\"%s\" on \"%s\" ",process_name,
-		my_full_hostname());
+	fprintf( mailer, "\"%s\" on \"%s\" ",process_name, 
+			 get_local_fqdn().Value() );
 
 	if ( was_not_responding ) {
 		fprintf( mailer, "was killed because\nit was no longer responding.\n");
@@ -1401,6 +1374,7 @@ daemon::InitParams()
 			
 	int length = strlen(name_in_config_file) + 32;
 	buf = (char *)malloc(length);
+	ASSERT( buf != NULL );
 	snprintf( buf, length, "%s_WATCH_FILE", name_in_config_file );
 	watch_name = param( buf );
 	free(buf);
@@ -1618,6 +1592,7 @@ Daemons::Daemons()
 	all_daemons_gone_action = MASTER_RESET;
 	immediate_restart = FALSE;
 	immediate_restart_master = FALSE;
+	stop_other_daemons_when_startds_gone = NONE;
 	prevLHF = 0;
 	m_retry_start_all_daemons_tid = -1;
 	master = NULL;
@@ -1693,16 +1668,39 @@ Daemons::CheckForNewExecutable()
 		return;
 	}
 	if( NewExecutable( master->watch_name, &master->timeStamp ) ) {
-		dprintf( D_ALWAYS,"%s was modified, restarting %s.\n", 
-				 master->watch_name, 
-				 master->process_name );
 		master->newExec = TRUE;
+		if (NONE == new_bin_restart_mode) {
+			dprintf( D_ALWAYS,"%s was modified, but master restart mode is NEVER\n", 
+					 master->watch_name);
+			//don't want to do this in case the user later reconfigs the restart mode.
+			//CancelNewExecTimer();
+		} else {
+			dprintf( D_ALWAYS,"%s was modified, restarting %s %s.\n", 
+					 master->watch_name, 
+					 master->process_name,
+					 (new_bin_restart_mode == PEACEFUL) ? "Peacefully" : "Gracefully");
 			// Begin the master restart procedure.
-		
-		RestartMaster();
+			if (PEACEFUL == new_bin_restart_mode) {
+				DoPeacefulShutdown(5, &Daemons::RestartMasterPeaceful, "RestartMasterPeaceful");
+			} else {
+				RestartMaster();
+			}
+		}
 		return;
 	}
 
+	// don't even bother to look at other binaries if the new bin restart
+	// mode is GRACEFUL/PEACEFUL or NONE we do this because for NONE we don't
+	// want to restart anyway, and for GRACEFUL/PEACEFUL, we don't want to
+	// restart in arbitrary order.  it would be better to detect any binary
+	// changes, wait for things to settle, and then restart everthing that
+	// changed in the correct order. Well save that for a future change
+	//
+	if (PEACEFUL == new_bin_restart_mode || 
+		GRACEFUL == new_bin_restart_mode ||
+		NONE == new_bin_restart_mode)
+		return;
+
 	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
 		if( iter->second->runs_here && !iter->second->newExec 
 			&& !iter->second->OnHold()
@@ -1852,17 +1850,44 @@ Daemons::StopAllDaemons()
 	CancelRetryStartAllDaemons();
 	daemons.SetAllReaper();
 	int running = 0;
-	std::map<std::string, class daemon*>::iterator iter;
 
+	int any_running = 0;
+	int startds_running = 0;
+
+	// first check to see if any startd's are running, if there are, request
+	// that they
+	std::map<std::string, class daemon*>::iterator iter;
 	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
 		if( iter->second->pid && iter->second->runs_here &&
 			!iter->second->OnlyStopWhenMasterStops() )
 		{
-			iter->second->Stop();
-			running++;
+			if (iter->second->type == DT_STARTD) {
+				iter->second->Stop();
+				++startds_running;
+			}
+			++any_running;
 		}
 	}
-	if( !running ) {
+
+	// if there are daemons running, but no startd's running.
+	// request that the daemons peacefully exit.
+	//
+	if (startds_running) {
+		// tell the all-reaper (actually the AllStartdsGone method) to stop-peaceful
+		// the remaining daemons instead of actually stopping them here.
+		stop_other_daemons_when_startds_gone = GRACEFUL;
+	} else if (any_running) {
+		for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+			if( iter->second->pid && iter->second->runs_here &&
+				!iter->second->OnlyStopWhenMasterStops() )
+			{
+				iter->second->Stop();
+				running++;
+			}
+		}
+	}
+
+	if( !any_running ) {
 		AllDaemonsGone();
 	}	   
 }
@@ -1909,25 +1934,118 @@ Daemons::StopFastAllDaemons()
 	}	   
 }
 
-void
-Daemons::StopPeacefulAllDaemons()
-{
-	CancelRetryStartAllDaemons();
-	daemons.SetAllReaper();
-	int running = 0;
-	std::map<std::string, class daemon*>::iterator iter;
 
+int  Daemons::SendSetPeacefulShutdown(class daemon* target, int timeout)
+{
+	// only STARTDs and SCHEDDs recognise this command.
+	if (target->type != DT_STARTD && target->type != DT_SCHEDD)
+		return 0;
+
+	// fire and forget, if the command fails, we just end up with a graceful shutdown
+	// rather than a peaceful one.
+	classy_counted_ptr<Daemon> d = new Daemon(target->type);
+	classy_counted_ptr<DCCommandOnlyMsg>  msg = new DCCommandOnlyMsg(DC_SET_PEACEFUL_SHUTDOWN);
+	msg->setSuccessDebugLevel(D_ALWAYS);
+	msg->setStreamType(Stream::reli_sock);
+	msg->setTimeout(timeout);
+	d->sendMsg(msg.get());
+	return 0;
+}
+
+int
+Daemons::SetPeacefulShutdown(int timeout)
+{
+	int messages = 0;
+
+	// tell STARTD's and SCHEDD's that this is to be a peaceful shutdown.
+	std::map<std::string, class daemon*>::iterator iter;
 	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
 		if( iter->second->pid && iter->second->runs_here &&
 			!iter->second->OnlyStopWhenMasterStops() )
 		{
-			iter->second->StopPeaceful();
-			running++;
+			if (iter->second->type == DT_STARTD || iter->second->type == DT_SCHEDD) {
+				SendSetPeacefulShutdown(iter->second, timeout);
+				++messages;
+			}
 		}
 	}
-	if( !running ) {
+
+	return messages;
+}
+
+void
+Daemons::DoPeacefulShutdown(
+	int             timeout,
+	void (Daemons::*pfn)(void),
+	const char *    lbl)
+{
+	int messages = SetPeacefulShutdown(timeout);
+
+	// if we sent any messages, set a timer to to do the StopPeaceful call.
+	// to give the messages a chance to arrive.
+	bool fTimer = false;
+	if (messages > 0) {
+		int tid = daemonCore->Register_Timer(timeout+1, 0, (TimerHandlercpp)pfn, lbl, this);
+		if (tid == -1) {
+			dprintf( D_ALWAYS, "ERROR! Can't register DaemonCore timer!\n" );
+		} else {
+			fTimer = true;
+		}
+	}
+
+	// if the shutdown/restart command isn't going to happen on a timer,
+	// then do it now.
+	if ( ! fTimer) {
+		((this)->*(pfn))();
+	}
+}
+
+void
+Daemons::StopPeacefulAllDaemons()
+{
+	CancelRetryStartAllDaemons();
+	daemons.SetAllReaper(false); // false, because we don't assume that there are any startds
+	int any_running = 0;
+	int startds_running = 0;
+
+	// first check to see if any startd's are running, if there are, request
+	// that they peacefully exit.
+	std::map<std::string, class daemon*>::iterator iter;
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->pid && iter->second->runs_here &&
+			!iter->second->OnlyStopWhenMasterStops() )
+		{
+			if (iter->second->type == DT_STARTD) {
+				iter->second->StopPeaceful();
+				++startds_running;
+			}
+			++any_running;
+		}
+	}
+
+	// if there are daemons running, but no startd's running.
+	// request that the daemons peacefully exit.
+	//
+	if (startds_running) {
+		// tell the all-reaper (actually the AllStartdsGone method) to stop-peaceful
+		// the remaining daemons instead of actually stopping them here.
+		stop_other_daemons_when_startds_gone = PEACEFUL;
+	} else if (any_running) {
+		for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+			if( iter->second->pid && iter->second->runs_here &&
+				!iter->second->OnlyStopWhenMasterStops() )
+			{
+				iter->second->StopPeaceful();
+			}
+		}
+	}
+
+	// if there were no daemons running, do the AllDaemonsGone dance.
+	// if there were startds running
+	//
+	if( !any_running ) {
 		AllDaemonsGone();
-	}	   
+	}
 }
 
 
@@ -2060,9 +2178,7 @@ Daemons::CleanupBeforeRestart()
 		// is a daemon core process.  but, its parent is gone ( we are doing
 		// an exec, so we disappear), thus we must blank out the 
 		// CONDOR_INHERIT env variable.
-	char	tmps[256];
-	sprintf( tmps, "%s=", EnvGetName( ENV_INHERIT ) );
-	putenv( tmps );
+	UnsetEnv(EnvGetName( ENV_INHERIT ));
 }
 
 void
@@ -2071,11 +2187,11 @@ Daemons::ExecMaster()
 	int i=0,j;
 	char **argv = (char **)malloc((condor_main_argc+2)*sizeof(char *));
 
-	ASSERT( condor_main_argc>0 );
+	ASSERT( argv != NULL && condor_main_argc > 0 );
 	argv[i++] = condor_main_argv[0];
 
 		// insert "-f" argument so that new master does not fork
-	argv[i++] = "-f";
+	argv[i++] = const_cast<char *>("-f");
 	for(j=1;j<condor_main_argc;j++) {
 		size_t len = strlen(condor_main_argv[j]);
 		if( strncmp(condor_main_argv[j],"-foreground",len)==0 ) {
@@ -2175,7 +2291,7 @@ Daemons::FinalRestartMaster()
 }
 
 
-char* Daemons::DaemonLog( int pid )
+const char* Daemons::DaemonLog( int pid )
 {
 	std::map<std::string, class daemon*>::iterator iter;
 
@@ -2221,6 +2337,24 @@ int Daemons::NumberOfChildren()
 	return result;
 }
 
+// This function returns the number of active child processes with the given daemon type
+// currently being supervised by the master.
+int Daemons::NumberOfChildrenOfType(daemon_t type)
+{
+	int result = 0;
+	std::map<std::string, class daemon*>::iterator iter;
+
+	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+		if( iter->second->runs_here && iter->second->pid
+			&& !iter->second->OnlyStopWhenMasterStops() 
+			&& iter->second->type == type) {
+			result++;
+		}
+	}
+	dprintf(D_FULLDEBUG,"NumberOfChildrenOfType(%d) returning %d\n",type,result);
+	return result;
+}
+
 
 int
 Daemons::AllReaper(int pid, int status)
@@ -2236,6 +2370,8 @@ Daemons::AllReaper(int pid, int status)
 		exit_allowed.erase( valid_iter );
 		if( NumberOfChildren() == 0 ) {
 			AllDaemonsGone();
+		} else if ( NumberOfChildrenOfType(DT_STARTD) == 0 ) {
+			AllStartdsGone();
 		}
 		return TRUE;
 	}
@@ -2245,6 +2381,8 @@ Daemons::AllReaper(int pid, int status)
 			iter->second->Exited( status );
 			if( NumberOfChildren() == 0 ) {
 				AllDaemonsGone();
+			} else if ( NumberOfChildrenOfType(DT_STARTD) == 0 ) {
+				AllStartdsGone();
 			}
 			return TRUE;
 		}
@@ -2286,7 +2424,7 @@ Daemons::DefaultReaper(int pid, int status)
 // This function will set the reaper to one that calls
 // AllDaemonsGone() when all the daemons have exited.
 void
-Daemons::SetAllReaper()
+Daemons::SetAllReaper(bool fStartdsFirst)
 {
 	if( reaper == ALL_R ) {
 			// All reaper is already set.
@@ -2296,6 +2434,7 @@ Daemons::SetAllReaper()
 							  (ReaperHandlercpp)&Daemons::AllReaper,
 							  "Daemons::AllReaper()",this);
 	reaper = ALL_R;
+	stop_other_daemons_when_startds_gone = fStartdsFirst ? GRACEFUL : NONE;
 }
 
 
@@ -2331,8 +2470,7 @@ Daemons::StopDaemonsBeforeMasterStops()
 	std::map<std::string, class daemon*>::iterator iter;
 
 	for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
-		if( ( iter->second->pid || iter->second->IsHA() )
-			&& iter->second->runs_here )
+		if( iter->second->pid && iter->second->runs_here )
 		{
 			iter->second->Stop();
 			running++;
@@ -2366,6 +2504,28 @@ Daemons::AllDaemonsGone()
 	}
 }
 
+void
+Daemons::AllStartdsGone()
+{
+	StopStateT stop = stop_other_daemons_when_startds_gone;
+	stop_other_daemons_when_startds_gone = NONE;
+	if ( GRACEFUL == stop || PEACEFUL == stop ) {
+		dprintf( D_ALWAYS, "All STARTDs are gone.  Stopping other daemons %s\n",
+				(GRACEFUL == stop) ? "Gracefully" : "Peacefully");
+
+		std::map<std::string, class daemon*>::iterator iter;
+		for( iter = daemon_ptr.begin(); iter != daemon_ptr.end(); iter++ ) {
+			if( iter->second->pid && iter->second->runs_here &&
+				!iter->second->OnlyStopWhenMasterStops() )
+			{
+				if (PEACEFUL == stop)
+					iter->second->StopPeaceful();
+				else
+					iter->second->Stop();
+			}
+		}
+	}
+}
 
 void
 Daemons::StartTimers()
@@ -2492,6 +2652,7 @@ Daemons::UpdateCollector()
 
 	Update(ad);
     daemonCore->publish(ad);
+    daemonCore->dc_stats.Publish(*ad);
     daemonCore->monitor_data.ExportData(ad);
 	daemonCore->sendUpdates(UPDATE_MASTER_AD, ad, NULL, true);
 
