@@ -61,6 +61,9 @@ static char const *StartTimeAttr="StartTime";
 
 static char const *SlotWeightAttr=ATTR_SLOT_WEIGHT;
 
+/* Disable gcc warnings about floating point comparisons */
+GCC_DIAG_OFF(float-equal)
+
 //------------------------------------------------------------------
 // Constructor - One time initialization
 //------------------------------------------------------------------
@@ -104,7 +107,6 @@ void Accountant::Initialize(GroupEntry* root_group)
   NiceUserPriorityFactor=10000000;
   RemoteUserPriorityFactor=10000;
   DefaultPriorityFactor=1;
-  HalfLifePeriod=86400;
 
   // Set up HGQ accounting-group related information
   hgq_root_group = root_group;
@@ -121,14 +123,7 @@ void Accountant::Initialize(GroupEntry* root_group)
       }
   }
   
- 
- // get half life period
-  
-  tmp = param("PRIORITY_HALFLIFE");
-  if(tmp) {
-	  HalfLifePeriod=(float)atof(tmp);
-	  free(tmp);
-  }
+  HalfLifePeriod = param_double("PRIORITY_HALFLIFE");
 
   // get nice users priority factor
 
@@ -292,7 +287,19 @@ void Accountant::Initialize(GroupEntry* root_group)
 	  }
   }
 
-  // Update priorities
+  // Ensure that table entries for acct groups get created on startup and reconfig
+  // Do this after loading the accountant log, to give log data precedence
+  grpq.clear();
+  grpq.push_back(hgq_root_group);
+  while (!grpq.empty()) {
+      GroupEntry* group = grpq.front();
+      grpq.pop_front();
+      // This creates entries if they don't already exist:
+      GetPriority(group->name);
+      for (vector<GroupEntry*>::iterator j(group->children.begin());  j != group->children.end();  ++j) {
+          grpq.push_back(*j);
+      }
+  }
 
   UpdatePriorities();
 }
@@ -660,7 +667,7 @@ void Accountant::AddMatch(const MyString& CustomerName, ClassAd* ResourceAd)
     free(str);
   }    
 
-  AcctLog->CommitTransaction();
+  AcctLog->CommitNondurableTransaction();
 
   dprintf(D_ACCOUNTANT,"(ACCOUNTANT) Added match between customer %s and resource %s\n",CustomerName.Value(),ResourceName.Value());
 }
@@ -750,7 +757,7 @@ void Accountant::RemoveMatch(const MyString& ResourceName, time_t T)
   SetAttributeFloat(CustomerRecord+GroupName.c_str(),WeightedUnchargedTimeAttr,WeightedGroupUnchargedTime);
 
   DeleteClassAd(ResourceRecord+ResourceName);
-  AcctLog->CommitTransaction();
+  AcctLog->CommitNondurableTransaction();
 
   dprintf(D_ACCOUNTANT, "(ACCOUNTANT) Removed match between customer %s and resource %s\n",
           CustomerName.Value(),ResourceName.Value());
@@ -832,6 +839,11 @@ void Accountant::UpdatePriorities()
   float WeightedResourcesUsed;
   int BeginUsageTime;
 
+	  // Each iteration of the loop should be atomic for consistency,
+	  // but instead of doing one transaction per iteration, wrap the
+	  // whole loop in one transaction for efficiency.
+  AcctLog->BeginTransaction();
+
   AcctLog->table.startIterations();
   while (AcctLog->table.iterate(HK,ad)) {
     MyString keybuf;
@@ -844,8 +856,12 @@ void Accountant::UpdatePriorities()
 	if (Priority<MinPriority) Priority=MinPriority;
     OldPrio=Priority;
 
+    // set_prio_factor indicates whether a priority factor has been explicitly set,
+    // in which case the record should be kept to preserve the setting
+    bool set_prio_factor = true;
     if (ad->LookupFloat(PriorityFactorAttr,PriorityFactor)==0) {
 	   	PriorityFactor = DefaultPriorityFactor;
+        set_prio_factor = false;
 	}
 
     if (ad->LookupInteger(UnchargedTimeAttr,UnchargedTime)==0) UnchargedTime=0;
@@ -862,7 +878,6 @@ void Accountant::UpdatePriorities()
     AccumulatedUsage+=ResourcesUsed*TimePassed+UnchargedTime;
     WeightedAccumulatedUsage+=WeightedResourcesUsed*TimePassed+WeightedUnchargedTime;
 
-	AcctLog->BeginTransaction();
     SetAttributeFloat(key,PriorityAttr,Priority);
     SetAttributeFloat(key,AccumulatedUsageAttr,AccumulatedUsage);
     SetAttributeFloat(key,WeightedAccumulatedUsageAttr,WeightedAccumulatedUsage);
@@ -871,18 +886,17 @@ void Accountant::UpdatePriorities()
     SetAttributeInt(key,UnchargedTimeAttr,0);
     SetAttributeFloat(key,WeightedUnchargedTimeAttr,0.0);
 
-    if (Priority<MinPriority && ResourcesUsed==0 &&
-		   	AccumulatedUsage==0 && PriorityFactor==DefaultPriorityFactor ) {
+    if (Priority<MinPriority && ResourcesUsed==0 && AccumulatedUsage==0 && !set_prio_factor) {
 		DeleteClassAd(key);
 	}
 
-	AcctLog->CommitTransaction();
-	
     dprintf(D_ACCOUNTANT,"CustomerName=%s , Old Priority=%5.3f , New Priority=%5.3f , ResourcesUsed=%d , WeightedResourcesUsed=%f\n",key,OldPrio,Priority,ResourcesUsed,WeightedResourcesUsed);
     dprintf(D_ACCOUNTANT,"RecentUsage=%8.3f (unweighted %8.3f), UnchargedTime=%8.3f (unweighted %d), AccumulatedUsage=%5.3f (unweighted %5.3f), BeginUsageTime=%d\n",WeightedRecentUsage,RecentUsage,WeightedUnchargedTime,UnchargedTime,WeightedAccumulatedUsage,AccumulatedUsage,BeginUsageTime);
 
   }
 
+  AcctLog->CommitTransaction();
+
   // Check if the log needs to be truncated
   struct stat statbuf;
   if( stat(LogFileName.Value(),&statbuf) ) {
@@ -1042,83 +1056,251 @@ AttrList* Accountant::ReportState(const MyString& CustomerName, int* NumResource
 // Report the whole list of priorities
 //------------------------------------------------------------------
 
-AttrList* Accountant::ReportState() {
-    dprintf(D_ACCOUNTANT,"Reporting State\n");
+ClassAd* Accountant::ReportState(bool rollup) {
+    dprintf(D_ACCOUNTANT, "Reporting State%s\n", (rollup) ? " using rollup mode" : "");
+
+    ClassAd* ad = new ClassAd();
+    ad->Assign("LastUpdate", LastUpdateTime);
+
+    // assign acct group index numbers first, breadth first ordering
+    int EntryNum=1;
+    map<string, int> gnmap;
+    deque<GroupEntry*> grpq;
+    grpq.push_back(hgq_root_group);
+    while (!grpq.empty()) {
+        GroupEntry* group = grpq.front();
+        grpq.pop_front();
+        gnmap[group->name] = EntryNum++;
+        for (vector<GroupEntry*>::iterator j(group->children.begin());  j != group->children.end();  ++j) {
+            grpq.push_back(*j);
+        }
+    }
+
+    // populate the ad with acct group entries, with optional rollup of
+    // attributes up the group hierarchy
+    ReportGroups(hgq_root_group, ad, rollup, gnmap);
 
     HashKey HK;
-    ClassAd* CustomerAd;
-    float PriorityFactor;
-    float AccumulatedUsage;
-    float WeightedAccumulatedUsage;
-    int BeginUsageTime;
-    int LastUsageTime;
-    int ResourcesUsed;
-    float WeightedResourcesUsed;
-
-    AttrList* ad=new AttrList();
-    MyString tmp;
-    tmp.sprintf("LastUpdate = %d", LastUpdateTime);
-    ad->Insert(tmp.Value());
-
-    int OwnerNum=1;
+    ClassAd* CustomerAd = NULL;
     AcctLog->table.startIterations();
     while (AcctLog->table.iterate(HK,CustomerAd)) {
         MyString key;
         HK.sprint(key);
-        if (strncmp(CustomerRecord.Value(),key.Value(),CustomerRecord.Length())) continue;
-
-        MyString CustomerName=key.Value()+CustomerRecord.Length();
-        tmp.sprintf("Name%d = \"%s\"",OwnerNum,CustomerName.Value());
-        ad->Insert(tmp.Value());
-
-        tmp.sprintf("Priority%d = %f",OwnerNum,GetPriority(CustomerName));
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupInteger(ResourcesUsedAttr,ResourcesUsed)==0) ResourcesUsed=0;
-        tmp.sprintf("ResourcesUsed%d = %d",OwnerNum,ResourcesUsed);
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupFloat(WeightedResourcesUsedAttr,WeightedResourcesUsed)==0) WeightedResourcesUsed=0;
-        tmp.sprintf("WeightedResourcesUsed%d = %f",OwnerNum,WeightedResourcesUsed);
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupFloat(AccumulatedUsageAttr,AccumulatedUsage)==0) AccumulatedUsage=0;
-        tmp.sprintf("AccumulatedUsage%d = %f",OwnerNum,AccumulatedUsage);
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupFloat(WeightedAccumulatedUsageAttr,WeightedAccumulatedUsage)==0) WeightedAccumulatedUsage=0;
-        tmp.sprintf("WeightedAccumulatedUsage%d = %f",OwnerNum,WeightedAccumulatedUsage);
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupInteger(BeginUsageTimeAttr,BeginUsageTime)==0) BeginUsageTime=0;
-        tmp.sprintf("BeginUsageTime%d = %d",OwnerNum,BeginUsageTime);
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupInteger(LastUsageTimeAttr,LastUsageTime)==0) LastUsageTime=0;
-        tmp.sprintf("LastUsageTime%d = %d",OwnerNum,LastUsageTime);
-        ad->Insert(tmp.Value());
-
-        if (CustomerAd->LookupFloat(PriorityFactorAttr,PriorityFactor)==0) PriorityFactor=0;
-        tmp.sprintf("PriorityFactor%d = %f",OwnerNum,PriorityFactor);
-        ad->Insert(tmp.Value());
+        if (strncmp(CustomerRecord.Value(), key.Value(), CustomerRecord.Length())) continue;
+        MyString CustomerName = key.Value()+CustomerRecord.Length();
 
         bool isGroup=false;
-        string cgrp = GetAssignedGroup(CustomerName, isGroup)->name;
-        tmp.sprintf("AccountingGroup%d = \"%s\"",OwnerNum,cgrp.c_str());
-        ad->Insert(tmp.Value());
-        tmp.sprintf("IsAccountingGroup%d = %s",OwnerNum,(isGroup)?"TRUE":"FALSE");
-        ad->Insert(tmp.Value());
+        GroupEntry* cgrp = GetAssignedGroup(CustomerName, isGroup);
 
-        OwnerNum++;
+        // entries for acct groups are now handled in ReportGroups(), above
+        if (isGroup) continue;
+
+        std::string cgname(cgrp->name);
+        int snum = EntryNum++;
+
+        string tmp;
+        sprintf(tmp, "Name%d", snum);
+        ad->Assign(tmp.c_str(), CustomerName);
+
+        sprintf(tmp, "IsAccountingGroup%d", snum);
+        ad->Assign(tmp.c_str(), isGroup);
+
+        sprintf(tmp, "AccountingGroup%d", snum);
+        ad->Assign(tmp.c_str(), cgname);
+
+        float Priority = GetPriority(CustomerName);
+        sprintf(tmp, "Priority%d", snum);
+        ad->Assign(tmp.c_str(), Priority);
+
+        float PriorityFactor = 0;
+        if (CustomerAd->LookupFloat(PriorityFactorAttr,PriorityFactor)==0) PriorityFactor=0;
+        sprintf(tmp, "PriorityFactor%d", snum);
+        ad->Assign(tmp.c_str(), PriorityFactor);
+
+        int ResourcesUsed = 0;
+        if (CustomerAd->LookupInteger(ResourcesUsedAttr,ResourcesUsed)==0) ResourcesUsed=0;
+        sprintf(tmp, "ResourcesUsed%d", snum);
+        ad->Assign(tmp.c_str(), ResourcesUsed);
+        
+        float WeightedResourcesUsed = 0;
+        if (CustomerAd->LookupFloat(WeightedResourcesUsedAttr,WeightedResourcesUsed)==0) WeightedResourcesUsed=0;
+        sprintf(tmp, "WeightedResourcesUsed%d", snum);
+        ad->Assign(tmp.c_str(), WeightedResourcesUsed);
+        
+        float AccumulatedUsage = 0;
+        if (CustomerAd->LookupFloat(AccumulatedUsageAttr,AccumulatedUsage)==0) AccumulatedUsage=0;
+        sprintf(tmp, "AccumulatedUsage%d", snum);
+        ad->Assign(tmp.c_str(), AccumulatedUsage);
+        
+        float WeightedAccumulatedUsage = 0;
+        if (CustomerAd->LookupFloat(WeightedAccumulatedUsageAttr,WeightedAccumulatedUsage)==0) WeightedAccumulatedUsage=0;
+        sprintf(tmp, "WeightedAccumulatedUsage%d", snum);
+        ad->Assign(tmp.c_str(), WeightedAccumulatedUsage);
+        
+        int BeginUsageTime = 0;
+        if (CustomerAd->LookupInteger(BeginUsageTimeAttr,BeginUsageTime)==0) BeginUsageTime=0;
+        sprintf(tmp, "BeginUsageTime%d", snum);
+        ad->Assign(tmp.c_str(), BeginUsageTime);
+        
+        int LastUsageTime = 0;
+        if (CustomerAd->LookupInteger(LastUsageTimeAttr,LastUsageTime)==0) LastUsageTime=0;
+        sprintf(tmp, "LastUsageTime%d", snum);
+        ad->Assign(tmp.c_str(), LastUsageTime);
     }
 
+    // total number of accountant entries, for acct groups and submittors
+    ad->Assign("NumSubmittors", EntryNum-1);
+
+    // include concurrency limit information
     ReportLimits(ad);
 
-    tmp.sprintf("NumSubmittors = %d", OwnerNum-1);
-    ad->Insert(tmp.Value());
     return ad;
 }
 
+
+void Accountant::ReportGroups(GroupEntry* group, ClassAd* ad, bool rollup, map<string, int>& gnmap) {
+    // begin by loading straight "non-rolled" data into the attributes for (group)
+    MyString CustomerName = group->name;
+
+    ClassAd* CustomerAd = NULL;
+    MyString HK(CustomerRecord + CustomerName);
+    if (AcctLog->table.lookup(HashKey(HK.Value()), CustomerAd) == -1) {
+        dprintf(D_ALWAYS, "WARNING: Expected AcctLog entry \"%s\" to exist", HK.Value());
+        return;
+    } 
+
+    bool isGroup=false;
+    GroupEntry* cgrp = GetAssignedGroup(CustomerName, isGroup);
+
+    std::string cgname;
+    int gnum = 0;
+    if (isGroup) {
+        cgname = (cgrp->parent != NULL) ? cgrp->parent->name : cgrp->name;
+        gnum = gnmap[cgrp->name];
+    } else {
+        dprintf(D_ALWAYS, "WARNING: Expected \"%s\" to be a defined group in the accountant", CustomerName.Value());
+        return;
+    }
+
+    string tmp;
+    sprintf(tmp, "Name%d", gnum);
+    ad->Assign(tmp.c_str(), CustomerName);
+
+    sprintf(tmp, "IsAccountingGroup%d", gnum);
+    ad->Assign(tmp.c_str(), isGroup);
+    
+    sprintf(tmp, "AccountingGroup%d", gnum);
+    ad->Assign(tmp.c_str(), cgname);
+    
+    float Priority = (!rollup) ? GetPriority(CustomerName) : 0;
+    sprintf(tmp, "Priority%d", gnum);
+    ad->Assign(tmp.c_str(), Priority);
+    
+    float PriorityFactor = 0;
+    if (!rollup && CustomerAd->LookupFloat(PriorityFactorAttr,PriorityFactor)==0) PriorityFactor=0;
+    sprintf(tmp, "PriorityFactor%d", gnum);
+    ad->Assign(tmp.c_str(), PriorityFactor);
+    
+    if (cgrp) {
+        sprintf(tmp, "EffectiveQuota%d", gnum);
+        ad->Assign(tmp.c_str(), cgrp->quota);
+        sprintf(tmp, "ConfigQuota%d", gnum);
+        ad->Assign(tmp.c_str(), cgrp->config_quota);
+        sprintf(tmp, "SubtreeQuota%d", gnum);
+        ad->Assign(tmp.c_str(), cgrp->subtree_quota);
+        sprintf(tmp, "GroupSortKey%d", gnum);
+        ad->Assign(tmp.c_str(), cgrp->sort_key);
+        sprintf(tmp, "SurplusPolicy%d", gnum);
+        const char * policy = "no";
+        if (cgrp->autoregroup) policy = "regroup";
+        else if (cgrp->accept_surplus) policy = "byquota";
+        ad->Assign(tmp.c_str(), policy);
+    }
+
+    int ResourcesUsed = 0;
+    if (CustomerAd->LookupInteger(ResourcesUsedAttr,ResourcesUsed)==0) ResourcesUsed=0;
+    sprintf(tmp, "ResourcesUsed%d", gnum);
+    ad->Assign(tmp.c_str(), ResourcesUsed);
+    
+    float WeightedResourcesUsed = 0;
+    if (CustomerAd->LookupFloat(WeightedResourcesUsedAttr,WeightedResourcesUsed)==0) WeightedResourcesUsed=0;
+    sprintf(tmp, "WeightedResourcesUsed%d", gnum);
+    ad->Assign(tmp.c_str(), WeightedResourcesUsed);
+    
+    float AccumulatedUsage = 0;
+    if (CustomerAd->LookupFloat(AccumulatedUsageAttr,AccumulatedUsage)==0) AccumulatedUsage=0;
+    sprintf(tmp, "AccumulatedUsage%d", gnum);
+    ad->Assign(tmp.c_str(), AccumulatedUsage);
+    
+    float WeightedAccumulatedUsage = 0;
+    if (CustomerAd->LookupFloat(WeightedAccumulatedUsageAttr,WeightedAccumulatedUsage)==0) WeightedAccumulatedUsage=0;
+    sprintf(tmp, "WeightedAccumulatedUsage%d", gnum);
+    ad->Assign(tmp.c_str(), WeightedAccumulatedUsage);
+    
+    int BeginUsageTime = 0;
+    if (CustomerAd->LookupInteger(BeginUsageTimeAttr,BeginUsageTime)==0) BeginUsageTime=0;
+    sprintf(tmp, "BeginUsageTime%d", gnum);
+    ad->Assign(tmp.c_str(), BeginUsageTime);
+    
+    int LastUsageTime = 0;
+    if (CustomerAd->LookupInteger(LastUsageTimeAttr,LastUsageTime)==0) LastUsageTime=0;
+    sprintf(tmp, "LastUsageTime%d", gnum);
+    ad->Assign(tmp.c_str(), LastUsageTime);
+    
+    // Populate group's children recursively, if it has any
+    // Recursion is to allow for proper rollup from children to parents
+    for (vector<GroupEntry*>::iterator j(group->children.begin());  j != group->children.end();  ++j) {
+        ReportGroups(*j, ad, rollup, gnmap);
+    }
+
+    // if we aren't doing rollup, finish now
+    if (!rollup || (NULL == group->parent)) return;
+
+    // get the index of our parent
+    int pnum = gnmap[group->parent->name];
+
+    int ival = 0;
+    float fval = 0;
+
+    // roll up values to parent
+    sprintf(tmp, "ResourcesUsed%d", gnum);
+    ad->LookupInteger(tmp.c_str(), ResourcesUsed);
+    sprintf(tmp, "ResourcesUsed%d", pnum);
+    ad->LookupInteger(tmp.c_str(), ival);
+    ad->Assign(tmp.c_str(), ival + ResourcesUsed);
+
+    sprintf(tmp, "WeightedResourcesUsed%d", gnum);
+    ad->LookupFloat(tmp.c_str(), WeightedResourcesUsed);
+    sprintf(tmp, "WeightedResourcesUsed%d", pnum);
+    ad->LookupFloat(tmp.c_str(), fval);    
+    ad->Assign(tmp.c_str(), fval + WeightedResourcesUsed);
+
+    sprintf(tmp, "AccumulatedUsage%d", gnum);
+    ad->LookupFloat(tmp.c_str(), AccumulatedUsage);
+    sprintf(tmp, "AccumulatedUsage%d", pnum);
+    ad->LookupFloat(tmp.c_str(), fval);
+    ad->Assign(tmp.c_str(), fval + AccumulatedUsage);
+
+    sprintf(tmp, "WeightedAccumulatedUsage%d", gnum);
+    ad->LookupFloat(tmp.c_str(), WeightedAccumulatedUsage);
+    sprintf(tmp, "WeightedAccumulatedUsage%d", pnum);
+    ad->LookupFloat(tmp.c_str(), fval);
+    ad->Assign(tmp.c_str(), fval + WeightedAccumulatedUsage);
+
+    sprintf(tmp, "BeginUsageTime%d", gnum);
+    ad->LookupInteger(tmp.c_str(), BeginUsageTime);
+    sprintf(tmp, "BeginUsageTime%d", pnum);
+    ad->LookupInteger(tmp.c_str(), ival);
+    ad->Assign(tmp.c_str(), min(ival, BeginUsageTime));
+
+    sprintf(tmp, "LastUsageTime%d", gnum);
+    ad->LookupInteger(tmp.c_str(), LastUsageTime);
+    sprintf(tmp, "LastUsageTime%d", pnum);
+    ad->LookupInteger(tmp.c_str(), ival);
+    ad->Assign(tmp.c_str(), max(ival, LastUsageTime));
+}
+
+
 //------------------------------------------------------------------
 // Extract resource name from class-ad
 //------------------------------------------------------------------
@@ -1468,9 +1650,15 @@ double Accountant::GetLimit(const MyString& limit)
 
 double Accountant::GetLimitMax(const MyString& limit)
 {
-	return param_double((limit + "_LIMIT").Value(),
-						 param_double("CONCURRENCY_LIMIT_DEFAULT",
-									   2308032));
+    double deflim = param_double("CONCURRENCY_LIMIT_DEFAULT", 2308032);
+    string lim(limit.Value());
+    string::size_type pos = lim.find_last_of('.');
+    if (pos != string::npos) {
+        string scopedef("CONCURRENCY_LIMIT_DEFAULT_");
+        scopedef += lim.substr(0,pos);
+        deflim = param_double(scopedef.c_str(), deflim);
+    }
+	return param_double((limit + "_LIMIT").Value(), deflim);
 }
 
 void Accountant::DumpLimits()
@@ -1485,13 +1673,18 @@ void Accountant::DumpLimits()
 
 void Accountant::ReportLimits(AttrList *attrList)
 {
-	MyString attr;
 	MyString limit;
  	double count;
 	concurrencyLimits.startIterations();
 	while (concurrencyLimits.iterate(limit, count)) {
-		attr.sprintf("ConcurrencyLimit_%s = %f\n", limit.Value(), count);
-		attrList->Insert(attr.Value());
+        string attr;
+        sprintf(attr, "ConcurrencyLimit_%s", limit.Value());
+        // classad wire protocol doesn't currently support attribute names that include
+        // punctuation or symbols outside of '_'.  If we want to include '.' or any other
+        // punct, we need to either model these as string values, or add support for quoted
+        // attribute names in wire protocol:
+        std::replace(attr.begin(), attr.end(), '.', '_');
+        attrList->Assign(attr.c_str(), count);
 	}
 }
 
@@ -1574,3 +1767,6 @@ float Accountant::GetSlotWeight(ClassAd *candidate)
 	}
 	return SlotWeight;
 }
+
+GCC_DIAG_ON(float-equal)
+
