@@ -59,10 +59,12 @@ struct GroupEntry {
     double config_quota;
 	bool static_quota;
 	bool accept_surplus;
+    bool autoregroup;
 
     // current usage information coming into this negotiation cycle
     double usage;
-    ClassAdList* submitterAds;
+    ClassAdListDoesNotDeleteAds* submitterAds;
+    double priority;
 
     // slot quota as computed by HGQ
     double quota;
@@ -84,8 +86,15 @@ struct GroupEntry {
     GroupEntry* parent;
     vector<GroupEntry*> children;
     map<string, size_type, Accountant::ci_less> chmap;
+
+    // attributes for configurable sorting
+    ClassAd* sort_ad;
+    double sort_key;
 };
 
+/* Disable floating-point equality warnings */
+
+GCC_DIAG_OFF(float-equal)
 
 class Matchmaker : public Service
 {
@@ -113,6 +122,7 @@ class Matchmaker : public Service
 		int SET_BEGINTIME_commandHandler(int, Stream*);
 		int SET_LASTTIME_commandHandler(int, Stream*);
 		int GET_PRIORITY_commandHandler(int, Stream*);
+		int GET_PRIORITY_ROLLUP_commandHandler(int, Stream*);
 		int GET_RESLIST_commandHandler(int, Stream*);
 
 		// timeout handler (for periodic negotiations)
@@ -128,6 +138,8 @@ class Matchmaker : public Service
 		static float EvalNegotiatorMatchRank(char const *expr_name,ExprTree *expr,
 		                              ClassAd &request,ClassAd *resource);
 
+		bool getGroupInfoFromUserId(const char* user, string& groupName, float& groupQuota, float& groupUsage);
+
     protected:
 		char * NegotiatorName;
 		int update_interval;
@@ -145,6 +157,7 @@ class Matchmaker : public Service
 		char * compute_significant_attrs(ClassAdListDoesNotDeleteAds & startdAds);
 		
 		/** Negotiate w/ one schedd for one user, for one 'pie spin'.
+            @param groupName name of group negotiating under (or NULL)
 			@param scheddName Name attribute from the submitter ad.
 			@param scheddAddr Sinful string of schedd for this submitter.
 			@param priority Priority of this user from the accountant.
@@ -162,24 +175,26 @@ class Matchmaker : public Service
 					MM_DONE if schedd got all the resources it wanted,
 					MM_ERROR if problem negotiating w/ this schedd.
 		**/
-		int negotiate( char const *scheddName, const ClassAd *scheddAd, 
-		   double priority, double share,
-		   double submitterLimit,
+		int negotiate(char const* groupName, char const *scheddName, const ClassAd *scheddAd, 
+		   double priority,
+           double submitterLimit, double submitterLimitUnclaimed,
 		   ClassAdListDoesNotDeleteAds &startdAds, ClaimIdHash &claimIds, 
-		   const CondorVersionInfo & scheddVersion,
 		   bool ignore_schedd_limit, time_t startTime, 
-		   int &numMatched, double &limitUsed, double &pieLeft);
+           int& numMatched, double &pieLeft);
 
 		int negotiateWithGroup ( int untrimmed_num_startds,
 								 double untrimmedSlotWeightTotal,
 								 double minSlotWeight,
 			ClassAdListDoesNotDeleteAds& startdAds, 
 			ClaimIdHash& claimIds, ClassAdListDoesNotDeleteAds& scheddAds, 
-			float groupQuota=INT_MAX, float groupusage=0, const char* groupAccountingName=NULL);
+			float groupQuota=INT_MAX, const char* groupName=NULL);
 
 		
-		ClassAd *matchmakingAlgorithm(const char*,const char*,ClassAd&,ClassAdListDoesNotDeleteAds&,
-									  double=-1.0, double=1.0, double=0.0, double=0.0, double=0.0, bool=false);
+		ClassAd *matchmakingAlgorithm(const char* scheddName, const char* scheddAddr, ClassAd& request, ClassAdListDoesNotDeleteAds& startdAds,
+									  double preemptPrio, 
+                                      double limitUsed, double limitUsedUnclaimed,
+                                      double submitterLimit, double submitterLimitUnclaimed, 
+                                      double pieLeft, bool only_for_startdrank);
 		int matchmakingProtocol(ClassAd &request, ClassAd *offer, 
 						ClaimIdHash &claimIds, Sock *sock,
 						const char* scheddName, const char* scheddAddr);
@@ -215,6 +230,7 @@ class Matchmaker : public Service
 								  double slotWeightTotal,
 		                            /* result parameters: */
 								  double &submitterLimit,
+                                  double& submitterLimitUnclaimed,
 								  double &submitterUsage,
 		                          double &submitterShare,
 		                          double &submitterAbsShare,
@@ -253,6 +269,7 @@ class Matchmaker : public Service
 		void MakeClaimIdHash(ClassAdList &startdPvtAdList, ClaimIdHash &claimIds);
 		char const *getClaimId (const char *, const char *, ClaimIdHash &, MyString &);
 		void addRemoteUserPrios( ClassAd* ad );
+		void addRemoteUserPrios( ClassAdListDoesNotDeleteAds &cal );
 		void insertNegotiatorMatchExprs(ClassAd *ad);
 		void insertNegotiatorMatchExprs( ClassAdListDoesNotDeleteAds &cal );
 		void reeval( ClassAd *ad );
@@ -293,9 +310,10 @@ class Matchmaker : public Service
 		bool want_inform_startd;	
 		/// Should the negotiator use non-blocking connect to contact startds?
 		bool want_nonblocking_startd_contact;
-        ExprTree *DynQuotaMachConstraint;   // Filter machineAds by this
-                                            // constraint before calculating
-                                            // dynamic quotas.  Added for CDF.
+        char * strSlotConstraint; // query collector for machineAds with this constraint
+        ExprTree *SlotPoolsizeConstraint;   // Filter machineAds by this
+                                         // constraint before calculating quotas
+                                         // formerly DynQuotaMachConstraint Added for CDF.
 
 		StringList NegotiatorMatchExprNames;
 		StringList NegotiatorMatchExprValues;
@@ -308,11 +326,6 @@ class Matchmaker : public Service
 		typedef HashTable<MyString, float> groupQuotasHashType;
 		groupQuotasHashType *groupQuotasHash;
 
-		bool getGroupInfoFromUserId( const char *user, float & groupQuota, 
-			 float & groupUsage );
-		
-
-
 		// rank condition on matches
 		ExprTree *rankCondStd;// no preemption or machine rank-preemption 
 							  // i.e., (Rank > CurrentRank)
@@ -343,6 +356,7 @@ class Matchmaker : public Service
 		int rejPreemptForPolicy; //   - PREEMPTION_REQUIREMENTS == False?
 		int rejPreemptForRank;	//   - startd RANKs new job lower?
 		int rejForSubmitterLimit;   //   - not enough group quota?
+        string rejectedConcurrencyLimit; // the name of concurrency limit rejected
 
 
 		// Class used to store each individual entry in the
@@ -447,6 +461,9 @@ class Matchmaker : public Service
         string hgq_root_name;
         vector<GroupEntry*> hgq_groups;
         map<string, GroupEntry*> group_entry_map;
+        bool accept_surplus;
+        bool autoregroup;
+        bool allow_quota_oversub;
 
         void hgq_construct_tree();
         void hgq_assign_quotas(GroupEntry* group, double quota);
@@ -475,7 +492,7 @@ class Matchmaker : public Service
 		void StartNewNegotiationCycleStat();
 		void publishNegotiationCycleStats( ClassAd *ad );
 };
-
+GCC_DIAG_ON(float-equal)
 
 
 #endif//__MATCHMAKER_H__
