@@ -37,10 +37,12 @@
 #include "condor_distribution.h"
 #include "CondorError.h"
 #include "str_isxxx.h"
+#include "enum_utils.h"
 
 
 char	*MyName;
 char 	*actionReason = NULL;
+char    *holdReasonSubCode = NULL;
 JobAction mode;
 bool All = false;
 bool ConstraintArg = false;
@@ -54,7 +56,7 @@ StringList* job_ids = NULL;
 	// Prototypes of local interest
 void addConstraint(const char *);
 void procArg(const char*);
-void usage();
+void usage(int iExitCode=1);
 void handleAll();
 void handleConstraints( void );
 ClassAd* doWorkByList( StringList* ids, CondorError * errstack );
@@ -77,6 +79,14 @@ actionWord( JobAction action, bool past )
 	case JA_HOLD_JOBS:
 		return past ? "held" : "hold";
 		break;
+	
+	case JA_SUSPEND_JOBS:
+		return past ? "suspended" : "suspend";
+		break;
+		
+	case JA_CONTINUE_JOBS:
+		return past ? "continued" : "continue";
+		break;
 
 	case JA_REMOVE_JOBS:
 	case JA_REMOVE_X_JOBS:
@@ -101,10 +111,10 @@ actionWord( JobAction action, bool past )
 
 
 void
-usage()
+usage(int iExitCode)
 {
 	char word[32];
-	sprintf( word, getJobActionString(mode) );
+	sprintf( word, "%s", getJobActionString(mode) );
 	fprintf( stderr, "Usage: %s [options] [constraints]\n", MyName );
 	fprintf( stderr, " where [options] is zero or more of:\n" );
 	fprintf( stderr, "  -help               Display this message and exit\n" );
@@ -123,6 +133,7 @@ usage()
 		fprintf( stderr, "  -reason reason      Use the given ReleaseReason\n");
 	} else if( mode == JA_HOLD_JOBS ) {
 		fprintf( stderr, "  -reason reason      Use the given HoldReason\n");
+		fprintf( stderr, "  -subcode number     Set HoldReasonSubCode\n");
 	}
 
 	if( mode == JA_REMOVE_JOBS || mode == JA_REMOVE_X_JOBS ) {
@@ -141,7 +152,7 @@ usage()
 	fprintf( stderr, "  -constraint expr    %s all jobs matching the boolean expression\n", word );
 	fprintf( stderr, "  -all                %s all jobs "
 			 "(cannot be used with other constraints)\n", word );
-	exit( 1 );
+	exit( iExitCode );
 }
 
 
@@ -164,6 +175,7 @@ main( int argc, char *argv[] )
 	DCCollector* pool = NULL;
 	char* scheddName = NULL;
 	char* scheddAddr = NULL;
+	param_functions *p_funcs = NULL;
 
 		// Initialize our global variables
 	has_constraint = false;
@@ -192,6 +204,16 @@ main( int argc, char *argv[] )
 		mode = JA_RELEASE_JOBS;
 
 	} else if ( cmd_str && 
+			strncasecmp( cmd_str, "_suspend", strlen("_suspend") ) == MATCH ) {
+
+		mode = JA_SUSPEND_JOBS;
+
+	} else if ( cmd_str && 
+			strncasecmp( cmd_str, "_continue", strlen("_continue") ) == MATCH ) {
+
+		mode = JA_CONTINUE_JOBS;
+
+	}else if ( cmd_str && 
 			strncasecmp( cmd_str, "_rm", strlen("_rm") ) == MATCH ) {
 
 		mode = JA_REMOVE_JOBS;
@@ -225,7 +247,8 @@ main( int argc, char *argv[] )
             if (match_prefix(arg, "-debug")) {
 				// dprintf to console
 				Termlog = 1;
-				dprintf_config ("TOOL");
+				p_funcs = get_param_functions();
+				dprintf_config ("TOOL", p_funcs);
             } else if (match_prefix(arg, "-constraint")) {
 				args[nArgs] = arg;
 				nArgs++;
@@ -278,6 +301,22 @@ main( int argc, char *argv[] )
 					fprintf( stderr, "Out of memory!\n" );
 					exit(1);
 				}
+			} else if (match_prefix(arg, "-subcode")) {
+				argv++;
+				if( ! *argv ) {
+					fprintf( stderr, 
+							 "%s: -subcode requires another argument\n", 
+							 MyName);
+					exit(1);
+				}		
+				char *end = NULL;
+				long code = strtol(*argv,&end,10);
+				if( code == LONG_MIN || !end || *end || end==*argv ) {
+					fprintf( stderr, "Invalid -subcode %s!\n", *argv );
+					exit(1);
+				}
+				holdReasonSubCode = strdup(*argv);
+				ASSERT( holdReasonSubCode );
             } else if (match_prefix(arg, "-forcex")) {
 				if( mode == JA_REMOVE_JOBS ) {
 					mode = JA_REMOVE_X_JOBS;
@@ -326,7 +365,7 @@ main( int argc, char *argv[] )
             } else if (match_prefix(arg, "-version")) {
 				version();
             } else if (match_prefix(arg, "-help")) {
-				usage();
+				usage(0);
             } else {
 				fprintf( stderr, "Unrecognized option: %s\n", arg ); 
 				usage();
@@ -358,16 +397,22 @@ main( int argc, char *argv[] )
 	if( actionReason == NULL ) {
 		switch( mode ) {
 		case JA_RELEASE_JOBS:
-			actionReason = "via condor_release";
+			actionReason = strdup("via condor_release");
 			break;
 		case JA_REMOVE_X_JOBS:
-			actionReason = "via condor_rm -forcex";
+			actionReason = strdup("via condor_rm -forcex");
 			break;
 		case JA_REMOVE_JOBS:
-			actionReason = "via condor_rm";
+			actionReason = strdup("via condor_rm");
 			break;
 		case JA_HOLD_JOBS:
-			actionReason = "via condor_hold";
+			actionReason = strdup("via condor_hold");
+			break;
+		case JA_SUSPEND_JOBS:
+			actionReason = strdup("via condor_suspend");
+			break;
+		case JA_CONTINUE_JOBS:
+			actionReason = strdup("via condor_continue");
 			break;
 		default:
 			actionReason = NULL;
@@ -451,7 +496,7 @@ main( int argc, char *argv[] )
 bool
 doWorkByConstraint( const char* constraint, CondorError * errstack )
 {
-	ClassAd* ad;
+	ClassAd* ad = 0;
 	int total_jobs = -1;
 	bool rval = true;
 	switch( mode ) {
@@ -459,8 +504,7 @@ doWorkByConstraint( const char* constraint, CondorError * errstack )
 		ad = schedd->releaseJobs( constraint, actionReason, errstack );
 		break;
 	case JA_REMOVE_X_JOBS:
-		ad = schedd->removeXJobs( constraint, actionReason,
-								  errstack );
+		ad = schedd->removeXJobs( constraint, actionReason, errstack );
 		break;
 	case JA_VACATE_JOBS:
 		ad = schedd->vacateJobs( constraint, VACATE_GRACEFUL, errstack );
@@ -472,7 +516,13 @@ doWorkByConstraint( const char* constraint, CondorError * errstack )
 		ad = schedd->removeJobs( constraint, actionReason, errstack );
 		break;
 	case JA_HOLD_JOBS:
-		ad = schedd->holdJobs( constraint, actionReason, errstack );
+		ad = schedd->holdJobs( constraint, actionReason, holdReasonSubCode, errstack );
+		break;
+	case JA_SUSPEND_JOBS:
+		ad = schedd->suspendJobs( constraint, actionReason, errstack );
+		break;
+	case JA_CONTINUE_JOBS:
+		ad = schedd->continueJobs( constraint, actionReason, errstack );
 		break;
 	default:
 		EXCEPT( "impossible: unknown mode in doWorkByConstraint" );
@@ -507,7 +557,7 @@ doWorkByConstraint( const char* constraint, CondorError * errstack )
 ClassAd*
 doWorkByList( StringList* ids, CondorError *errstack )
 {
-	ClassAd* rval;
+	ClassAd* rval = 0;
 	switch( mode ) {
 	case JA_RELEASE_JOBS:
 		rval = schedd->releaseJobs( ids, actionReason, errstack );
@@ -525,7 +575,13 @@ doWorkByList( StringList* ids, CondorError *errstack )
 		rval = schedd->removeJobs( ids, actionReason, errstack );
 		break;
 	case JA_HOLD_JOBS:
-		rval = schedd->holdJobs( ids, actionReason, errstack );
+		rval = schedd->holdJobs( ids, actionReason, holdReasonSubCode, errstack );
+		break;
+	case JA_SUSPEND_JOBS:
+		rval = schedd->suspendJobs( ids, actionReason, errstack );
+		break;
+	case JA_CONTINUE_JOBS:
+		rval = schedd->continueJobs( ids, actionReason, errstack );
 		break;
 	default:
 		EXCEPT( "impossible: unknown mode in doWorkByList" );
@@ -755,6 +811,3 @@ printNewMessages( ClassAd* result_ad, StringList* ids )
 		free( msg );
 	}
 }
-
-
-#include "daemon_core_stubs.h"
