@@ -136,7 +136,7 @@ GridUniverseLogic::JobAdded(const char* owner, const char* domain,
 	// start timer to signal gridmanager if we haven't already
 	if ( node->add_timer_id == -1 ) {  // == -1 means no timer set
 		node->add_timer_id = daemonCore->Register_Timer(job_added_delay,
-			(TimerHandler) &GridUniverseLogic::SendAddSignal,
+			GridUniverseLogic::SendAddSignal,
 			"GridUniverseLogic::SendAddSignal");
 		daemonCore->Register_DataPtr(node);
 	}
@@ -160,9 +160,9 @@ GridUniverseLogic::JobRemoved(const char* owner, const char* domain,
 	}
 
 	// start timer to signal gridmanager if we haven't already
-	if ( node->add_timer_id == -1 ) {  // == -1 means no timer set
+	if ( node->remove_timer_id == -1 ) {  // == -1 means no timer set
 		node->remove_timer_id = daemonCore->Register_Timer(job_removed_delay,
-			(TimerHandler) &GridUniverseLogic::SendRemoveSignal,
+			GridUniverseLogic::SendRemoveSignal,
 			"GridUniverseLogic::SendRemoveSignal");
 		daemonCore->Register_DataPtr(node);
 	}
@@ -170,8 +170,8 @@ GridUniverseLogic::JobRemoved(const char* owner, const char* domain,
 	return;
 }
 
-int
-GridUniverseLogic::SendAddSignal(Service *)
+void
+GridUniverseLogic::SendAddSignal()
 {
 	// This method is called via a DC Timer set in JobAdded method
 
@@ -186,12 +186,10 @@ GridUniverseLogic::SendAddSignal(Service *)
 	if ( node->pid ) {
 		daemonCore->Send_Signal(node->pid,GRIDMAN_ADD_JOBS);
 	}
-
-	return 0;
 }
 
-int
-GridUniverseLogic::SendRemoveSignal(Service *)
+void
+GridUniverseLogic::SendRemoveSignal()
 {
 	// This method is called via a DC Timer set in JobRemoved method
 
@@ -206,8 +204,6 @@ GridUniverseLogic::SendRemoveSignal(Service *)
 	if ( node->pid ) {
 		daemonCore->Send_Signal(node->pid,GRIDMAN_REMOVE_JOBS);
 	}
-
-	return 0;
 }
 
 void
@@ -366,14 +362,10 @@ GridUniverseLogic::lookupGmanByOwner(const char* owner, const char* attr_value,
 	gman_node_t* result = NULL;
 	MyString owner_key(owner);
 	if(attr_value){
-		MyString attr_key(attr_value);
-		owner_key += attr_key;
+		owner_key += attr_value;
 	}
 	if (cluster) {
-		char tmp[100];
-		sprintf(tmp,"-%d.%d",cluster,proc);
-		MyString job_key(tmp);
-		owner_key += job_key;
+		owner_key.sprintf_cat( "-%d.%d", cluster, proc );
 	}
 
 	if (!gman_pid_table) {
@@ -387,6 +379,25 @@ GridUniverseLogic::lookupGmanByOwner(const char* owner, const char* attr_value,
 	return result;
 }
 
+int
+GridUniverseLogic::FindGManagerPid(const char* owner,
+					const char* attr_value,	
+					int cluster, int proc)
+{
+	gman_node_t* gman_node;
+
+	if ( attr_value && strlen(attr_value)==0 ) {
+		attr_value = NULL;
+	}
+
+	if ( (gman_node=lookupGmanByOwner(owner, attr_value, cluster, proc)) ) {
+		return gman_node->pid;
+	}
+	else {
+		return -1;
+	}
+}
+
 GridUniverseLogic::gman_node_t *
 GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 	   	const char* attr_value, const char* attr_name, int cluster, int proc)
@@ -394,6 +405,13 @@ GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 	gman_node_t* gman_node;
 	int pid;
 
+		// If attr_value is an empty string, convert to NULL since code
+		// after this point expects that.
+	if ( attr_value && strlen(attr_value)==0 ) {
+		attr_value = NULL;
+		attr_name = NULL;
+	}
+
 	if ( (gman_node=lookupGmanByOwner(owner, attr_value, cluster, proc)) ) {
 		// found it
 		return gman_node;
@@ -410,7 +428,7 @@ GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 
 
 #ifndef WIN32
-	if (owner && stricmp(owner, "root") == 0 ) {
+	if (owner && strcasecmp(owner, "root") == 0 ) {
 		dprintf(D_ALWAYS, "Tried to start condor_gmanager as root.\n");
 		return NULL;
 	}
@@ -443,12 +461,6 @@ GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 	}
 	free(gman_args);
 
-	if (cluster) {
-		dprintf( D_ALWAYS, "ERROR - gridman_per_job not supported!\n" );
-		free(gman_binary);
-		return NULL;
-	}
-
 	// build a constraint
 	if ( !owner ) {
 		dprintf(D_ALWAYS,"ERROR - missing owner field\n");
@@ -461,9 +473,10 @@ GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 						   ATTR_OWNER,owner,
 						   ATTR_JOB_UNIVERSE,CONDOR_UNIVERSE_GRID);
 	} else {
-		constraint.sprintf("(%s=?=\"%s\"&&%s=?=\"%s\")",
+		constraint.sprintf("(%s=?=\"%s\"&&%s=?=\"%s\"&&%s==%d)",
 						   ATTR_OWNER,owner,
-						   attr_name,attr_value);
+						   attr_name,attr_value,
+						   ATTR_JOB_UNIVERSE,CONDOR_UNIVERSE_GRID);
 	}
 	args.AppendArg("-C");
 	args.AppendArg(constraint.Value());
@@ -598,14 +611,10 @@ GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 	}
 	MyString owner_key(owner);
 	if(attr_value){
-		MyString attr_key(attr_value);
-		owner_key += attr_key;
+		owner_key += attr_value;
 	}
 	if (cluster) {
-		char tmp[100];
-		sprintf(tmp,"-%d.%d",cluster,proc);
-		MyString job_key(tmp);
-		owner_key += job_key;
+		owner_key.sprintf_cat( "-%d.%d", cluster, proc );
 	}
 
 	ASSERT( gman_pid_table->insert(owner_key,gman_node) == 0 );
@@ -613,7 +622,7 @@ GridUniverseLogic::StartOrFindGManager(const char* owner, const char* domain,
 	// start timer to signal gridmanager if we haven't already
 	if ( gman_node->add_timer_id == -1 ) {  // == -1 means no timer set
 		gman_node->add_timer_id = daemonCore->Register_Timer(job_added_delay,
-			(TimerHandler) &GridUniverseLogic::SendAddSignal,
+			GridUniverseLogic::SendAddSignal,
 			"GridUniverseLogic::SendAddSignal");
 		daemonCore->Register_DataPtr(gman_node);
 	}
