@@ -130,9 +130,11 @@ Job::abort(CondorError & /* errstack */)
 	while (declaredFiles.iterate(currentKey, jobFile)) {
 		close(jobFile.file);
 		declaredFiles.remove(currentKey);
+		MSC_SUPPRESS_WARNING_FIXME(6031);
 		remove(jobFile.name.Value());
 	}
 
+	MSC_SUPPRESS_WARNING_FIXME(6031);
 	remove(spoolDirectory.Value());
 
 	return 0;
@@ -229,7 +231,7 @@ Job::declare_file(const MyString &name,
 	jobFile.name = name;
 
 	jobFile.file =
-		safe_open_wrapper((spoolDirectory + DIR_DELIM_STRING + jobFile.name).Value(),
+		safe_open_wrapper_follow((spoolDirectory + DIR_DELIM_STRING + jobFile.name).Value(),
 			 O_WRONLY | O_CREAT | _O_BINARY,
 			 0600);
 	if (-1 != jobFile.file) {
@@ -458,7 +460,7 @@ Job::get_file(const MyString &name,
               unsigned char *&data,
 			  CondorError &errstack)
 {
-	int file = safe_open_wrapper((spoolDirectory + DIR_DELIM_STRING + name).Value(),
+	int file = safe_open_wrapper_follow((spoolDirectory + DIR_DELIM_STRING + name).Value(),
 					O_RDONLY | _O_BINARY,
 					0);
 
@@ -619,6 +621,27 @@ ScheddTransaction::commit()
 		// XXX: This will need to take a transaction
 	CommitTransaction();
 
+		// aboutToSpawnJobHandler() needs to be called before a job
+		// can potentially run. For most job types, it's called just
+		// before spawning the shadow or starter. For grid jobs, we
+		// call it at the end of job submission.
+		// If we don't call it at all, job files in the spool directory
+		// will be owned by the condor user, and we'll get permission
+		// errors when attempting to access them as the user.
+	PROC_ID currentKey;
+	Job *job;
+	ClassAd *job_ad;
+	int universe;
+	jobs.startIterations();
+	while (jobs.iterate(currentKey, job)) {
+		job_ad = GetJobAd( currentKey.cluster, currentKey.proc );
+		if ( job_ad->LookupInteger( ATTR_JOB_UNIVERSE, universe ) &&
+			 universe == CONDOR_UNIVERSE_GRID ) {
+
+			aboutToSpawnJobHandler( currentKey.cluster, currentKey.proc, NULL );
+		}
+	}
+
 	return 0;
 }
 
