@@ -30,6 +30,7 @@
 #include "basename.h"
 #include "nullfile.h"
 #include "spooled_job_files.h"
+#include "condor_url.h"
 
 /* In this service function, the client tells the schedd a bunch of jobs
 	it would like to perform a transfer for into/out of a sandbox. The
@@ -495,8 +496,8 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 	// Ok, figure out if I have a transferd already setup for this user.
 	td = m_tdman.find_td_by_user(fquser);
 	if (td == NULL || 
-		(td->get_status() != TD_REGISTERED) && 
-			(td->get_status() != TD_INVOKED)) 
+		((td->get_status() != TD_REGISTERED) && 
+			(td->get_status() != TD_INVOKED)) )
 	{
 		// Since it looks like I'm going to have to wait for a transferd
 		// to wake up and register to this schedd, let the client know we
@@ -505,6 +506,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 		if (respad.put(*rsock) == 0) {
 			dprintf(D_ALWAYS, "Submittor %s closed connection. Aborting "
 				"getting sandbox info for user.\n", fquser.Value());
+			delete treq;
 			return FALSE;
 		}
 		rsock->end_of_message();
@@ -563,6 +565,7 @@ Scheduler::requestSandboxLocation(int mode, Stream* s)
 	if (respad.put(*rsock) == 0) {
 		dprintf(D_ALWAYS, "Submittor %s closed connection. Aborting "
 			"getting sandbox info for user.\n", fquser.Value());
+		delete treq;
 		return FALSE;
 	}
 	rsock->end_of_message();
@@ -868,7 +871,9 @@ Scheduler::treq_upload_update_callback(TransferRequest *treq,
 			MyString new_path_buf;
 			while ( (old_path_buf=old_paths.next()) ) {
 				base = condor_basename(old_path_buf);
-				if ( strcmp(base,old_path_buf)!=0 ) {
+				if ((AttrsToModify[index] == ATTR_TRANSFER_INPUT_FILES) && IsUrl(old_path_buf)) {
+					base = old_path_buf;
+				} else if ( strcmp(base,old_path_buf)!=0 ) {
 					new_path_buf.sprintf(
 						"%s%c%s",SpoolSpace,DIR_DELIM_CHAR,base);
 					base = new_path_buf.Value();
