@@ -33,14 +33,15 @@
 
 package Condor;
 
-require 5.0;
-use Carp;
-use Cwd;
-use FileHandle;
-use POSIX "sys_wait_h";
 use strict;
 use warnings;
 
+use Carp;
+use File::Spec;
+use POSIX qw/sys_wait_h strftime/;
+
+use CondorUtils;
+
 my $CONDOR_SUBMIT = 'condor_submit';
 my $CONDOR_SUBMIT_DAG = 'condor_submit_dag';
 my $CONDOR_VACATE = 'condor_vacate';
@@ -77,26 +78,6 @@ my $JobErrCallback;
 my $TimedCallback;
 my $WantErrorCallback;
 
-BEGIN
-{
-    $CONDOR_SUBMIT = 'condor_submit';
-    $CONDOR_SUBMIT_DAG = 'condor_submit_dag';
-    $CONDOR_VACATE = 'condor_vacate';
-    $CONDOR_VACATE_JOB = 'condor_vacate_job';
-    $CONDOR_RESCHD = 'condor_reschedule';
-    $CONDOR_RM = 'condor_rm';
-
-    $DEBUG = 0;
-	$DEBUGLEVEL = 1; # turn on lowest level output
-    $cluster = 0;
-    $num_active_jobs = 0;
-    $saw_submit = 0;
-
-	$submit_time = 0;
-	$timer_time = 0;
-	$TimedCallbackWait = 0;
-
-}
 
 sub Reset
 {
@@ -565,33 +546,9 @@ sub Wait
     return $?;
 }
 
-sub IsAbsolutePath
-{
-	my $testpath = shift;
-	my $os = "$^O";
-	fullchomp($os);
-
-	#print "---$os---\n";
-
-	# 3 cases to consider
-	# / window or unix slash
-	# \ windows slash
-	# a: drive letter windows
-	if($os eq "MSWin32") # XP, WIN2k & server 2003 return this
-	{
-		if( $testpath =~ /^([a-zA-Z]:)?[\\\/].*/ )
-		{
-			return(1);
-		}
-	}
-	else # some Unix
-	{
-		if( $testpath =~ /^\\.*/ )
-		{
-			return(1);
-		}
-	}
-	return(0); #false
+sub IsAbsolutePath {
+    my ($path) = @_;
+    return File::Spec->file_name_is_absolute($path);
 }
 
 # spawn process to monitor the submit log file and execute callbacks
@@ -692,7 +649,7 @@ sub Monitor
 		}
 		$line = <SUBMIT_LOG>;
 	}
-	fullchomp($line);
+	CondorUtils::fullchomp($line);
 	$linenum++;
 
       PARSE:
@@ -703,7 +660,7 @@ sub Monitor
 	# if this line is for another cluster, ignore
 	if ( $line =~ /^\d+\s+\(0*(\d+)\./ && $1 != $cluster )
 	{
-	    debug( "log line for cluster $1, not $cluster -- ignoring...\n" ,6);
+	    debug( "log line for cluster $1, not $cluster -- ignoring...\n" ,1);
 	    next LINE;
 	}
 	
@@ -713,7 +670,7 @@ sub Monitor
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 
-	    debug( "Saw job evicted\n" ,5);
+	    debug( "Saw job ($1.$2) evicted\n" ,1);
 
 	    # execute callback if one is registered
 	    &$EvictedCallback( %info )
@@ -733,12 +690,12 @@ sub Monitor
 			}
 			$line = <SUBMIT_LOG>;
 		}
-	    fullchomp($line);
+	    CondorUtils::fullchomp($line);
 	    $linenum++;
 
 	    if( $line =~ /^\s+\(0\) Job was not checkpointed\./ )
 	    {
-		debug( "job was evicted without ckpt\n" ,5);
+		debug( "job was evicted without ckpt\n" ,1);
 		# execute callback if one is registered
 		&$EvictedWithoutCheckpointCallback( %info )
 		    if defined $EvictedWithoutCheckpointCallback;
@@ -746,14 +703,14 @@ sub Monitor
 	    }
 	    elsif( $line =~ /^\s+\(1\) Job was checkpointed\./ )
 	    {
-		debug( "job was evicted with ckpt\n" ,5);
+		debug( "job was evicted with ckpt\n" ,1);
 		# execute callback if one is registered
 		&$EvictedWithCheckpointCallback( %info )
 		    if defined $EvictedWithCheckpointCallback;
 	    }
 	    elsif( $line =~ /^\s+\(0\) Job terminated and was requeued.*$/ )
 	    {
-		debug( "job was evicted and requeued\n" ,5);
+		debug( "job was evicted and requeued\n" ,1);
 		# execute callback if one is registered
 		&$EvictedWithRequeueCallback( %info )
 		    if defined $EvictedWithRequeueCallback;
@@ -762,7 +719,7 @@ sub Monitor
 	    {
 		debug( "parse error on line $linenum of $info{'log'}:\n" .
 		       "   no checkpoint message found after eviction: " .
-		       "continuing...\n" ,5);
+		       "continuing...\n" ,1);
 		# re-parse line so we don't miss whatever it said
 		goto PARSE;
 	    }
@@ -775,7 +732,7 @@ sub Monitor
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 
-	    debug( "Saw job terminated\n" ,5);
+	    debug( "Saw job terminated\n" ,1);
 
 	    # decrement # of queued jobs so we will know when to exit monitor
 	    $num_active_jobs--;
@@ -795,7 +752,7 @@ sub Monitor
 			}
 			$line = <SUBMIT_LOG>;
 		}
-	    fullchomp($line);
+	    CondorUtils::fullchomp($line);
 	    $linenum++;
 
 	    # terminated successfully
@@ -817,11 +774,11 @@ sub Monitor
 	    # abnormal termination
 	    elsif( $line =~ /^\s+\(0\) Abnormal termination \(signal (\d+)\)/ )
 	    {
-		debug( "Loading $1 as info{'signal'}\n" ,5);
+		debug( "Loading $1 as info{'signal'}\n" ,1);
 		$info{'signal'} = $1;
 		#print "keys:".join(" ",keys %info)."\n";
 
-		debug( "checking for core file...\n" ,5);
+		debug( "checking for core file...\n" ,1);
 
 		# read next line to find core file
 		$line = <SUBMIT_LOG>;
@@ -834,7 +791,7 @@ sub Monitor
 			}
 			$line = <SUBMIT_LOG>;
 		}
-	    fullchomp($line);
+	    CondorUtils::fullchomp($line);
 		$linenum++;
 
 		if( $line =~ /^\s+\(1\) Corefile in: (.*)/ )
@@ -860,7 +817,7 @@ sub Monitor
 	    {
 		debug( "parse error on line $linenum of $info{'log'}:\n" .
 		       "   no termination status message found after " .
-		       "termination: continuing...\n" ,5);
+		       "termination: continuing...\n" ,1);
 		# re-parse line so we don't miss whatever it said
 		goto PARSE;
 	    }
@@ -873,7 +830,7 @@ sub Monitor
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 
-	    debug( "Saw Shadow Exception\n" ,5);
+	    debug( "Saw Shadow Exception\n" ,1);
 
 		if(! defined $ShadowCallback)
 		{
@@ -895,7 +852,7 @@ sub Monitor
 			}
 			$line = <SUBMIT_LOG>;
 		}
-	    fullchomp($line);
+	    CondorUtils::fullchomp($line);
 	    $linenum++;
 
 		$info{'shadowerror'} = $line;
@@ -909,14 +866,17 @@ sub Monitor
 
 	# 001: job executing
 	elsif( $line =~ 
-	       /^001\s+\(0*(\d+)\.0*(\d+).*<([^>]+)>/ )
+	       /^001\s+\(0*(\d+)\.0*(\d+).*: (.+)/ )
 	{
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
+	    $info{'sinful'} = $3;
 	    $info{'host'} = $3;
-	    $info{'sinful'} = "<$3>";
+	    # Strip off the < and > from a sinful string for 'host'
+	    # For grid universe jobs, there won't be any
+	    $info{'host'} =~ s/^<([^>]+)>/$1/;
 	    
-	    debug( "Saw job executing\n" ,5);
+	    debug( "Saw job executing\n" ,1);
 
 	    # execute callback if one is registered
 	    &$ExecuteCallback( %info )
@@ -934,7 +894,7 @@ sub Monitor
 	    $info{'host'} = $3;
 	    $info{'sinful'} = "<$3>";
 
-	    debug( "Saw job submitted\n" ,5);
+	    debug( "Saw job submitted\n" ,1);
 	    $submit_info{'cluster'} = $1; # squirrel it away for TimedWait
 
 	    # mark that we've seen a submit so we can start watching # of jobs
@@ -955,7 +915,7 @@ sub Monitor
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 
-	    debug( "Saw job abort cluster $1 job $2\n" ,5);
+	    debug( "Saw job abort cluster $1 job $2\n" ,1);
 
 	    # decrement # of queued jobs so we will know when to exit monitor
 	    $num_active_jobs--;
@@ -984,12 +944,12 @@ sub Monitor
 			}
 			$line = <SUBMIT_LOG>;
 		}
-	    fullchomp($line);
+	    CondorUtils::fullchomp($line);
 	    $linenum++;
 
 		$info{'holdreason'} = $line;
 
-	    debug( "Saw job held\n" ,5);
+	    debug( "Saw job held\n" ,1);
 
 	    
 	    # execute callback if one is registered
@@ -1005,7 +965,7 @@ sub Monitor
 	    #$info{'host'} = $3;
 	    #$info{'sinful'} = "<$3:$4>";
 	    
-	    debug( "Saw job released\n" ,5);
+	    debug( "Saw job released\n" ,1);
 
 	    
 	    # execute callback if one is registered
@@ -1061,30 +1021,27 @@ sub CheckTimedCallback
 #
 ################################################################################
 
-sub debug
-{
-    my $string = shift;
-	my $level = shift;
-	if(!(defined $level)) {
-    	print( "", timestamp(), ": $string" ) if $DEBUG;
-	} elsif($level <= $DEBUGLEVEL) {
-    	print( "", timestamp(), ": $string" ) if $DEBUG;
-	}
+sub debug {
+    return unless $DEBUG;
+    my ($msg, $level) = @_;
+    
+    if(!(defined $level)) {
+    	print timestamp() . ": $msg";
+    }
+    elsif($level <= $DEBUGLEVEL) {
+    	print timestamp() . ": $msg";
+    }
 }
 
-sub DebugLevel
-{
-	my $newlevel = shift;
-	$DEBUGLEVEL = $newlevel;
+sub DebugLevel {
+    $DEBUGLEVEL = shift;
 }
 
-sub DebugOn
-{
+sub DebugOn {
     $DEBUG = 1;
 }
 
-sub DebugOff
-{
+sub DebugOff {
     $DEBUG = 0;
 }
 
@@ -1104,7 +1061,7 @@ sub ParseSubmitFile
     debug( "reading submit file...\n" ,5);
     while( <SUBMIT_FILE> )
     {
-	fullchomp($_);	# do a windows strip, fine for linux
+	CondorUtils::fullchomp($_);	# do a windows strip, fine for linux
 	$line++;
 
 	# skip comments & blank lines
@@ -1130,7 +1087,7 @@ sub ParseSubmitFile
 	    # compress whitespace and remove trailing newline for readability
 		# Don't change white space. Some environment tests expect tabs to stay tabs.
 	    # $value =~ s/\s+/ /g;
-	    fullchomp($value);
+	    CondorUtils::fullchomp($value);
 
 	
 		# Do proper environment substitution
@@ -1156,7 +1113,7 @@ sub ParseSubmitFile
 }
 
 sub timestamp {
-    return scalar localtime();
+    return strftime("%Y/%m/%d %H:%M:%S", localtime);
 }
 
 sub safe_WIFEXITED {
@@ -1189,21 +1146,5 @@ sub safe_WEXITSTATUS {
 	}
 }
 
-#
-# Cygwin's perl chomp does not remove cntrl-m but this one will
-# and linux and windows can share the same code. The real chomp
-# totals the number or changes but I currently return the modified
-# array. bt 10/06
-#
-
-sub fullchomp
-{
-	push (@_,$_) if( scalar(@_) == 0);
-	foreach my $arg (@_) {
-		$arg =~ s/\012+$//;
-		$arg =~ s/\015+$//;
-	}
-	return(0);
-}
 
 1;
