@@ -59,6 +59,8 @@ sub new( $$ )
     $self->{ConfigProg} = $Distribution . "_config_val";
     $self->SetFromConfig( "ARCH" );
     $self->SetFromConfig( "OPSYS" );
+    $self->SetFromConfig( "OPSYSVER" );
+    $self->SetFromConfig( "OPSYS_AND_VER" );
     $self->SetFromConfig( "FILESYSTEM_DOMAIN" );
     $self->SetFromConfig( "UID_DOMAIN" );
     $self->SetFromConfig( "IP_ADDRESS" );
