@@ -1072,6 +1072,16 @@ sub TunePersonalCondor
 	    print NEW "# Done appending from 'append_condor_config'\n";
 	}
 
+	# Gittrac Ticket 2889
+	# This is ok when tests are running as a slot user but if/when we start running tests as root we will
+	# need to put these into a directory with permissions 1777.
+	print NEW "# Relocate C_GAHP files to prevent collision from multiple tests (running on multiple slots) writing into /tmp\n";
+	print NEW "# If we are running tests as root we might need to relocate these to a directory with permissions 1777\n";
+	print NEW "C_GAHP_LOG = \$(LOG)/CGAHPLog.\$(USERNAME)\n";
+	print NEW "C_GAHP_LOCK = \$(LOCK)/CGAHPLock.\$(USERNAME)\n";
+	print NEW "C_GAHP_WORKER_THREAD_LOG = \$(LOG)/CGAHPWorkerLog.\$(USERNAME)\n";
+	print NEW "C_GAHP_WORKER_THREAD_LOCK = \$(LOCK)/CGAHPWorkerLock.\$(USERNAME)\n";
+
 	close(NEW);
 
 	PostTunePersonalCondor($personal_config_file);
