@@ -73,6 +73,7 @@ BaseShadow::BaseShadow() {
 	m_lazy_queue_update = true;
 	m_cleanup_retry_tid = -1;
 	m_cleanup_retry_delay = 30;
+	m_RunAsNobody = false;
 }
 
 BaseShadow::~BaseShadow() {
@@ -155,17 +156,36 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 	
 	config();
 
-	initUserLog();
-
 		// Make sure we've got enough swap space to run
 	checkSwap();
 
 	// handle system calls with Owner's privilege
 // XXX this belong here?  We'll see...
 	if ( !init_user_ids(owner.Value(), domain.Value())) {
-		dprintf(D_ALWAYS, "init_user_ids() failed!\n");
+		dprintf(D_ALWAYS, "init_user_ids() failed as user %s\n",owner.Value() );
 		// uids.C will EXCEPT when we set_user_priv() now
 		// so there's not much we can do at this point
+		
+#if ! defined(WIN32)
+		if ( param_boolean( "SHADOW_RUN_UNKNOWN_USER_JOBS", false ) )
+		{
+			dprintf(D_ALWAYS, "trying init_user_ids() as user nobody\n" );
+			
+			owner="nobody";
+			domain=NULL;
+			if (!init_user_ids(owner.Value(), domain.Value()))
+			{
+				dprintf(D_ALWAYS, "init_user_ids() failed!\n");
+			}
+			else
+			{
+				jobAd->Assign( ATTR_JOB_RUNAS_OWNER, "FALSE" );
+				m_RunAsNobody=true;
+				dprintf(D_ALWAYS, "init_user_ids() now running as user nobody\n");
+			}
+		}
+#endif
+
 	}
 	set_user_priv();
 	daemonCore->Register_Priv_State( PRIV_USER );
@@ -188,6 +208,12 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 		job_updater = new NullQmgrJobUpdater( jobAd, scheddAddr, CondorVersion() );
 	}
 
+		// init user log; hold on failure
+		// NOTE: job_updater must be initialized _before_ initUserLog(),
+		// in order to handle the case of the job going on hold as a
+		// result of failure in initUserLog().
+	initUserLog();
+
 		// change directory; hold on failure
 	if ( cdToIwd() == -1 ) {
 		EXCEPT("Could not cd to initial working directory");
@@ -287,6 +313,15 @@ void BaseShadow::config()
 
 
 int BaseShadow::cdToIwd() {
+	int iRet =0;
+	
+#if ! defined(WIN32)
+	priv_state p = PRIV_UNKNOWN;
+	
+	if (m_RunAsNobody)
+		p = set_root_priv();
+#endif
+	
 	if (chdir(iwd.Value()) < 0) {
 		int chdir_errno = errno;
 		dprintf(D_ALWAYS, "\n\nPath does not exist.\n"
@@ -296,10 +331,16 @@ int BaseShadow::cdToIwd() {
 		hold_reason.sprintf("Cannot access initial working directory %s: %s",
 		                    iwd.Value(), strerror(chdir_errno));
 		dprintf( D_ALWAYS, "%s\n",hold_reason.Value());
-		holdJob(hold_reason.Value(),CONDOR_HOLD_CODE_IwdError,chdir_errno);
-		return -1;
+		holdJobAndExit(hold_reason.Value(),CONDOR_HOLD_CODE_IwdError,chdir_errno);
+		iRet = -1;
 	}
-	return 0;
+	
+#if ! defined(WIN32)
+	if ( m_RunAsNobody )
+		set_priv(p);
+#endif
+	
+	return iRet;
 }
 
 
@@ -320,7 +361,10 @@ BaseShadow::shutDown( int reason )
 		// Only if the job is trying to leave the queue should we
 		// evaluate the user job policy...
 	if( reason == JOB_EXITED || reason == JOB_COREDUMPED ) {
-		shadow_user_policy.checkAtExit();
+		if( !waitingToUpdateSchedd() ) {
+			shadow_user_policy.checkAtExit();
+				// WARNING: 'this' may have been deleted by the time we get here!!!
+		}
 	}
 	else {
 		// if we aren't trying to evaluate the user's policy, we just
@@ -387,7 +431,14 @@ BaseShadow::holdJob( const char* reason, int hold_reason_code, int hold_reason_s
 		dprintf( D_ALWAYS, "Failed to update job queue!\n" );
 	}
 
-		// finally, exit and tell the schedd what to do
+}
+
+void
+BaseShadow::holdJobAndExit( const char* reason, int hold_reason_code, int hold_reason_subcode )
+{
+	holdJob(reason,hold_reason_code,hold_reason_subcode);
+
+	// finally, exit and tell the schedd what to do
 	DC_Exit( JOB_SHOULD_HOLD );
 }
 
@@ -437,8 +488,7 @@ BaseShadow::mockTerminateJob( MyString exit_reason,
 }
 
 
-void
-BaseShadow::removeJob( const char* reason )
+void BaseShadow::removeJobPre( const char* reason )
 {
 	if( ! jobAd ) {
 		dprintf( D_ALWAYS, "In removeJob() w/ NULL JobAd!" );
@@ -446,10 +496,10 @@ BaseShadow::removeJob( const char* reason )
 	dprintf( D_ALWAYS, "Job %d.%d is being removed: %s\n", 
 			 getCluster(), getProc(), reason );
 
-		// cleanup this shadow (kill starters, etc)
+	// cleanup this shadow (kill starters, etc)
 	cleanUp();
 
-		// Put the reason in our job ad.
+	// Put the reason in our job ad.
 	int size = strlen( reason ) + strlen( ATTR_REMOVE_REASON ) + 4;
 	char* buf = (char*)malloc( size * sizeof(char) );
 	if( ! buf ) {
@@ -461,15 +511,20 @@ BaseShadow::removeJob( const char* reason )
 
 	emailRemoveEvent( reason );
 
-		// update the job ad in the queue with some important final
-		// attributes so we know what happened to the job when using
-		// condor_history...
+	// update the job ad in the queue with some important final
+	// attributes so we know what happened to the job when using
+	// condor_history...
 	if( !updateJobInQueue(U_REMOVE) ) {
 			// trouble!  TODO: should we do anything else?
 		dprintf( D_ALWAYS, "Failed to update job queue!\n" );
 	}
+}
 
-		// does not return.
+void BaseShadow::removeJob( const char* reason )
+{
+	this->removeJobPre(reason);
+	
+	// does not return.
 	DC_Exit( JOB_SHOULD_REMOVE );
 }
 
@@ -614,6 +669,13 @@ BaseShadow::terminateJob( update_style_t kind ) // has a default argument of US_
 	// update the job ad in the queue with some important final
 	// attributes so we know what happened to the job when using
 	// condor_history...
+    if (m_num_cleanup_retries < 1 &&
+        param_boolean("SHADOW_TEST_JOB_CLEANUP_RETRY", false)) {
+		dprintf( D_ALWAYS,
+				 "Testing Failure to perform final update to job queue!\n");
+		retryJobCleanup();
+		return;
+    }
 	if( !updateJobInQueue(U_TERMINATE) ) {
 		dprintf( D_ALWAYS, 
 				 "Failed to perform final update to job queue!\n");
@@ -771,9 +833,28 @@ void BaseShadow::initUserLog()
 	int  use_xml;
 	bool result;
 
+		// we expect job_updater to already be initialized, in case we
+		// need to put the job on hold as a result of failure to open
+		// the log
+	ASSERT( job_updater );
+
 	if ( getPathToUserLog(jobAd, logfilename) ) {
 		result = uLog.initialize (owner.Value(), domain.Value(), logfilename.Value(), cluster, proc, 0, gjid);
+		// It is important to NOT ignore a failure to initialize the user log,
+		// since if we fail to initialize here, then all event logging 
+		// in the shadow from this point forward are effectively ignored.
+		// So if we fail to initialize the user log, put this job on hold.
+		// Future work: it would be good to pass use the error stack to 
+		// figure out -why- the initialization failed, allowing the shadow
+		// to retry automatically -vs- go on hold depending upon the details
+		// of the failure.
 		if ( result == false ) {
+			MyString hold_reason;
+			hold_reason.sprintf(
+				"Failed to initialize user log to %s", logfilename.Value());
+			dprintf( D_ALWAYS, "%s\n",hold_reason.Value());
+			holdJobAndExit(hold_reason.Value(),CONDOR_HOLD_CODE_UnableToInitUserLog,0);
+			// holdJobAndExit() should not return, but just in case it does EXCEPT
 			EXCEPT("Failed to initialize user log to %s",logfilename.Value());
 		}
 		if (jobAd->LookupBool(ATTR_ULOG_USE_XML, use_xml)
@@ -819,6 +900,44 @@ int getJobAdExitSignal(ClassAd *jad, int &exit_signal)
 	return TRUE;
 }
 
+static void set_usageAd (ClassAd* jobAd, ClassAd ** ppusageAd) 
+{
+	std::string resslist;
+	if ( ! jobAd->LookupString("PartitionableResources", resslist))
+		resslist = "Cpus, Disk, Memory";
+
+	StringList reslist(resslist.c_str());
+	if (reslist.number() > 0) {
+		int64_t int64_value = 0;
+		ClassAd * puAd = new ClassAd();
+		puAd->Clear(); // get rid of default "CurrentTime = time()" value.
+
+		reslist.rewind();
+		char * resname = NULL;
+		while ((resname = reslist.next()) != NULL) {
+			MyString attr;
+			int64_value = -1;
+			attr.sprintf("%s", resname); // provisioned value
+			if (jobAd->LookupInteger(attr.Value(), int64_value)) {
+				puAd->Assign(resname, int64_value);
+			} 
+			// /*for debugging*/ else { puAd->Assign(resname, 42); }
+			int64_value = -2;
+			attr.sprintf("Request%s", resname);	// requested value
+			if (jobAd->LookupInteger(attr.Value(), int64_value)) {
+				puAd->Assign(attr.Value(), int64_value);
+			}
+			// /*for debugging*/ else { puAd->Assign(attr.Value(), 99); }
+			int64_value = -3;
+			attr.sprintf("%sUsage", resname); // usage value
+			if (jobAd->LookupInteger(attr.Value(), int64_value)) {
+				puAd->Assign(attr.Value(), int64_value);
+			}
+		}
+		*ppusageAd = puAd;
+	}
+}
+
 // kind defaults to US_NORMAL.
 void
 BaseShadow::logTerminateEvent( int exitReason, update_style_t kind )
@@ -927,6 +1046,45 @@ BaseShadow::logTerminateEvent( int exitReason, update_style_t kind )
 	if( exitReason == JOB_COREDUMPED ) {
 		event.setCoreFile( core_file_name );
 	}
+
+#if 1
+	set_usageAd(jobAd, &event.pusageAd);
+#else
+	std::string resslist;
+	if ( ! jobAd->LookupString("PartitionableResources", resslist))
+		resslist = "Cpus, Disk, Memory";
+
+	StringList reslist(resslist.c_str());
+	if (reslist.number() > 0) {
+		int64_t int64_value = 0;
+		ClassAd * puAd = new ClassAd();
+		puAd->Clear(); // get rid of default "CurrentTime = time()" value.
+
+		reslist.rewind();
+		char * resname = NULL;
+		while ((resname = reslist.next()) != NULL) {
+			MyString attr;
+			int64_value = -1;
+			attr.sprintf("%s", resname); // provisioned value
+			if (jobAd->LookupInteger(attr.Value(), int64_value)) {
+				puAd->Assign(resname, int64_value);
+			} 
+			// /*for debugging*/ else { puAd->Assign(resname, 42); }
+			int64_value = -2;
+			attr.sprintf("Request%s", resname);	// requested value
+			if (jobAd->LookupInteger(attr.Value(), int64_value)) {
+				puAd->Assign(attr.Value(), int64_value);
+			}
+			// /*for debugging*/ else { puAd->Assign(attr.Value(), 99); }
+			int64_value = -3;
+			attr.sprintf("%sUsage", resname); // usage value
+			if (jobAd->LookupInteger(attr.Value(), int64_value)) {
+				puAd->Assign(attr.Value(), int64_value);
+			}
+		}
+		event.pusageAd = puAd;
+	}
+#endif
 	
 	if (!uLog.writeEvent (&event,jobAd)) {
 		dprintf (D_ALWAYS,"Unable to log "
@@ -965,6 +1123,8 @@ BaseShadow::logEvictEvent( int exitReason )
 		// remote rusage
 	event.run_remote_rusage = run_remote_rusage;
 	
+	set_usageAd(jobAd, &event.pusageAd);
+
 		/*
 		  we want to log the events from the perspective of the user
 		  job, so if the shadow *sent* the bytes, then that means the
@@ -1062,7 +1222,7 @@ BaseShadow::log_except(const char *msg)
 	bool exception_already_logged = false;
 
 	if(!msg) msg = "";
-	sprintf(event.message, msg);
+	sprintf(event.message, "%s", msg);
 
 	if ( BaseShadow::myshadow_ptr ) {
 		BaseShadow *shadow = BaseShadow::myshadow_ptr;
@@ -1121,6 +1281,8 @@ BaseShadow::updateJobInQueue( update_t type )
 											   bytesSent()) );
 	jobAd->Insert( buf.Value() );
 
+	ASSERT( job_updater );
+
 		// Now that the ad is current, just let our QmgrJobUpdater
 		// object take care of the rest...
 	return job_updater->updateJob( type );
@@ -1316,3 +1478,14 @@ BaseShadow::handleUpdateJobAd( int sig )
 	shadow_user_policy.checkPeriodic();
 	return 0;
 }
+
+bool
+BaseShadow::jobWantsGracefulRemoval()
+{
+	bool job_wants_graceful_removal = false;
+	ClassAd *thejobAd = getJobAd();
+	if( thejobAd ) {
+		thejobAd->LookupBool( ATTR_WANT_GRACEFUL_REMOVAL, job_wants_graceful_removal );
+	}
+	return job_wants_graceful_removal;
+}
