@@ -104,8 +104,6 @@ ParallelShadow::init( ClassAd* job_ad, const char* schedd_addr, const char *xfer
 
 	rr->setStartdInfo( job_ad );
 
-	job_ad->Assign( ATTR_JOB_STATUS, RUNNING );
-
     ResourceList[ResourceList.getlast()+1] = rr;
 
 	shutdownPolicy = ParallelShadow::WAIT_FOR_NODE0;
@@ -325,6 +323,19 @@ ParallelShadow::getResources( void )
             host = NULL;
             claim_id = NULL;
 
+				/* Double check that the number of matches for this proc
+				   is the same as in the job ad.  This prevents races
+				   where a claim can be vacated after the schedd forks this
+					shadow 
+				*/
+			
+
+			int jobAdNumInProc = 0;
+			if (!job_ad->LookupInteger(ATTR_MAX_HOSTS, jobAdNumInProc)) {
+				dprintf(D_ALWAYS, "ERROR: no attribute MaxHosts in parallel job\n");	
+				return false;
+			};
+			ASSERT( jobAdNumInProc == numInProc);
         } // end of for loop for this proc
 
     } // end of for loop on all procs...
@@ -402,7 +413,7 @@ ParallelShadow::spawnNode( MpiResource* rr )
 void 
 ParallelShadow::cleanUp( void )
 {
-		// kill all the starters
+	// kill all the starters
 	MpiResource *r;
 	int i;
     for( i=0 ; i<=ResourceList.getlast() ; i++ ) {
@@ -411,6 +422,42 @@ ParallelShadow::cleanUp( void )
 	}		
 }
 
+int ParallelShadow::JobSuspend(int sig)
+{
+	int iRet = 0;
+	MpiResource *r;
+	int i;
+    for( i=0 ; i<=ResourceList.getlast() ; i++ ) {
+		r = ResourceList[i];
+		if (!r || !r->suspend())
+		{
+			iRet = 1;
+			dprintf ( D_ALWAYS, "ParallelShadow::JobSuspend() sig %d FAILED\n", sig );
+		}
+	}		
+	
+	return iRet;
+	
+}
+
+int ParallelShadow::JobResume(int sig)
+{
+	int iRet = 0;
+	MpiResource *r;
+	int i;
+    for( i=0 ; i<=ResourceList.getlast() ; i++ ) {
+		r = ResourceList[i];
+		if (!r || !r->resume())
+		{
+			iRet = 1;
+			dprintf ( D_ALWAYS, "ParallelShadow::JobResume() sig %d FAILED\n", sig );
+		}
+	}
+	
+	return iRet;
+}
+
+
 bool
 ParallelShadow::claimIsClosing( void )
 {
@@ -790,19 +837,24 @@ ParallelShadow::getRUsage( void )
 }
 
 
-int
-ParallelShadow::getImageSize( void )
+int64_t
+ParallelShadow::getImageSize( int64_t & memory_usage, int64_t & rss, int64_t & pss )
 {
 	MpiResource* mpi_res;
-	int i, max = 0, val;
-	for( i=0; i<=ResourceList.getlast() ; i++ ) {
+	int64_t max_size = 0, max_usage = 0, max_rss = 0, max_pss = -1;
+	for(int i=0; i<=ResourceList.getlast() ; i++ ) {
 		mpi_res = ResourceList[i];
-		val = mpi_res->getImageSize();
-		if( val > max ) {
-			max = val;
-		}
+		int64_t usage = 0, rs = 0, ps = 0;
+		int64_t val = mpi_res->getImageSize( usage, rs, ps );
+		max_size = MAX(val, max_size);
+		max_usage = MAX(usage, max_usage);
+		max_rss = MAX(rs, max_rss);
+		max_pss = MAX(ps, max_pss);
 	}
-	return max;
+	rss = max_rss;
+	pss = max_pss;
+	memory_usage = max_usage;
+	return max_size;
 }
 
 
@@ -912,7 +964,7 @@ ParallelShadow::resourceBeganExecution( RemoteResource* rr )
 			// All nodes in this computation are now running, so we 
 			// can finally log the execute event.
 		ExecuteEvent event;
-		strcpy( event.executeHost, "MPI_job" );
+		event.setExecuteHost( "MPI_job" );
 		if ( !uLog.writeEvent( &event, jobAd )) {
 			dprintf ( D_ALWAYS, "Unable to log EXECUTE event." );
 		}
