@@ -899,14 +899,13 @@ part_send_job(
 	  // communication.
   char sinfulstring[SINFUL_STRING_BUF_SIZE];
 
-  snprintf(sinfulstring, SINFUL_STRING_BUF_SIZE, "<%s:%d>", sock->peer_ip_str(), ports.port1);
-
+  generate_sinful(sinfulstring, SINFUL_STRING_BUF_SIZE, sock->peer_ip_str(), ports.port1);
   if( (sd1 = do_connect(sinfulstring, (char *)0, (u_short)ports.port1)) < 0 ) {
     dprintf( D_ALWAYS, "failed to connect to scheduler on %s\n", sinfulstring );
 	goto returnfailure;
   }
- 
-  snprintf(sinfulstring, SINFUL_STRING_BUF_SIZE, "<%s:%d>", sock->peer_ip_str(), ports.port2);
+
+  generate_sinful(sinfulstring, SINFUL_STRING_BUF_SIZE, sock->peer_ip_str(), ports.port2);
   if( (sd2 = do_connect(sinfulstring, (char *)0, (u_short)ports.port2)) < 0 ) {
     dprintf( D_ALWAYS, "failed to connect to scheduler on %s\n", sinfulstring );
 	close(sd1);
@@ -976,7 +975,6 @@ MakeProc(ClassAd *ad, PROC *p)
 {
 	char buf[ATTRLIST_MAX_EXPRESSION];
 	float	utime,stime;
-	char	*s;
 	ExprTree *e;
 	
 	p->version_num = 3;
